#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Script for deprovisioning network interfaces
"""

import sys
import os
import re
import traceback
import errno
import time

import vi_utils
import vi_os
import vi_xml
import storage_tech

from optparse import OptionParser
from optparse import OptionGroup

####################################################################################################

global_logDirectory		= '/var/log/deprovision_network'
global_baseLogFileName	= 'deprovision'

global_errAndLog		= None		# VI_ErrAndLog_OBJ
global_logFilePath		= ''

####################################################################################################

class Affected_Bridge_OBJ:

	def __init__( self, bridgeName, vifName, interface ):
		self.mBridgeName	= bridgeName
		self.mVIFname		= vifName
		self.mInterface		= interface

	def __repr__(self):
		return "Brg: %s VIF: %s Iface: %s" % (
				self.mBridgeName
			,	self.mVIFname
			,	self.mInterface )

class Affected_OpenISCSIsession_OBJ:

	def __init__( self, openISCSIstate, affectedRoute_LIST, unaffectedRoute_LIST ):
		self.mOpenISCSIstate		= openISCSIstate
		self.mAffectedRoute_LIST	= affectedRoute_LIST
		self.mUnaffectedRoute_LIST	= unaffectedRoute_LIST

	def __repr__(self):
		return "State: %s Af: %s UNaf: %s" % (
				self.mOpenISCSIstate
			,	self.mAffectedRoute_LIST
			,	self.mUnaffectedRoute_LIST )

####################################################################################################
def get_NIC_dict( vi_errorAndLog_obj, options_OBJ, network_LIST ):
	"""
	This routine
	"""
#
	nic_DICT				= {}
	unknown_network_LIST	= []

	interface_DICT	= vi_os.vi_get_NIC_dict( vi_errorAndLog_obj, True, True )

	for network in network_LIST:
		if not network.startswith( 'eth' ) and not network.startswith( 'bond' ):
			unknown_network_LIST.append( network )
			continue
		for interface, nic_state in interface_DICT.iteritems():
			if network == interface:
				nic_DICT[ interface ]	= nic_state
				break
			if network == nic_state.mMAC:
				nic_DICT[ interface ]	= nic_state
				break
		else:
			unknown_network_LIST.append( network )

	return nic_DICT, unknown_network_LIST

####################################################################################################
def get_affected_bridges( vi_errorAndLog_obj, options_OBJ, nic_DICT ):
	"""
	This routine
	"""
#
	affectedBridge_LIST	= []

	for interface in nic_DICT.keys():

		networkBridgeInfoDirPath	= "/sys/class/net/%s/brport/bridge/brif/" % ( interface )
		if not os.path.exists( networkBridgeInfoDirPath ):	# No bridges
			continue

		for vifName in os.listdir( networkBridgeInfoDirPath ):
			if not vifName.startswith( 'vif' ):
				continue
			vifBridgeLink		= "%s/%s/bridge" % ( networkBridgeInfoDirPath, vifName )
			bridgeRelativePath	= os.readlink( vifBridgeLink )
			bridgeName			= bridgeRelativePath[ bridgeRelativePath.rindex( '/' ) + 1 : ]
			affectedBridge		= Affected_Bridge_OBJ( bridgeName, vifName, interface )
			affectedBridge_LIST.append( affectedBridge )

	return affectedBridge_LIST

####################################################################################################
def get_affected_routes( vi_errorAndLog_obj, options_OBJ, nic_DICT ):
	"""
	This routine
	"""
#
	affectedRoute_LIST	= []

	route_LIST	= vi_os.vi_get_route_list( vi_errorAndLog_obj, False )
	for route in route_LIST:
		nic_state	= nic_DICT.get( route.mInterface, None )
		if nic_state:
			affectedRoute_LIST.append( route )

	return affectedRoute_LIST

####################################################################################################
def get_affected_openiscsi_sessions( vi_errorAndLog_obj, options_OBJ, nic_DICT ):
	"""
	This routine
	"""
#
	affectedOpenISCSIsession_LIST	= []

	openiscsi_session_info_DICT	= storage_tech.get_openiscsi_session_info_dict( vi_errorAndLog_obj, False )

	for sessionNum, openISCSIstate in openiscsi_session_info_DICT.iteritems():
		affectedRoute_LIST		= []
		unaffectedRoute_LIST	= []
		ipRoute_LIST			= vi_os.vi_get_route_list_for_IP( vi_errorAndLog_obj, openISCSIstate.mTargetIP, True, False )
		for route in ipRoute_LIST:
			nic_state	= nic_DICT.get( route.mInterface, None )
			if not nic_state:
				unaffectedRoute_LIST.append( route )
				continue
			affectedRoute_LIST.append( route )
		if len(affectedRoute_LIST) < 1:
			continue

		affectedOpenISCSIsession	= Affected_OpenISCSIsession_OBJ( openISCSIstate, affectedRoute_LIST, unaffectedRoute_LIST )
		affectedOpenISCSIsession_LIST.append( affectedOpenISCSIsession )

	return affectedOpenISCSIsession_LIST

####################################################################################################
def get_affected_RSRCs( vi_errorAndLog_obj, options_OBJ, nic_DICT ):
	"""
	This routine
	"""
#
	affectedBridge_LIST				= get_affected_bridges( vi_errorAndLog_obj, options_OBJ, nic_DICT )
	affectedRoute_LIST				= get_affected_routes( vi_errorAndLog_obj, options_OBJ, nic_DICT )
	affectedOpenISCSIsession_LIST	= get_affected_openiscsi_sessions( vi_errorAndLog_obj, options_OBJ, nic_DICT )

	defaultRouteAffected	= None
	for route in affectedRoute_LIST:
		if route.mIPv4_netmask == 0:
			defaultRouteAffected	= route
			break

	if options_OBJ.ignoreIfAlternateRouteExists:
		unaffectedOpenISCSIsessionIdx_LIST	= []
		for idx, affectedOpenISCSIsession in enumerate(affectedOpenISCSIsession_LIST):
			if len(affectedOpenISCSIsession.mUnaffectedRoute_LIST) > 0:
				unaffectedOpenISCSIsessionIdx_LIST.append( idx )
		for idx, in reversed(unaffectedOpenISCSIsessionIdx_LIST):
			del affectedOpenISCSIsession_LIST[idx]

	affectedBlkDev_LIST				= []
	for affectedOpenISCSIsession in affectedOpenISCSIsession_LIST:
		affectedBlkDev_LIST.extend( affectedOpenISCSIsession.mOpenISCSIstate.mBlkDev_LIST )
	vi_utils.vi_sort_uniq( affectedBlkDev_LIST, False )

	blk_dev_to_dm_map_lookup_DICT	= storage_tech.get_blk_dev_to_dm_map_lookup_dict( vi_errorAndLog_obj, False )
	affectedWWID_LIST				= []
	for blkdev in affectedBlkDev_LIST:
		wwid						= blk_dev_to_dm_map_lookup_DICT.get( blkdev, None )
		affectedWWID_LIST.append( wwid )
	vi_utils.vi_sort_uniq( affectedWWID_LIST, False )

	return affectedBridge_LIST, affectedRoute_LIST, defaultRouteAffected, affectedOpenISCSIsession_LIST, affectedBlkDev_LIST, affectedWWID_LIST

####################################################################################################
def list_affected_RSRCs( vi_errorAndLog_obj, options_OBJ, nic_DICT, domDocHelper ):
	"""
	This routine
	"""
#
	affectedBridge_LIST, affectedRoute_LIST, defaultRouteAffected, affectedOpenISCSIsession_LIST, affectedBlkDev_LIST, affectedWWID_LIST	= get_affected_RSRCs( vi_errorAndLog_obj, options_OBJ, nic_DICT )

	affectedBridgeName_LIST		= []
	for affectedBridge in affectedBridge_LIST:
		affectedBridgeName_LIST.append( affectedBridge.mBridgeName )
	affectedBridgeName_LIST.sort();

	affectedIQN_LIST			= []
	for affectedOpenISCSIsession in affectedOpenISCSIsession_LIST:
		affectedIQN_LIST.append( affectedOpenISCSIsession.mOpenISCSIstate.mIQN )
	vi_utils.vi_sort_uniq( affectedIQN_LIST, False )

	if domDocHelper == None:
		vi_errorAndLog_obj.vi_print_std( '    "AffectedBridges" : %s' % ( affectedBridgeName_LIST ) )
		vi_errorAndLog_obj.vi_print_std( '    "AffectedWWIDs"   : %s' % ( affectedWWID_LIST ) )
		vi_errorAndLog_obj.vi_print_std( '    "AffectedIQNs"    : %s' % ( affectedIQN_LIST ) )
		if defaultRouteAffected:
			vi_errorAndLog_obj.vi_print_std( '    "DefaultRoute"    : %s' % ( defaultRouteAffected.mInterface ) )
	else:
		if len(affectedBridgeName_LIST) > 0:
			domDocHelper.push_level( 'AffectedBridges', { 'Num' : str(len(affectedBridgeName_LIST)) } )
			for affectedBridgeName in affectedBridgeName_LIST:
				domDocHelper.add_data_elem( 'AffectedBridge', affectedBridgeName )
			domDocHelper.pop_level()
		if len(affectedWWID_LIST) > 0:
			domDocHelper.push_level( 'AffectedWWIDs', { 'Num' : str(len(affectedWWID_LIST)) } )
			for affectedWWID in affectedWWID_LIST:
				domDocHelper.add_data_elem( 'AffectedWWID', affectedWWID )
			domDocHelper.pop_level()
		if len(affectedIQN_LIST) > 0:
			domDocHelper.push_level( 'AffectedIQNs', { 'Num' : str(len(affectedIQN_LIST)) } )
			for affectedIQN in affectedIQN_LIST:
				domDocHelper.add_data_elem( 'AffectedIQN', affectedIQN )
			domDocHelper.pop_level()
		if defaultRouteAffected:
			domDocHelper.add_data_elem( 'DefaultRoute', defaultRouteAffected.mInterface )

	return 0

####################################################################################################
def do_deprovision( vi_errorAndLog_obj, options_OBJ, nic_DICT ):
	"""
	This routine
	"""
#
	affectedBridge_LIST, affectedRoute_LIST, defaultRouteAffected, affectedOpenISCSIsession_LIST, affectedBlkDev_LIST, affectedWWID_LIST	= get_affected_RSRCs( vi_errorAndLog_obj, options_OBJ, nic_DICT )

	if defaultRouteAffected:
		vi_errorAndLog_obj.vi_print_err( "DefaultRoute affected : %s" % ( defaultRouteAffected.mInterface ) )
		return errno.EPERM

	if len(affectedBridge_LIST) > 0:
		affectedBridgeName_LIST			= []
		for affectedBridge in affectedBridge_LIST:
			affectedBridgeName_LIST.append( affectedBridge.mBridgeName )
		affectedBridgeName_LIST.sort();
		vi_errorAndLog_obj.vi_print_err( "AffectedBridges exist : %s" % ( affectedBridgeName_LIST ) )
		return errno.EPERM

	if len(affectedWWID_LIST) > 0:
		vi_errorAndLog_obj.vi_print_err( "AffectedWWIDs exist : %s" % ( affectedWWID_LIST ) )
		return errno.EPERM

	vi_errorAndLog_obj.vi_print_alt( '=== Considerations' )
	vi_errorAndLog_obj.vi_print_alt( '----------------------------' )
	vi_errorAndLog_obj.vi_print_alt( "AffectedOpenISCSIsessions  : %s" % ( affectedOpenISCSIsession_LIST ) )
	vi_errorAndLog_obj.vi_print_alt( "AffectedRoutes             : %s" % ( affectedRoute_LIST ) )
	vi_errorAndLog_obj.vi_print_alt( "AffectedNICs               : %s" % ( nic_DICT.values() ) )
	vi_errorAndLog_obj.vi_print_log( '----------------------------' )
	vi_errorAndLog_obj.vi_print_log( 'PRE routes:' )

	stdout_LIST, stderr_LIST, exitStatus, deltaTime	= vi_utils.vi_exe_sync( vi_errorAndLog_obj, [ 'route' ] )
	if stdout_LIST != None and len(stdout_LIST) > 0:
		vi_errorAndLog_obj.vi_print_log( ''.join( stdout_LIST ), None )
	if stderr_LIST != None and len(stderr_LIST) > 0:
		vi_errorAndLog_obj.vi_print_log( ''.join( stderr_LIST ), None )

	vi_errorAndLog_obj.vi_print_log( '----------------------------' )
	vi_errorAndLog_obj.vi_print_log( 'PRE ifconfig:' )

	cmd_LIST	= [ 'ifconfig', '-a' ]

	stdout_LIST, stderr_LIST, exitStatus, deltaTime	= vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST )
	if stdout_LIST != None and len(stdout_LIST) > 0:
		for idx, line in enumerate(stdout_LIST):
			if line.startswith( 'lo ' ):
				break
		stdout_LIST	= stdout_LIST[ 0 : idx-1 ]
		vi_errorAndLog_obj.vi_print_log( ''.join( stdout_LIST ), None )
	if stderr_LIST != None and len(stderr_LIST) > 0:
		vi_errorAndLog_obj.vi_print_log( ''.join( stderr_LIST ), None )

	vi_errorAndLog_obj.vi_print_alt( '----------------------------' )
	vi_errorAndLog_obj.vi_print_alt( '' )

	if len(affectedOpenISCSIsession_LIST) > 0:
		vi_errorAndLog_obj.vi_print_alt( '=== Performing open iSCSI logout' )
		vi_errorAndLog_obj.vi_print_log( '----------------------------' )
		vi_errorAndLog_obj.vi_print_log( 'PRE iscsiadm node:' )

		cmd_LIST	= [ 'iscsiadm', '-m', 'node' ]
		stdout_LIST, stderr_LIST, exitStatus, deltaTime	= storage_tech.iscsiadm_exe_sync( vi_errorAndLog_obj, cmd_LIST )
		if stdout_LIST != None and len(stdout_LIST) > 0:
			vi_errorAndLog_obj.vi_print_log( ''.join( stdout_LIST ), None )

		vi_errorAndLog_obj.vi_print_log( '----------------------------' )
		vi_errorAndLog_obj.vi_print_log( 'PRE iscsiadm session:' )

		cmd_LIST	= [ 'iscsiadm', '-m', 'session' ]
		stdout_LIST, stderr_LIST, exitStatus, deltaTime	= storage_tech.iscsiadm_exe_sync( vi_errorAndLog_obj, cmd_LIST )
		if stdout_LIST != None and len(stdout_LIST) > 0:
			vi_errorAndLog_obj.vi_print_log( ''.join( stdout_LIST ), None )

		vi_errorAndLog_obj.vi_print_alt( '----------------------------' )
		vi_errorAndLog_obj.vi_print_alt( 'iscsiadm logouts:' )

		for affectedOpenISCSIsession in affectedOpenISCSIsession_LIST:

			cmd_LIST	= [ 'iscsiadm', '-m', 'node', '--logout', '-T', affectedOpenISCSIsession.mOpenISCSIstate.mIQN ]

			vi_errorAndLog_obj.vi_print_alt( '   ', " ".join(cmd_LIST) )

			stdout_LIST, stderr_LIST, exitStatus, deltaTime	= storage_tech.iscsiadm_exe_sync( vi_errorAndLog_obj, cmd_LIST )
			if stdout_LIST != None and len(stdout_LIST) > 0:
				vi_errorAndLog_obj.vi_print_alt( ''.join( stdout_LIST ), None )
			if stderr_LIST != None and len(stderr_LIST) > 0:
				vi_errorAndLog_obj.vi_print_alt( ''.join( stderr_LIST ), None )

			cmd_LIST	= [ 'iscsiadm', '-m', 'node', '-o', 'delete', '-T', affectedOpenISCSIsession.mOpenISCSIstate.mIQN ]

			stdout_LIST, stderr_LIST, exitStatus, deltaTime	= storage_tech.iscsiadm_exe_sync( vi_errorAndLog_obj, cmd_LIST )

		vi_errorAndLog_obj.vi_print_log( '----------------------------' )
		vi_errorAndLog_obj.vi_print_log( 'PST iscsiadm node:' )

		cmd_LIST	= [ 'iscsiadm', '-m', 'node' ]
		stdout_LIST, stderr_LIST, exitStatus, deltaTime	= storage_tech.iscsiadm_exe_sync( vi_errorAndLog_obj, cmd_LIST )
		if stdout_LIST != None and len(stdout_LIST) > 0:
			vi_errorAndLog_obj.vi_print_log( ''.join( stdout_LIST ), None )

		vi_errorAndLog_obj.vi_print_log( '----------------------------' )
		vi_errorAndLog_obj.vi_print_log( 'PST iscsiadm session:' )

		cmd_LIST	= [ 'iscsiadm', '-m', 'session' ]
		stdout_LIST, stderr_LIST, exitStatus, deltaTime	= storage_tech.iscsiadm_exe_sync( vi_errorAndLog_obj, cmd_LIST )
		if stdout_LIST != None and len(stdout_LIST) > 0:
			vi_errorAndLog_obj.vi_print_log( ''.join( stdout_LIST ), None )

		vi_errorAndLog_obj.vi_print_alt( '----------------------------' )
		vi_errorAndLog_obj.vi_print_alt( '' )

	vi_errorAndLog_obj.vi_print_alt( '=== Performing route removal' )
	vi_errorAndLog_obj.vi_print_alt( '----------------------------' )

	for route in affectedRoute_LIST:

		cmd_LIST	= [ 'route', 'del', '-net', route.get_IPv4_dest_str(), 'netmask', route.get_IPv4_netmask_str(), 'dev', route.mInterface ]

		vi_errorAndLog_obj.vi_print_alt( '   ', " ".join(cmd_LIST) )

		stdout_LIST, stderr_LIST, exitStatus, deltaTime	= vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST )
		if stdout_LIST != None and len(stdout_LIST) > 0:
			vi_errorAndLog_obj.vi_print_alt( ''.join( stdout_LIST ), None )
		if stderr_LIST != None and len(stderr_LIST) > 0:
			vi_errorAndLog_obj.vi_print_alt( ''.join( stderr_LIST ), None )

	vi_errorAndLog_obj.vi_print_log( '----------------------------' )
	vi_errorAndLog_obj.vi_print_log( 'PST routes:' )

	stdout_LIST, stderr_LIST, exitStatus, deltaTime	= vi_utils.vi_exe_sync( vi_errorAndLog_obj, [ 'route' ] )
	if stdout_LIST != None and len(stdout_LIST) > 0:
		vi_errorAndLog_obj.vi_print_log( ''.join( stdout_LIST ), None )
	if stderr_LIST != None and len(stderr_LIST) > 0:
		vi_errorAndLog_obj.vi_print_log( ''.join( stderr_LIST ), None )

	vi_errorAndLog_obj.vi_print_alt( '----------------------------' )
	vi_errorAndLog_obj.vi_print_alt( '' )
	vi_errorAndLog_obj.vi_print_alt( '=== Clearing IP address on interfaces' )
	vi_errorAndLog_obj.vi_print_alt( '----------------------------' )

	for nic_state in nic_DICT.values():

		if nic_state.mIPv4_addr == None:
			continue

		cmd_LIST	= [ 'ifconfig', nic_state.mInterface, 'inet', '0' ]

		vi_errorAndLog_obj.vi_print_alt( '   ', " ".join(cmd_LIST) )

		stdout_LIST, stderr_LIST, exitStatus, deltaTime	= vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST )
		if stdout_LIST != None and len(stdout_LIST) > 0:
			vi_errorAndLog_obj.vi_print_alt( ''.join( stdout_LIST ), None )
		if stderr_LIST != None and len(stderr_LIST) > 0:
			vi_errorAndLog_obj.vi_print_alt( ''.join( stderr_LIST ), None )

	vi_errorAndLog_obj.vi_print_alt( '----------------------------' )
	vi_errorAndLog_obj.vi_print_alt( '' )
	vi_errorAndLog_obj.vi_print_alt( '=== Downing interfaces' )
	vi_errorAndLog_obj.vi_print_alt( '----------------------------' )

	for nic_state in nic_DICT.values():

		if nic_state.mIsUP == False:
			continue

		cmd_LIST	= [ 'ifconfig', nic_state.mInterface, 'down' ]

		vi_errorAndLog_obj.vi_print_alt( '   ', " ".join(cmd_LIST) )

		stdout_LIST, stderr_LIST, exitStatus, deltaTime	= vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST )
		if stdout_LIST != None and len(stdout_LIST) > 0:
			vi_errorAndLog_obj.vi_print_alt( ''.join( stdout_LIST ), None )
		if stderr_LIST != None and len(stderr_LIST) > 0:
			vi_errorAndLog_obj.vi_print_alt( ''.join( stderr_LIST ), None )

	vi_errorAndLog_obj.vi_print_alt( '----------------------------' )
	vi_errorAndLog_obj.vi_print_alt( '' )
	vi_errorAndLog_obj.vi_print_alt( '=== Reseting MTUs on interfaces' )
	vi_errorAndLog_obj.vi_print_alt( '----------------------------' )

	for nic_state in nic_DICT.values():

		if nic_state.mMTU == 1500:
			continue

		cmd_LIST	= [ 'ifconfig', nic_state.mInterface, 'mtu', '1500' ]

		vi_errorAndLog_obj.vi_print_alt( '   ', " ".join(cmd_LIST) )

		stdout_LIST, stderr_LIST, exitStatus, deltaTime	= vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST )
		if stdout_LIST != None and len(stdout_LIST) > 0:
			vi_errorAndLog_obj.vi_print_alt( ''.join( stdout_LIST ), None )
		if stderr_LIST != None and len(stderr_LIST) > 0:
			vi_errorAndLog_obj.vi_print_alt( ''.join( stderr_LIST ), None )

	vi_errorAndLog_obj.vi_print_alt( '----------------------------' )
	vi_errorAndLog_obj.vi_print_alt( '' )

	bondName_LIST	= []
	for nic_state in nic_DICT.values():
		if nic_state.mInterface.startswith( 'bond' ):
			bondName_LIST.append( nic_state.mInterface )

	if len(bondName_LIST) > 0:
		vi_errorAndLog_obj.vi_print_alt( '=== Breaking Bonds' )
		vi_errorAndLog_obj.vi_print_alt( '----------------------------' )

		for bondName in bondName_LIST:

			cmd_LIST	= [ '/etc/nex/scripts/nex-bond-if', 'stop', "bondm=%s" % ( bondName ) ]

			vi_errorAndLog_obj.vi_print_alt( '   ', " ".join(cmd_LIST) )

			stdout_LIST, stderr_LIST, exitStatus, deltaTime	= vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST )
			if stdout_LIST != None and len(stdout_LIST) > 0:
				vi_errorAndLog_obj.vi_print_alt( ''.join( stdout_LIST ), None )
			if stderr_LIST != None and len(stderr_LIST) > 0:
				vi_errorAndLog_obj.vi_print_alt( ''.join( stderr_LIST ), None )

		vi_errorAndLog_obj.vi_print_alt( '----------------------------' )
		vi_errorAndLog_obj.vi_print_alt( '' )

	vi_errorAndLog_obj.vi_print_log( '=== PST ifconfig' )
	vi_errorAndLog_obj.vi_print_log( '----------------------------' )

	cmd_LIST	= [ 'ifconfig', '-a' ]

	stdout_LIST, stderr_LIST, exitStatus, deltaTime	= vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST )
	if stdout_LIST != None and len(stdout_LIST) > 0:
		for idx, line in enumerate(stdout_LIST):
			if line.startswith( 'lo ' ):
				break
		stdout_LIST	= stdout_LIST[ 0 : idx-1 ]
		vi_errorAndLog_obj.vi_print_log( ''.join( stdout_LIST ), None )
	if stderr_LIST != None and len(stderr_LIST) > 0:
		vi_errorAndLog_obj.vi_print_log( ''.join( stderr_LIST ), None )

	vi_errorAndLog_obj.vi_print_log( '----------------------------' )
	vi_errorAndLog_obj.vi_print_log( '' )

	vi_errorAndLog_obj.vi_print_alt( 'Deprovisioning Complete' )

	return 0

####################################################################################################
def parse_given_args():
	"""
	This routine
	"""
#
	parser = OptionParser()

	parser.add_option("", "--debugLvl", dest="debugLvl", default=0, type="int",
					help="debugLvl message printing to stderr" )

	parser.add_option("", "--networkList", dest="networkList", default=None, type="string",
					help="networkList:  network1,...,networkN  OR  <<STDIN>>  to read in the list from stdin. This is a list of eth or MAC names" )

	parser.add_option( "", "--onlyListAffectedRSRCs", dest="onlyListAffectedRSRCs", default=False, action="store_true",
					help="only list the affected RSRCs that must be disconnected before --doDeprovision can be performed" )

	parser.add_option("", "--ignoreIfAlternateRouteExists", dest="ignoreIfAlternateRouteExists", default=False, action="store_true",
					help="ignore iSCSI resources for those with alternate routes not on the given networkList")

	parser.add_option("", "--listUsingXML", dest="listUsingXML", default=False, action="store_true",
					help="use XML when listing the affected RSRCs")

	parser.add_option("", "--doDeprovision", dest="doDeprovision", default=False, action="store_true",
					help="deprovision the specified networks (includes iSCSI logout, route table, bonds, MTU, etc)")

	options_OBJ, args_LIST = parser.parse_args()

	network_LIST = []
	if options_OBJ.networkList == '<<STDIN>>':
		if options_OBJ.debugLvl >= 4:
			print >> sys.stderr, 'Reading dist_LIST from stdin'
		try:
			while True:
				stdinInputLine = raw_input()
				if stdinInputLine:
					stdinInputLine	= stdinInputLine.strip()
					if len(stdinInputLine):
						network_LIST.append( stdinInputLine )
		except EOFError:
			pass
	else:
		if options_OBJ.networkList:
			for givenItem in options_OBJ.networkList.split(','):
				givenItem	= givenItem.strip()
				if len(givenItem):
					network_LIST.append( givenItem )

	vi_utils.vi_sort_uniq( network_LIST, True )

	if 0:
		print >> sys.stderr, options_OBJ
		print >> sys.stderr, args_LIST
		print >> sys.stderr, dir(options_OBJ)
		print >> sys.stderr, parser.values
		print >> sys.stderr, task_LIST
		vi_utils.vi_dbg_stop()

	if len( network_LIST ) == 0 or not (options_OBJ.onlyListAffectedRSRCs or options_OBJ.doDeprovision):
		print >> sys.stderr, '\nno work requested\n'
		parser.print_help(sys.stderr)
		print >> sys.stderr, ''
		vi_utils.vi_exit( 1 )

	if options_OBJ.onlyListAffectedRSRCs and options_OBJ.doDeprovision:
		print >> sys.stderr, '\nconflicting work requested\n'
		parser.print_help(sys.stderr)
		print >> sys.stderr, ''
		vi_utils.vi_exit( 1 )

	return options_OBJ, args_LIST, network_LIST

####################################################################################################
def init_complex_globals( options_OBJ, args_LIST ):
	"""
	This routine initializes and complex global variables
	"""
#
	global global_errAndLog
	global global_logFilePath

	newLog_FILE = None
	if not options_OBJ.onlyListAffectedRSRCs:
		vi_utils.vi_rototillLogs( global_logDirectory, global_baseLogFileName, 40, 10 )

		logFileName			= vi_utils.vi_generate_log_filename( global_baseLogFileName )
		global_logFilePath	= "%s/%s" % ( global_logDirectory, logFileName )
		try:
			newLog_FILE			= open( global_logFilePath, 'a', 0 )	# UNbuffered
		except:
			pass

	errAndLog_dst_DICT =	{	'stdErr'			:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__ERR
														|	vi_utils.VI_ErrAndLog__FOR_FLAG__WRN
														|	vi_utils.VI_ErrAndLog__FOR_FLAG__DBG
														,	sys.stderr
														)
							,	'stdout'			:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__STD
														,	sys.stdout
														)
							,	'alt_to_stdout'		:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__ALT
														,	sys.stdout
														)
							}

	if newLog_FILE:
		errAndLog_dst_DICT[ 'log' ]			=	(	vi_utils.VI_ErrAndLog__FOR_FLAG__LOG
												,	newLog_FILE
												)
		errAndLog_dst_DICT[ 'alt_to_log' ]	=	(	vi_utils.VI_ErrAndLog__FOR_FLAG__ALT
												,	newLog_FILE
												)

	global_errAndLog = vi_utils.VI_ErrAndLogRedirector_OBJ( errAndLog_dst_DICT, options_OBJ.debugLvl )

	if newLog_FILE:
		global_errAndLog.vi_print_alt( "LOG: %s\n" % ( global_logFilePath ) )
	elif not options_OBJ.onlyListAffectedRSRCs:
		global_errAndLog.vi_print_wrn( "could not create global log FILE '%s'" % ( global_logFilePath ) )
		global_logFilePath	= ''

	return global_errAndLog

####################################################################################################
def main():
	"""
	This routine is the script entry point
	"""
#
	options_OBJ, args_LIST, network_LIST = parse_given_args()

	vi_errorAndLog_obj	= init_complex_globals( options_OBJ, args_LIST )

	retVal	= 0
	while True:
		try:
			nic_DICT, unknown_network_LIST	= get_NIC_dict( vi_errorAndLog_obj, options_OBJ, network_LIST )
		except:
			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			retVal	= errno.EFAULT
			break

		if options_OBJ.onlyListAffectedRSRCs:
			if options_OBJ.listUsingXML:
				domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( 0, 2 )
				domDocHelper.push_level( 'AffectedRSRCs', None )
			else:
				domDocHelper	= None
				vi_errorAndLog_obj.vi_print_std( '{' )
			if len(unknown_network_LIST) > 0:
				vi_errorAndLog_obj.vi_print_log( 'Unknown networks given:' )
				vi_errorAndLog_obj.vi_print_log( "    %s" % ( unknown_network_LIST ) )
				if domDocHelper == None:
					vi_errorAndLog_obj.vi_print_std( '    "UnknownNetworks" : %s' % ( unknown_network_LIST ) )
				else:
					domDocHelper.push_level( 'UnknownNetworks', { 'Num' : str(len(unknown_network_LIST)) } )
					for unknownNetwork in unknown_network_LIST:
						domDocHelper.add_data_elem( 'UnknownNetwork', unknownNetwork )
					domDocHelper.pop_level()
			try:
				retVal	= list_affected_RSRCs( vi_errorAndLog_obj, options_OBJ, nic_DICT, domDocHelper )
			except:
				vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
				retVal	= errno.EFAULT
				break
			if domDocHelper == None:
				vi_errorAndLog_obj.vi_print_std( '}' )
			else:
				vi_errorAndLog_obj.vi_print_std( '\n'.join( domDocHelper.finalize() ) )

		if options_OBJ.doDeprovision:
			if len(unknown_network_LIST) > 0:
				vi_errorAndLog_obj.vi_print_alt( 'Unknown networks given:' )
				vi_errorAndLog_obj.vi_print_alt( "    %s" % ( unknown_network_LIST ) )
				retVal	= errno.EPERM
				break
			try:
				retVal	= do_deprovision( vi_errorAndLog_obj, options_OBJ, nic_DICT )
			except:
				vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
				retVal	= errno.EFAULT
				break

		break

	if len(global_logFilePath) > 0:
		vi_errorAndLog_obj.close_dst_files()
		vi_utils.vi_exe_sync( vi_errorAndLog_obj, [ 'gzip', '--best', global_logFilePath ], 30 )

	if retVal:
		return retVal

	return 0

####################################################################################################
### main

sys.stdout = os.fdopen( sys.stdout.fileno(), sys.stdout.mode, 0 )	# UNbuffer stdout

exitValue  = main()

sys.exit( exitValue )

# END
####################################################################################################
