#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Script for provisioning network interfaces
"""

import sys
import os
import re
import traceback
import errno
import time

import vi_utils
import vi_os
import storage_tech

from optparse import OptionParser
from optparse import OptionGroup

####################################################################################################

global_logDirectory		= '/var/log/provision_network'
global_baseLogFileName	= 'provision'

global_errAndLog		= None		# VI_ErrAndLog_OBJ
global_logFilePath		= ''

####################################################################################################

class SetupStaticIP_Params_OBJ:

	def __init__( self, ipv4_addr, ipv4_netmask ):
		self.mIPv4_addr		= ipv4_addr
		self.mIPv4_netmask	= ipv4_netmask

	def __repr__(self):
		return "IP: %s nMsk: %s" % (
				self.mIPv4_addr
			,	self.mIPv4_netmask )

class CreateBond_Params_OBJ:

	def __init__( self, bondTag, bondTypeStr, bondTypeInt ):
		self.mBondTag		= bondTag
		self.mBondTypeStr	= bondTypeStr
		self.mBondTypeInt	= bondTypeInt

	def __repr__(self):
		return "tag: %s typ: %s [%d]" % (
				self.mBondTag
			,	self.mBondTypeStr
			,	self.mBondTypeInt )

####################################################################################################
def get_NIC_list( vi_errorAndLog_obj, options_OBJ, network_LIST ):
	"""
	This routine
	"""
#
	nic_LIST				= []
	unknown_network_LIST	= []

	interface_DICT	= vi_os.vi_get_NIC_dict( vi_errorAndLog_obj, True, True )

	for network in network_LIST:
		if not network.startswith( 'eth' ) and not network.startswith( 'bond' ):
			unknown_network_LIST.append( network )
			continue
		for interface, nic_state in interface_DICT.iteritems():
			if network == interface:
				for alt_nic_state in nic_LIST:
					if alt_nic_state.mInterface == nic_state.mInterface:
						break
				else:
					nic_LIST.append( nic_state )
				break
			if network == nic_state.mMAC:
				for alt_nic_state in nic_LIST:
					if alt_nic_state.mInterface == nic_state.mInterface:
						break
				else:
					nic_LIST.append( nic_state )
				break
		else:
			unknown_network_LIST.append( network )

	return nic_LIST, unknown_network_LIST

####################################################################################################
def do_setupDHCP( vi_errorAndLog_obj, options_OBJ, nic_state ):
	"""
	This routine
	"""
#
	timeoutInSecs	= 120 + options_OBJ.extraTime

	vi_errorAndLog_obj.vi_print_log( "Resetting NIC : %s : mtu = %d" % ( nic_state.mInterface, options_OBJ.setMTU ) )

	if vi_os.vi_reset_NIC_and_wait_for_RUNNING( vi_errorAndLog_obj, nic_state, timeoutInSecs, options_OBJ.setMTU ):
		vi_errorAndLog_obj.vi_print_err( "NIC reset failed" )
		return errno.EBUSY

	vi_errorAndLog_obj.vi_print_log( "Invoking DHCP" )

	if vi_os.vi_invoke_dhcp( vi_errorAndLog_obj, nic_state, timeoutInSecs ):
		vi_errorAndLog_obj.vi_print_err( "DHCP failed" )
		return errno.EBUSY

	interface_DICT	= vi_os.vi_get_NIC_dict( vi_errorAndLog_obj, False, True )
	new_nic_state	= interface_DICT.get( nic_state.mInterface, None )
	if not new_nic_state:
		vi_errorAndLog_obj.vi_print_err( "DHCP completed, but ifconfig info unavailable ?!?" )
		return errno.EBUSY

	vi_errorAndLog_obj.vi_print_log( "DHCP succeeded :: IP %s netmask %s" % ( new_nic_state.mIPv4_addr, new_nic_state.mIPv4_netmask ) )

	return 0

####################################################################################################
def do_setupStaticIP( vi_errorAndLog_obj, options_OBJ, nic_state ):
	"""
	This routine
	"""
#
	setupStaticIP_params	= options_OBJ.setupStaticIP
	timeoutInSecs			= 120 + options_OBJ.extraTime

	vi_errorAndLog_obj.vi_print_log( "Resetting NIC : %s : mtu = %d" % ( nic_state.mInterface, options_OBJ.setMTU ) )

	if vi_os.vi_reset_NIC_and_wait_for_RUNNING( vi_errorAndLog_obj, nic_state, timeoutInSecs, options_OBJ.setMTU ):
		vi_errorAndLog_obj.vi_print_err( "NIC reset failed" )
		return errno.EBUSY

	cmd_LIST	= [ 'ifconfig', nic_state.mInterface, setupStaticIP_params.mIPv4_addr, 'netmask', setupStaticIP_params.mIPv4_netmask ]

	vi_errorAndLog_obj.vi_print_log( "Setting StaticIP: [%d] %s" % ( timeoutInSecs, ' '.join( cmd_LIST ) ) )

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, timeoutInSecs )
	if exitStatus != 0:

		vi_errorAndLog_obj.vi_print_err( "ERROR: staticIP ifconfig failed for %s (%s) %s %s : %s" % \
				(	nic_state.mInterface
				,	nic_state.mMAC
				,	setupStaticIP_params.mIPv4_addr
				,	setupStaticIP_params.mIPv4_netmask
				,	exitStatus ) )
		if len( stdout_LIST ) > 0:
			vi_errorAndLog_obj.vi_print_log( ''.join( stdout_LIST ) )
		if len( stderr_LIST ) > 0:
			vi_errorAndLog_obj.vi_print_log( ''.join( stderr_LIST ) )

		# Leave the NIC UP and RUNNING, but clear the IP
		cmd_LIST	= [ 'ifconfig', nic_state.mInterface, 'inet', '0' ]
		vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, timeoutInSecs )

		return errno.EBUSY

	if vi_os.vi_wait_for_NIC_RUNNING( vi_errorAndLog_obj, nic_state, timeoutInSecs ):
		vi_errorAndLog_obj.vi_print_err( "NIC did not complete staticIP init" )
		return errno.EBUSY

	vi_errorAndLog_obj.vi_print_log( "staticIP succeeded" )

	return 0

####################################################################################################
def do_createBond( vi_errorAndLog_obj, options_OBJ, nic_LIST ):
	"""
	This routine
	"""
#
	createBond_params	= options_OBJ.createBond
	timeoutInSecs		= 120 + options_OBJ.extraTime

	vi_errorAndLog_obj.vi_print_log( "Resetting NIC : %s %s : mtu = %d" % ( nic_LIST[0].mInterface, nic_LIST[1].mInterface, options_OBJ.setMTU ) )

	if vi_os.vi_reset_NIC_and_wait_for_RUNNING( vi_errorAndLog_obj, nic_LIST[0], timeoutInSecs, options_OBJ.setMTU ):
		vi_errorAndLog_obj.vi_print_err( "NIC reset failed : %s" % ( nic_LIST[0].mInterface ) )
		return errno.EBUSY

	if vi_os.vi_reset_NIC_and_wait_for_RUNNING( vi_errorAndLog_obj, nic_LIST[1], timeoutInSecs, options_OBJ.setMTU ):
		vi_errorAndLog_obj.vi_print_err( "NIC reset failed : %s" % ( nic_LIST[1].mInterface ) )
		return errno.EBUSY

	cmd_LIST	=	[	'/etc/nex/scripts/nex-bond-if'
					,	'start'
					,	"bondm=%s"		% ( createBond_params.mBondTag )
					,	"primary=%s"	% ( nic_LIST[ 0 ].mInterface )
					,	"backup=%s"		% ( nic_LIST[ 1 ].mInterface )
					,	"bondmode=%s"	% ( createBond_params.mBondTypeInt )
					]

	vi_errorAndLog_obj.vi_print_log( "Creating Bond: [%d] %s" % ( timeoutInSecs, ' '.join( cmd_LIST ) ) )

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, timeoutInSecs )

	if len( stdout_LIST ) > 0:
		vi_errorAndLog_obj.vi_print_log( ''.join( stdout_LIST ) )
	if len( stderr_LIST ) > 0:
		vi_errorAndLog_obj.vi_print_log( ''.join( stderr_LIST ) )

	if exitStatus != 0:
		vi_errorAndLog_obj.vi_print_err( "Bond creation failed" )
		return errno.EBUSY

	vi_errorAndLog_obj.vi_print_log( "createBond succeeded" )

	return 0

####################################################################################################
def parse_given_args():
	"""
	This routine
	"""
#
	bondType_LIST	= [ 'Failover', 'LinkAggregation', 'LoadBalance' ]

	parser = OptionParser()

	parser.add_option("", "--debugLvl", dest="debugLvl", default=0, type="int",
					help="debugLvl message printing to stderr" )

	parser.add_option("", "--extraTime", dest="extraTime", default=0, type="int",
					help="amount of extra time in seconds to add when waiting for network operations" )

	parser.add_option("", "--networkList", dest="networkList", default=None, type="string",
					help="networkList:  network1,...,networkN  OR  <<STDIN>>  to read in the list from stdin. This is a list of eth or MAC names" )

	parser.add_option("", "--setupDHCP", dest="setupDHCP", default=False, action="store_true",
					help="setup a setupDHCP on the single network given" )

	parser.add_option("", "--setupStaticIP", dest="setupStaticIP", default=None, type="string",
					help="setup a staticIP on the single network given : format = <IP>:<netmask>" )

	parser.add_option("", "--setMTU", dest="setMTU", default=1500, type="int",
					help="specifies the mtu to be used for the interface" )

	parser.add_option("", "--createBond", dest="createBond", default=None, type="string",
					help="create bond for networkList (1st == primary) : format = <bondTag>:<bondType>" )

	group = OptionGroup( parser, "BondTypes", "%s" % '    '.join(bondType_LIST) )

	parser.add_option_group( group )

	options_OBJ, args_LIST = parser.parse_args()

	network_LIST = []
	if options_OBJ.networkList == '<<STDIN>>':
		if options_OBJ.debugLvl >= 4:
			print >> sys.stderr, 'Reading dist_LIST from stdin'
		try:
			while True:
				stdinInputLine = raw_input()
				if stdinInputLine:
					stdinInputLine	= stdinInputLine.strip()
					if len(stdinInputLine):
						network_LIST.append( stdinInputLine )
		except EOFError:
			pass
	else:
		if options_OBJ.networkList:
			for givenItem in options_OBJ.networkList.split(','):
				givenItem	= givenItem.strip()
				if len(givenItem):
					network_LIST.append( givenItem )

	if 0:
		print >> sys.stderr, options_OBJ
		print >> sys.stderr, args_LIST
		print >> sys.stderr, dir(options_OBJ)
		print >> sys.stderr, parser.values
		print >> sys.stderr, task_LIST
		vi_utils.vi_dbg_stop()

	amountOfWorkRequested	= 0

	if options_OBJ.setupDHCP:
		amountOfWorkRequested	= amountOfWorkRequested + 1
		if len(network_LIST) != 1:
			print >> sys.stderr, '\nsetupDHCP requires 1 network\n'
			parser.print_help(sys.stderr)
			print >> sys.stderr, ''
			vi_utils.vi_exit( 1 )

	if options_OBJ.setupStaticIP:
		amountOfWorkRequested	= amountOfWorkRequested + 1
		if len(network_LIST) != 1:
			print >> sys.stderr, '\nsetupStaticIP requires 1 network\n'
			parser.print_help(sys.stderr)
			print >> sys.stderr, ''
			vi_utils.vi_exit( 1 )
		setupStaticIP_frmt_cmpldRegEx	= re.compile( "^(\d+.\d+.\d+.\d+):(\d+.\d+.\d+.\d+)$" )
		setupStaticIP_frmt_match		= setupStaticIP_frmt_cmpldRegEx.match( options_OBJ.setupStaticIP )
		if not setupStaticIP_frmt_match:
			print >> sys.stderr, '\nbad setupStaticIP format\n'
			parser.print_help(sys.stderr)
			print >> sys.stderr, ''
			vi_utils.vi_exit( 1 )
		setupStaticIP_params		= SetupStaticIP_Params_OBJ( setupStaticIP_frmt_match.group(1), setupStaticIP_frmt_match.group(2) )
		options_OBJ.setupStaticIP	= setupStaticIP_params

	if options_OBJ.createBond:
		amountOfWorkRequested	= amountOfWorkRequested + 1
		if len(network_LIST) != 2:
			print >> sys.stderr, "\ncreateBond requires 2 networks : %s\n" % ( network_LIST )
			parser.print_help(sys.stderr)
			print >> sys.stderr, ''
			vi_utils.vi_exit( 1 )
		createBond_frmt_cmpldRegEx	= re.compile( "^(bond\d+):(\S+)$" )
		createBond_frmt_match		= createBond_frmt_cmpldRegEx.match( options_OBJ.createBond )
		if not createBond_frmt_match:
			print >> sys.stderr, '\nbad createBond format\n'
			parser.print_help(sys.stderr)
			print >> sys.stderr, ''
			vi_utils.vi_exit( 1 )
		for bondTypeInt, bondTypeStr in enumerate(bondType_LIST):
			if bondTypeStr == createBond_frmt_match.group(2):
				break
		else:
			print >> sys.stderr, "\nbad createBond bondType '%s'\n" % ( createBond_frmt_match.group(2) )
			parser.print_help(sys.stderr)
			print >> sys.stderr, ''
			vi_utils.vi_exit( 1 )
		createBond_params		= CreateBond_Params_OBJ( createBond_frmt_match.group(1), createBond_frmt_match.group(2), bondTypeInt )
		options_OBJ.createBond	= createBond_params

	if len(network_LIST) == 0 or amountOfWorkRequested == 0:
		print >> sys.stderr, '\nno work requested\n'
		parser.print_help(sys.stderr)
		print >> sys.stderr, ''
		vi_utils.vi_exit( 1 )

	if amountOfWorkRequested > 1:
		print >> sys.stderr, '\nconflicting work requested\n'
		parser.print_help(sys.stderr)
		print >> sys.stderr, ''
		vi_utils.vi_exit( 1 )

	if options_OBJ.extraTime < 0:
		print >> sys.stderr, '\nbad extraTime\n'
		parser.print_help(sys.stderr)
		print >> sys.stderr, ''
		vi_utils.vi_exit( 1 )

	if options_OBJ.setMTU < 1500:
		print >> sys.stderr, '\nbad setMTU\n'
		parser.print_help(sys.stderr)
		print >> sys.stderr, ''
		vi_utils.vi_exit( 1 )

	return options_OBJ, args_LIST, network_LIST

####################################################################################################
def init_complex_globals( options_OBJ, args_LIST ):
	"""
	This routine initializes and complex global variables
	"""
#
	global global_errAndLog
	global global_logFilePath

	newLog_FILE = None
	vi_utils.vi_rototillLogs( global_logDirectory, global_baseLogFileName, 40, 10 )

	logFileName			= vi_utils.vi_generate_log_filename( global_baseLogFileName )
	global_logFilePath	= "%s/%s" % ( global_logDirectory, logFileName )
	try:
		newLog_FILE			= open( global_logFilePath, 'a', 0 )	# UNbuffered
	except:
		pass

	errAndLog_dst_DICT =	{	'stdErr'			:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__ERR
														|	vi_utils.VI_ErrAndLog__FOR_FLAG__WRN
														|	vi_utils.VI_ErrAndLog__FOR_FLAG__DBG
														,	sys.stderr
														)
							,	'stdout'			:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__STD
														,	sys.stdout
														)
							,	'log_to_stdout'		:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__LOG
														,	sys.stdout
														)
							}

	if newLog_FILE:
		errAndLog_dst_DICT[ 'log' ]			=	(	vi_utils.VI_ErrAndLog__FOR_FLAG__LOG
												,	newLog_FILE
												)
		errAndLog_dst_DICT[ 'alt_to_log' ]	=	(	vi_utils.VI_ErrAndLog__FOR_FLAG__ALT
												,	newLog_FILE
												)

	global_errAndLog = vi_utils.VI_ErrAndLogRedirector_OBJ( errAndLog_dst_DICT, options_OBJ.debugLvl )

	if newLog_FILE:
		global_errAndLog.vi_print_log( "LOG: %s\n" % ( global_logFilePath ) )
	else:
		global_errAndLog.vi_print_wrn( "could not create global log FILE '%s'" % ( global_logFilePath ) )
		global_logFilePath	= ''

	return global_errAndLog

####################################################################################################
def main():
	"""
	This routine is the script entry point
	"""
#
	options_OBJ, args_LIST, network_LIST = parse_given_args()

	vi_errorAndLog_obj	= init_complex_globals( options_OBJ, args_LIST )

	retVal	= 0
	while True:
		try:
			nic_LIST, unknown_network_LIST	= get_NIC_list( vi_errorAndLog_obj, options_OBJ, network_LIST )
		except:
			vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
			retVal	= errno.EFAULT
			break

		if len(unknown_network_LIST) > 0:
			vi_errorAndLog_obj.vi_print_log( 'Unknown networks given:' )
			vi_errorAndLog_obj.vi_print_log( "    %s" % ( unknown_network_LIST ) )
			retVal	= errno.EPERM
			break

		if options_OBJ.setupDHCP:
			try:
				retVal	= do_setupDHCP( vi_errorAndLog_obj, options_OBJ, nic_LIST[0] )
			except:
				vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
				retVal	= errno.EFAULT
				break

		if options_OBJ.setupStaticIP:
			try:
				retVal	= do_setupStaticIP( vi_errorAndLog_obj, options_OBJ, nic_LIST[0] )
			except:
				vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
				retVal	= errno.EFAULT
				break

		if options_OBJ.createBond:
			if len(network_LIST) != len(nic_LIST):
				vi_errorAndLog_obj.vi_print_err( 'networkList collisions detected' )
				retVal	= errno.EPERM
				break

			try:
				retVal	= do_createBond( vi_errorAndLog_obj, options_OBJ, nic_LIST )
			except:
				vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
				retVal	= errno.EFAULT
				break

		break

	if len(global_logFilePath) > 0:
		vi_errorAndLog_obj.close_dst_files()
		vi_utils.vi_exe_sync( vi_errorAndLog_obj, [ 'gzip', '--best', global_logFilePath ], 30 )

	if retVal:
		return retVal

	return 0

####################################################################################################
### main

sys.stdout = os.fdopen( sys.stdout.fileno(), sys.stdout.mode, 0 )	# UNbuffer stdout

exitValue  = main()

sys.exit( exitValue )

# END
####################################################################################################
