#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Script for helping with storage configuration changes
"""

import sys
import os
import re
import traceback

import vi_utils
import storage_tech

from optparse import OptionParser
from optparse import OptionGroup

####################################################################################################

global_errAndLog		= None		# VI_ErrAndLog_OBJ

####################################################################################################

class Task_OBJ:

	def __init__(self, theArgNameStr, theHelpStr, theFcn):

		self.mArgNameStr	= theArgNameStr
		self.mHelpStr		= theHelpStr
		self.mDoScan		= False
		self.mValue			= None
		self.mFcn			= theFcn

	def __repr__(self):

		return "mArgNameStr: %s mHelpStr: %s mDoScan: %s mValue: %s mFcn: %s" % (
						self.mArgNameStr
					,	self.mHelpStr
					,	self.mDoScan
					,	self.mValue
					,	self.mFcn )

####################################################################################################
def task_vgchangeAllEnable( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
	storage_tech.do_dom0_memory_safe_vgchange( vi_errorAndLog_obj, True, None, False )

####################################################################################################
def task_vgchangeAllDisable( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
	storage_tech.do_dom0_memory_safe_vgchange( vi_errorAndLog_obj, False, None, False )

####################################################################################################
def task_vgchangeListEnable( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
	vgname_LIST	= theValue.split( ',' )

	storage_tech.do_dom0_memory_safe_vgchange( vi_errorAndLog_obj, True, vgname_LIST, False )

####################################################################################################
def task_vgchangeListDisable( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
	vgname_LIST	= theValue.split( ',' )

	storage_tech.do_dom0_memory_safe_vgchange( vi_errorAndLog_obj, False, vgname_LIST, False )

####################################################################################################
def parse_given_args():
	"""
	This routine
	"""
#
	task_no_arg_LIST	=	[	Task_OBJ(	'vgchange_all_enable'
										,	'safe vgchange --available ly (All VGs)'
										,	task_vgchangeAllEnable
										)
							,	Task_OBJ(	'vgchange_all_disable'
										,	'safe vgchange --available ln (All VGs)'
										,	task_vgchangeAllDisable
										)
							]

	task_with_arg_LIST	=	[	Task_OBJ(	'vgchange_list_enable'
										,	'safe vgchange --available ly <VG1,VG2,...,VGn>'
										,	task_vgchangeListEnable
										)
							,	Task_OBJ(	'vgchange_list_disable'
										,	'safe vgchange --available ln <VG1,VG2,...,VGn>'
										,	task_vgchangeListDisable
										)
							]

	parser = OptionParser()

	parser.add_option("", "--debugLvl", dest="debugLvl", default=0, type="int",
					help="debugLvl message printing to stderr" )

	def store_value____task_no_arg_do( option, opt_str, value, parser ):
		taskIdx 		= int(option.dest)
		task			= task_no_arg_LIST[ taskIdx ]
		task.mDoScan	= True
		task.mValue		= value

	def store_value____task_with_arg_do( option, opt_str, value, parser ):
		taskIdx 		= int(option.dest)
		task			= task_with_arg_LIST[ taskIdx ]
		task.mDoScan	= True
		task.mValue		= value

	taskIdx = -1
	for task in task_no_arg_LIST:
		taskIdx = taskIdx + 1

		parser.add_option(	"", "--do_%s" % ( task.mArgNameStr ), dest=str(taskIdx), action="callback",
							callback=store_value____task_no_arg_do,  help="Do Task :: %s" % ( task.mHelpStr ) )

	taskIdx = -1
	for task in task_with_arg_LIST:
		taskIdx = taskIdx + 1

		parser.add_option(	"", "--do_%s" % ( task.mArgNameStr ), dest=str(taskIdx), action="callback", type="str",
							callback=store_value____task_with_arg_do,  help="Do Task :: %s" % ( task.mHelpStr ) )

	options_OBJ, args_LIST = parser.parse_args()

	tasksToPerform_LIST = []
	for task in task_no_arg_LIST:
		if task.mDoScan:
			tasksToPerform_LIST.append( task )
	for task in task_with_arg_LIST:
		if task.mDoScan:
			tasksToPerform_LIST.append( task )

	if 0:
		print >> sys.stderr, options_OBJ
		print >> sys.stderr, args_LIST
		print >> sys.stderr, dir(options_OBJ)
		print >> sys.stderr, parser.values
		print >> sys.stderr, tasksToPerform_LIST
		vi_utils.vi_dbg_stop()

	if len( tasksToPerform_LIST ) == 0:
		print >> sys.stderr, 'no work requested\n'
		parser.print_help(sys.stderr)
		print >> sys.stderr, ''
		vi_utils.vi_exit( 2 )

	return ( options_OBJ, args_LIST, tasksToPerform_LIST )

####################################################################################################
def init_complex_globals( options_OBJ, args_LIST ):
	"""
	This routine initializes and complex global variables
	"""
#
	global global_errAndLog

	errAndLog_dst_DICT =	{	'stdout'			:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__ALL
														,	sys.stdout
														)
							}

	global_errAndLog = vi_utils.VI_ErrAndLogRedirector_OBJ( errAndLog_dst_DICT, options_OBJ.debugLvl )

####################################################################################################
def main():
	"""
	This routine is the script entry point
	"""
#
	options_OBJ, args_LIST, tasksToPerform_LIST = parse_given_args()

	init_complex_globals( options_OBJ, args_LIST )

	for task in tasksToPerform_LIST:
		try:
			task.mFcn( global_errAndLog, options_OBJ, task.mValue )
		except:
			global_errAndLog.vi_croak_exception( traceback.format_exc() )

	return 0

####################################################################################################
### main

sys.stdout = os.fdopen( sys.stdout.fileno(), sys.stdout.mode, 0 )	# UNbuffer stdout

exitValue  = main()

sys.exit( exitValue )

# END
####################################################################################################
