#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Script for extracting storage information
"""

import sys
import os
import re
import traceback
import difflib

import vi_utils
import storage_tech

from optparse import OptionParser
from optparse import OptionGroup

####################################################################################################

global_errAndLog		= None		# VI_ErrAndLog_OBJ

####################################################################################################

class Task_OBJ:

	def __init__(self, theArgNameStr, theHelpStr, theFcn):

		self.mArgNameStr	= theArgNameStr
		self.mHelpStr		= theHelpStr
		self.mDoScan		= False
		self.mValue			= None
		self.mFcn			= theFcn

	def __repr__(self):

		return "mArgNameStr: %s mHelpStr: %s mDoScan: %s mValue: %s mFcn: %s" % (
						self.mArgNameStr
					,	self.mHelpStr
					,	self.mDoScan
					,	self.mValue
					,	self.mFcn )

####################################################################################################
def task_listDisks( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
	dm_maps_DICT	= storage_tech.get_device_mapper_maps( vi_errorAndLog_obj, False )
	for dm_map_name, dm_map in dm_maps_DICT.iteritems():
		vi_errorAndLog_obj.vi_print_std( "%s" % ( dm_map_name ) )

####################################################################################################
def task_mpa_walk( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
 	device_mapper_major_num	= storage_tech.get_device_mapper_major_num( vi_errorAndLog_obj )
 	if device_mapper_major_num < 0:
		return

	multipathInfo_DICT						= storage_tech.get_multipath_info( vi_errorAndLog_obj, False )
	blk_dev_to_openiscsi_session_info_DICT	= storage_tech.get_blkdev_to_openiscsi_session_info_dict( vi_errorAndLog_obj, False )
	blk_dev_to_SAS_state_DICT				= storage_tech.get_SAS_blk_dict( vi_errorAndLog_obj, False)
	sorted_wwids		= multipathInfo_DICT.keys()
	sorted_wwids.sort()
	for wwid in sorted_wwids:
		blkDev_DICT		= multipathInfo_DICT[ wwid ]
		sorted_blkDevs	= blkDev_DICT.keys()
		sorted_blkDevs.sort()
		for blkDev in sorted_blkDevs:
			multipathBlkDevInfo	= blkDev_DICT[ blkDev ]

			pathPortName		= ''
			pathSCSIdevID		= storage_tech.gen_SCSI_device_ID_OBJ_from_str( multipathBlkDevInfo.mSCSIdevIDstr )
			if pathSCSIdevID:
				pathPortNamePath	= "/sys/class/fc_transport/target%d:%d:%d/port_name" % (
											pathSCSIdevID.mHost
										,	pathSCSIdevID.mChannel
										,	pathSCSIdevID.mSCSIid )
				if os.path.exists( pathPortNamePath ):
					pathPortName, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, pathPortNamePath )
					if read_failed:
						pathPortName	= ''
				else:
					openISCSIstate	= blk_dev_to_openiscsi_session_info_DICT.get( multipathBlkDevInfo.mBlkDev, None )
					if openISCSIstate:
						pathPortName	= "%s,%d" % ( openISCSIstate.mIQN, openISCSIstate.mTpgt )
					else:
						SAS_SAN_device_state = blk_dev_to_SAS_state_DICT.get(multipathBlkDevInfo.mBlkDev,None)
						if SAS_SAN_device_state:
							pathPortName	="%s" % (SAS_SAN_device_state.mHBAAddress)

			print "mpDevNum: '%s,%s' mpUUID: '%s' pathSCSIgenID: '%s' pathDevNum: '%s' pathDevName: '%s' pathPortName: '%s' dmMapState: '%s' ckrState: '%s'" % (
						device_mapper_major_num
					,	multipathBlkDevInfo.mDevMapNum
					,	multipathBlkDevInfo.mWWID
					,	multipathBlkDevInfo.mSCSIdevIDstr
					,	multipathBlkDevInfo.mDevNums
					,	multipathBlkDevInfo.mBlkDev
					,	pathPortName
					,	multipathBlkDevInfo.mPathDMmapState
					,	multipathBlkDevInfo.mCheckerState
					)

####################################################################################################
def task_dm_walk_debug( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
	dmMapGraph_DICT, blkDevParentDMmap_DICT	= storage_tech.get_device_mapper_map_graph( vi_errorAndLog_obj, False )

	multipathInfo_DICT						= storage_tech.get_multipath_info( vi_errorAndLog_obj, False )
	pvInfo_uuid_DICT, vgInfo_uuid_DICT		= storage_tech.get_lvm_info( vi_errorAndLog_obj, False )

	rootMap_cmpldRegEx	= re.compile( "^(\S+)\s+\((\d+):(\d+)\)\s+\[(\S*)]$" )
	lineSub1_cmpldRegEx	= re.compile( "^ \`- \(" )
	lineSub2_cmpldRegEx	= re.compile( " .- \(" )
	lineSub3_cmpldRegEx	= re.compile( " \|-" )
	lineSub4_cmpldRegEx	= re.compile( " \| " )
	lineSub5_cmpldRegEx	= re.compile( " \`-" )

	cmdList						= [ 'dmsetup', 'ls', '--tree', '-o', 'device,ascii,notrunc,uuid' ]
	stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmdList )
	orderedRootMap_LIST			= []
	originalGraphOutput_LIST	= []
	for idx, line in enumerate( stdout_LIST ):
		rootMap_match	= rootMap_cmpldRegEx.match( line )
		if rootMap_match:
			orderedRootMap_LIST.append( rootMap_match.group(1) )
		line	= lineSub1_cmpldRegEx.sub( '  (', line )
		line	= lineSub2_cmpldRegEx.sub( ' (', line )
		line	= lineSub3_cmpldRegEx.sub( '  ', line )
		line	= lineSub4_cmpldRegEx.sub( '   ', line )
		line	= lineSub5_cmpldRegEx.sub( '  ', line )
		originalGraphOutput_LIST.append( line.rstrip() )

#	print '\n'.join( originalGraphOutput_LIST )

	ingestedGraphOutput_LIST	= []
	for orderedRootMap in orderedRootMap_LIST:
		rootDMmap	= dmMapGraph_DICT.get( orderedRootMap, None )
		if rootDMmap:
			ingestedGraphOutput_LIST.extend( rootDMmap.resursive_str(0,True).split( '\n' ) )
		else:
			print "MISSING: '%s'" % ( orderedRootMap )

#	print '\n'.join( ingestedGraphOutput_LIST )

	theDiffer			= difflib.Differ()
	diffResult_LIST		= list( theDiffer.compare( originalGraphOutput_LIST, ingestedGraphOutput_LIST ) )
	idx_diff_TPL_LIST	= []
	for idx, diffResult_line in enumerate(diffResult_LIST):
		if diffResult_line[0] != '-' and diffResult_line[0] != '+':
			continue
		idx_diff_TPL_LIST.append( ( idx, diffResult_line.rstrip() ) )
	if len(idx_diff_TPL_LIST) > 0:
		if 1:
			for idx, diffResult_line in enumerate(diffResult_LIST):
				print "%d] %s" % ( idx, diffResult_line.rstrip() )
		print "DIFFs: [%d]" % ( len(idx_diff_TPL_LIST) )
		for idx, diffResult_line in idx_diff_TPL_LIST:
			print "%4d] %s" % ( idx, diffResult_line )
	else:
		print 'get_device_mapper_map_graph() inSyncWith: ', ' '.join( cmdList )

	print ''

	if 1:
		for blkDevParentDMmapName, blkDevParentDMmap in blkDevParentDMmap_DICT.iteritems():
			print "%s %s" % ( blkDevParentDMmapName, blkDevParentDMmap.mDep_LIST )
			for lvmUsing in blkDevParentDMmap.mLVMusing_LIST:
				print "    %s" % ( lvmUsing.mMapName )
		print ''

	for blkDevParentDMmapName, blkDevParentDMmap in blkDevParentDMmap_DICT.iteritems():
		blkDev_DICT	= multipathInfo_DICT.get( blkDevParentDMmapName, None )
		if not blkDev_DICT:
			print "Missing from get_multipath_info(): wwid '%s'" % ( blkDevParentDMmapName )
			continue
		for dep in blkDevParentDMmap.mDep_LIST:
			mpBlkDevInfo	= blkDev_DICT.get( dep.mBlkDevName, None )
			if not mpBlkDevInfo:
				print "Missing from get_multipath_info(): wwid '%s' blkDev '%s'" % ( blkDevParentDMmapName, dep.mBlkDevName )
	for wwid, blkDev_DICT in multipathInfo_DICT.iteritems():
		blkDevParentDMmap	= blkDevParentDMmap_DICT.get( wwid, None )
		if not blkDevParentDMmap:
			print "Missing from get_device_mapper_map_graph(): wwid '%s'" % ( wwid )
			continue
		for blkDevName, mpBlkDevInfo in blkDev_DICT.iteritems():
			found	= False
			for dep in blkDevParentDMmap.mDep_LIST:
				if dep.mBlkDevName == blkDevName:
					found	= True
					break
			if not found:
				print "Missing from get_device_mapper_map_graph(): wwid '%s' blkDev '%s'" % ( wwid, blkDevName )
				continue

	for vg_uuid, vg_info in vgInfo_uuid_DICT.iteritems():
		for lv_name, lv_info in vg_info.mLVinfo_name_DICT.iteritems():
			lvMapName					= "%s-%s" % ( vg_info.mName, lv_name )
			lvDevPath					= "/dev/%s/%s" % ( vg_info.mName, lv_name )
			foundBlkDevParentDMmap_LIST	= []
			for blkDevParentDMmapName, blkDevParentDMmap in blkDevParentDMmap_DICT.iteritems():
				for lvmUsing in blkDevParentDMmap.mLVMusing_LIST:
					if lvMapName == lvmUsing.mMapName:
						foundBlkDevParentDMmap_LIST.append( blkDevParentDMmap )
						break
			for blkDevParentDMmap in foundBlkDevParentDMmap_LIST:
				pv_info	= vg_info.mPVinfo_wwid_DICT.get( blkDevParentDMmap.mMapName, None )
				if not pv_info:
					print "Missing from get_lvm_info(%s): wwid '%s'" % ( lvMapName, blkDevParentDMmapName.mMapName )
					continue
			pvCount	= len( vg_info.mPVinfo_wwid_DICT )
			for wwid, pv_info in vg_info.mPVinfo_wwid_DICT.iteritems():
				found	= False
				for blkDevParentDMmap in foundBlkDevParentDMmap_LIST:
					if wwid == blkDevParentDMmap.mMapName:
						found	= True
						break
				if not found:	# This WILL happen when a VG has multiple PVs BUT the LV does not span all of them
					cmdList	= [ 'lvs', lvDevPath, '--options=seg_count', '--noheadings' ]
					stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmdList )
					try:
						lvSegCount	= int(stdout_LIST[0].strip())
#						print "wwid '%s' lv '%s' :: %d %d" % ( wwid, lvMapName, lvSegCount, pvCount )
					except:
						lvSegCount	= pvCount
					if lvSegCount == pvCount:
						print "Missing from get_device_mapper_map_graph(): wwid '%s' lv '%s'" % ( wwid, lvMapName )
					continue

####################################################################################################
def task_discover_disk_cache_debug__0( vi_errorAndLog_obj ):
	"""
	This routine
	"""
#
	cmd_LIST	= [ 'storage_discover', '--cacheDirective=no_cache_change', '--infoDumpType=disk_blkdev_list' ]

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, 10 )

	if stdout_LIST != None and len(stdout_LIST) > 0:
		for line in stdout_LIST:
			print line,

	if stderr_LIST != None and len(stderr_LIST) > 0:
		for line in stderr_LIST:
			print line,

####################################################################################################
def task_discover_disk_cache_debug__1( vi_errorAndLog_obj, doAddAllBlockData ):
	"""
	This routine
	"""
#
	if doAddAllBlockData:
		cmd_LIST	= [ 'storage_discover', '--cacheDirective=add_all_block_data', '--infoDumpType=block_data_XML' ]
	else:
		cmd_LIST	= [ 'storage_discover', '--cacheDirective=no_cache_change', '--infoDumpType=block_data_XML' ]

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, 30 )

	if stdout_LIST != None and len(stdout_LIST) > 0:
		for line in stdout_LIST:
			print line,

	if stderr_LIST != None and len(stderr_LIST) > 0:
		for line in stderr_LIST:
			print line,

####################################################################################################
def task_discover_disk_cache_debug__2( vi_errorAndLog_obj, doAddAllBlockData ):
	"""
	This routine
	"""
#
	if doAddAllBlockData:
		cmd_LIST	= [ 'storage_discover', '--cacheDirective=add_all_block_data', '--infoDumpType=disk_list' ]
	else:
		cmd_LIST	= [ 'storage_discover', '--cacheDirective=no_cache_change', '--infoDumpType=disk_list' ]

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, 10 )
	if not stdout_LIST or len(stdout_LIST) < 1:
		return

	cmd_LIST	= [ 'storage_discover', '--skipLVM', '--cacheDirective=ensure_specific_disk_and_lvm_data', '--infoDumpType=disk_and_lvm_data_XML', '--diskList=<<STDIN>>' ]

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, 40, stdout_LIST )

	if stdout_LIST != None and len(stdout_LIST) > 0:
		for line in stdout_LIST:
			print line,

	if stderr_LIST != None and len(stderr_LIST) > 0:
		for line in stderr_LIST:
			print line,

####################################################################################################
def task_discover_disk_cache_debug__3( vi_errorAndLog_obj, doAddAllBlockData ):
	"""
	This routine
	"""
#
	if doAddAllBlockData:
		cmd_LIST	= [ 'storage_discover', '--cacheDirective=add_all_block_data', '--infoDumpType=disk_list' ]
	else:
		cmd_LIST	= [ 'storage_discover', '--cacheDirective=no_cache_change', '--infoDumpType=disk_list' ]

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, 10 )
	if not stdout_LIST or len(stdout_LIST) < 1:
		return

	cmd_LIST	= [ 'storage_discover', '--cacheDirective=ensure_specific_disk_and_lvm_data', '--infoDumpType=disk_and_lvm_data_XML', '--diskList=<<STDIN>>' ]

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, 60, stdout_LIST )

	if stdout_LIST != None and len(stdout_LIST) > 0:
		for line in stdout_LIST:
			print line,

	if stderr_LIST != None and len(stderr_LIST) > 0:
		for line in stderr_LIST:
			print line,

####################################################################################################
def task_discover_disk_cache_debug( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
	try:
		theValueAsInt	= int(theValue)
	except:
		theValueAsInt	= 0

	if theValueAsInt < 0:
		doAddAllBlockData	= True
		theValueAsInt		= abs(theValueAsInt)
	else:
		doAddAllBlockData	= False

	if theValueAsInt == 1:
		return task_discover_disk_cache_debug__1( vi_errorAndLog_obj, doAddAllBlockData )

	if theValueAsInt == 2:
		return task_discover_disk_cache_debug__2( vi_errorAndLog_obj, doAddAllBlockData )

	if theValueAsInt == 3:
		return task_discover_disk_cache_debug__3( vi_errorAndLog_obj, doAddAllBlockData )

	return task_discover_disk_cache_debug__0( vi_errorAndLog_obj )

####################################################################################################
def task_getDiskSizeInBytes( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
	dm_maps_DICT		= storage_tech.get_device_mapper_maps( vi_errorAndLog_obj, False )
	dm_map				= dm_maps_DICT.get( theValue, None )
	if not dm_map:
		return

	if len( dm_map.mDep_LIST ) == 0:
		return

	blkdev				= dm_map.mDep_LIST[ 0 ].mBlkDevName

	blkdev_info_path	= "/sys/block/%s/size" % ( blkdev )
	blkdev_sizeInSectors, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, blkdev_info_path )
	if read_failed:
		return

	blkdev_sizeInBytes	= int( blkdev_sizeInSectors ) * 512

	vi_errorAndLog_obj.vi_print_std( "%s" % ( blkdev_sizeInBytes ) )

####################################################################################################
def task_getVolGrpFreeSizeInBytes( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
	pvInfo_uuid_DICT, vgInfo_uuid_DICT	= storage_tech.get_lvm_info( vi_errorAndLog_obj, False )

	for vgInfo in vgInfo_uuid_DICT.values():
		if vgInfo.mName != theValue:
			continue

		vi_errorAndLog_obj.vi_print_std( "%s" % ( vgInfo.mFreeSizeInBytes ) )
		break

####################################################################################################
def task_getLogVolSizeInBytes( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
	try:
		volGrp, logVol	= theValue.split( '-' )
	except ValueError:
		return

	pvInfo_uuid_DICT, vgInfo_uuid_DICT	= storage_tech.get_lvm_info( vi_errorAndLog_obj, False )

	for vgInfo in vgInfo_uuid_DICT.values():
		if vgInfo.mName != volGrp:
			continue

		lvInfo	= vgInfo.mLVinfo_name_DICT.get( logVol, None )
		if not lvInfo:
			break

		vi_errorAndLog_obj.vi_print_std( "%s" % ( lvInfo.mSizeInBytes ) )
		break

####################################################################################################
def parse_given_args():
	"""
	This routine
	"""
#
	task_no_arg_LIST	=	[	Task_OBJ(	'listDisks'
										,	'list disk WWIDs'
										,	task_listDisks
										)
							,	Task_OBJ(	'mpa_walk'
										,	'dump multipath -ll info in a machine readable format'
										,	task_mpa_walk
										)
							,	Task_OBJ(	'dm_walk_debug'
										,	'dump dmsetup ls --tree debug info'
										,	task_dm_walk_debug
										)
							]

	task_with_arg_LIST	=	[	Task_OBJ(	'getDiskSizeInBytes'
										,	'given a WWID output the disk size in bytes'
										,	task_getDiskSizeInBytes
										)
							,	Task_OBJ(	'getVolGrpFreeSizeInBytes'
										,	'given a VolumeGroup ascertain its free size in bytes'
										,	task_getVolGrpFreeSizeInBytes
										)
							,	Task_OBJ(	'getLogVolSizeInBytes'
										,	'given a VolumeGroup-LogicalVolume pair ascertain its size in bytes'
										,	task_getLogVolSizeInBytes
										)
							,	Task_OBJ(	'discover_disk_cache_debug'
										,	'dump discover disk cache debug info :: 1=basic, 2=detailNoLVM, 3=detailWithLVM'
										,	task_discover_disk_cache_debug
										)
							]

	parser = OptionParser()

	parser.add_option("", "--debugLvl", dest="debugLvl", default=0, type="int",
					help="debugLvl message printing to stderr" )

	def store_value____task_no_arg_do( option, opt_str, value, parser ):
		taskIdx 		= int(option.dest)
		task			= task_no_arg_LIST[ taskIdx ]
		task.mDoScan	= True
		task.mValue		= value

	def store_value____task_with_arg_do( option, opt_str, value, parser ):
		taskIdx 		= int(option.dest)
		task			= task_with_arg_LIST[ taskIdx ]
		task.mDoScan	= True
		task.mValue		= value

	taskIdx = -1
	for task in task_no_arg_LIST:
		taskIdx = taskIdx + 1

		parser.add_option(	"", "--do_%s" % ( task.mArgNameStr ), dest=str(taskIdx), action="callback",
							callback=store_value____task_no_arg_do,  help="Do Task :: %s" % ( task.mHelpStr ) )

	taskIdx = -1
	for task in task_with_arg_LIST:
		taskIdx = taskIdx + 1

		parser.add_option(	"", "--do_%s" % ( task.mArgNameStr ), dest=str(taskIdx), action="callback", type="str",
							callback=store_value____task_with_arg_do,  help="Do Task :: %s" % ( task.mHelpStr ) )

	options_OBJ, args_LIST = parser.parse_args()

	tasksToPerform_LIST = []
	for task in task_no_arg_LIST:
		if task.mDoScan:
			tasksToPerform_LIST.append( task )
	for task in task_with_arg_LIST:
		if task.mDoScan:
			tasksToPerform_LIST.append( task )

	if 0:
		print >> sys.stderr, options_OBJ
		print >> sys.stderr, args_LIST
		print >> sys.stderr, dir(options_OBJ)
		print >> sys.stderr, parser.values
		print >> sys.stderr, tasksToPerform_LIST
		vi_utils.vi_dbg_stop()

	if len( tasksToPerform_LIST ) == 0:
		print >> sys.stderr, 'no work requested\n'
		parser.print_help(sys.stderr)
		print >> sys.stderr, ''
		vi_utils.vi_exit( 2 )

	return ( options_OBJ, args_LIST, tasksToPerform_LIST )

####################################################################################################
def init_complex_globals( options_OBJ, args_LIST ):
	"""
	This routine initializes and complex global variables
	"""
#
	global global_errAndLog

	errAndLog_dst_DICT =	{	'stdout'			:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__ALL
														,	sys.stdout
														)
							}

	global_errAndLog = vi_utils.VI_ErrAndLogRedirector_OBJ( errAndLog_dst_DICT, options_OBJ.debugLvl )

####################################################################################################
def main():
	"""
	This routine is the script entry point
	"""
#
	options_OBJ, args_LIST, tasksToPerform_LIST = parse_given_args()

	init_complex_globals( options_OBJ, args_LIST )

	for task in tasksToPerform_LIST:
		try:
			task.mFcn( global_errAndLog, options_OBJ, task.mValue )
		except:
			global_errAndLog.vi_croak_exception( traceback.format_exc() )

	return 0

####################################################################################################
### main

sys.stdout = os.fdopen( sys.stdout.fileno(), sys.stdout.mode, 0 )	# UNbuffer stdout

exitValue  = main()

sys.exit( exitValue )

# END
####################################################################################################
