#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for parsing a multipath.conf file
"""

import vi_utils

import os
import re
import traceback

from st_blockDeviceOperations	import *
from st_device			import *

####################################################################################################

__all__ = [	'AOEdisk_CurState_OBJ'
		,	'AOEdisk_OBJ'
		,	'AOE_BlkDev_Operations'
		,	'find_aoe_disks' ]

####################################################################################################

class AOEdisk_CurState_OBJ:

	def __init__(self):
		self.mDisk               = None
		self.mDiskShelf          = None
		self.mDiskSlot           = None
		self.mInterface          = None
		self.mInterfaceStatus    = None
		self.mBlkDev             = None
		self.mSizeInBytes        = None

	def __repr__(self):
		return "mDisk: %s mDiskShelf: %s mDiskSlot: %s mInterface: %s mInterfaceStatus: mBlkDev: %s %s mSizeInBytes: %s" % (
						self.mDisk
					,	self.mDiskShelf
					,	self.mDiskSlot
					,	self.mInterface
					,	self.mInterfaceStatus
					,	self.mBlkDev
					,	self.mSizeInBytes )

class AOEdisk_OBJ:

	def __init__(self):
		self.mDisk           = None
		self.mDiskShelf      = None
		self.mDiskSlot       = None
		self.mInterface      = None
		self.mBlkDev         = None
		self.mDevNums        = None
		self.mDevPath        = None
		self.mWWID           = None
		self.mSerialNum      = None
		self.mFirmwareRev    = None
		self.mModel          = None
		self.mCurState       = None # AOEdisk_CurState_OBJ

	def __repr__(self):
		return "mDisk: %s mDiskShelf: %s mDiskSlot: %s mInterface: %s mBlkDev: %s mDevNums: %s mDevPath: %s mWWID: %s mSerialNum: %s mFirmwareRev: %s mModel: %s mCurState: %s" % (
						self.mDisk
					,	self.mDiskShelf
					,	self.mDiskSlot
					,	self.mInterface
					,	self.mBlkDev
					,	self.mDevNums
					,	self.mDevPath
					,	self.mWWID
					,	self.mSerialNum
					,	self.mFirmwareRev
					,	self.mModel
					,	self.mCurState )

####################################################################################################
class AOE_BlkDev_Operations(BlkDev_Operations):

	"""AOE class for BlkDev_Operations
	"""

	########################################
	def __init__( self, blkDevInfo, aoe_disk ):

		if blkDevInfo.mTransport != 'AOE':
			raise ValueError, "given blkDevInfo %s is not AOE [%s] !?!" % ( blkDevInfo.mBlkDev, blkDevInfo.mTransport )

		self.mBlkDevInfo	= blkDevInfo
		self.mAOEdisk		= aoe_disk

	########################################
	def __repr__( self ):

		return "AOE_BlkDev_Operations[%s]" % ( self.mBlkDevInfo.mBlkDev )

	########################################
	def disconnect_block_device( self, vi_errorAndLog_obj, forceDisconnection ):
		"""
		"""
	#
		vi_errorAndLog_obj.vi_print_wrn( "disconnect_block_device(%s) not implemented for AOE" % ( self.mBlkDevInfo.mBlkDev ) )
		return 0

	########################################
	def revalidate_block_device( self, vi_errorAndLog_obj ):
		"""
		Updates a standard block device to the latests sysfs values
		"""
	#
		blkdev_info_path = "/sys/block/%s" % ( self.mBlkDevInfo.mBlkdev )
		if not os.path.exists( blkdev_info_path ):
			return 1

		blkdev_info_path = "/sys/block/%s/dev" % ( self.mBlkDevInfo.mBlkdev )
		self.mBlkDevInfo.mDevNums, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, blkdev_info_path )
		if read_failed:
			vi_errorAndLog_obj.vi_print_wrn( "blkdev '%s' vanished [%s] !?!" % ( self.mBlkDevInfo.mBlkdev, blkdev_info_path ) )
			return 1

		blkdev_info_path = "/sys/block/%s/size" % ( self.mBlkDevInfo.mBlkdev )
		blkdev_sizeInSectors, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, blkdev_info_path )
		if read_failed:
			vi_errorAndLog_obj.vi_print_wrn( "blkdev '%s' vanished [%s] !?!" % ( self.mBlkDevInfo.mBlkdev, blkdev_info_path ) )
			return 1

		self.mBlkDevInfo.mCurState.mSizeInBytes	= int( blkdev_sizeInSectors ) * 512

		return 0

####################################################################################################

global__cached__aoe_disk_LIST		= None

####################################################################################################
def find_aoe_disks( vi_errorAndLog_obj, refresh ):
	"""
	This routine
	"""
#
	global global__cached__aoe_disk_LIST
	if not refresh and global__cached__aoe_disk_LIST:
		return global__cached__aoe_disk_LIST

	aoe_disk_LIST = []
	aoe_major = get_block_device_major_num( vi_errorAndLog_obj, 'aoe', False )
	if aoe_major <= 0:
		return []

	try:
		aoe_stat_stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, 'aoe-stat', 20 )
		del( stderr_LIST )
		for aoe_stat_line in aoe_stat_stdout_LIST:
			aoe_stat_line_components     = aoe_stat_line.split()
			aoe_disk_name                = aoe_stat_line_components[0]
			aoe_blkdev                   = "etherd!%s" % ( aoe_disk_name )
			aoe_devPath                  = "/dev/etherd/%s" % ( aoe_disk_name )
			aoe_disk_cut_idx             = aoe_disk_name.index( '.' )
			aoe_disk_shelf               = aoe_disk_name[ 1 : aoe_disk_cut_idx ]
			aoe_disk_slot                = aoe_disk_name[ aoe_disk_cut_idx+1 : ]
			aoe_disk_interface           = aoe_stat_line_components[2]
			aoe_disk_interface_status    = aoe_stat_line_components[4]

			stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, [ 'ata_id', '--export', aoe_devPath ], 10 )

			serial_cmpldRegEx	= re.compile( '^ID_SERIAL=\s*' )
			rev_cmpldRegEx		= re.compile( '^ID_REVISION=\s*' )
			model_cmpldRegEx	= re.compile( '^ID_MODEL=\s*' )

			aoe_diskInfo_LIST = [ None, None, None ]
			for ata_id_line in stdout_LIST:
				ata_id_line = ata_id_line.strip()
				if serial_cmpldRegEx.match( ata_id_line ):
					aoe_diskInfo_LIST[0] = serial_cmpldRegEx.sub( '', ata_id_line )
					continue
				if rev_cmpldRegEx.match( ata_id_line ):
					aoe_diskInfo_LIST[1] = rev_cmpldRegEx.sub( '', ata_id_line )
					continue
				if model_cmpldRegEx.match( ata_id_line ):
					aoe_diskInfo_LIST[2] = model_cmpldRegEx.sub( '', ata_id_line )
					continue

			if not aoe_diskInfo_LIST[0] or not aoe_diskInfo_LIST[1] or not aoe_diskInfo_LIST[2]:
				vi_errorAndLog_obj.vi_print_err( "MISSING AOE data :", aoe_diskInfo_LIST )

			aoe_disk_wwid	= "AOE_%s" % ( aoe_diskInfo_LIST[0] )

			blkdev_info_path = "/sys/block/%s/dev" % ( aoe_blkdev )
			aoe_disk_devNums, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, blkdev_info_path )
			if read_failed:
				vi_errorAndLog_obj.vi_print_wrn( "blkdev '%s' vanished [%s] !?!" % ( aoe_blkdev, blkdev_info_path ) )
				continue

			blkdev_info_path = "/sys/block/%s/size" % ( aoe_blkdev )
			aoe_disk_sizeInSectors, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, blkdev_info_path )
			if read_failed:
				vi_errorAndLog_obj.vi_print_wrn( "blkdev '%s' vanished [%s] !?!" % ( aoe_blkdev, blkdev_info_path ) )
				continue

			aoe_disk_sizeInBytes = int(aoe_disk_sizeInSectors) * 512

			aoe_disk_curState                     = AOEdisk_CurState_OBJ()
			aoe_disk_curState.mDisk               = aoe_disk_name
			aoe_disk_curState.mDiskShelf          = aoe_disk_shelf
			aoe_disk_curState.mDiskSlot           = aoe_disk_slot
			aoe_disk_curState.mInterface          = aoe_disk_interface
			aoe_disk_curState.mInterfaceStatus    = aoe_disk_interface_status
			aoe_disk_curState.mBlkDev             = aoe_blkdev
			aoe_disk_curState.mSizeInBytes        = aoe_disk_sizeInBytes

			aoe_disk                 = AOEdisk_OBJ()
			aoe_disk.mDisk           = aoe_disk_name
			aoe_disk.mDiskShelf      = aoe_disk_shelf
			aoe_disk.mDiskSlot       = aoe_disk_slot
			aoe_disk.mInterface      = aoe_disk_interface
			aoe_disk.mBlkDev         = aoe_blkdev
			aoe_disk.mDevNums        = aoe_disk_devNums
			aoe_disk.mDevPath        = aoe_devPath
			aoe_disk.mWWID           = aoe_disk_wwid
			aoe_disk.mSerialNum      = aoe_diskInfo_LIST[0]
			aoe_disk.mFirmwareRev    = aoe_diskInfo_LIST[1]
			aoe_disk.mModel          = aoe_diskInfo_LIST[2]
			aoe_disk.mCurState       = aoe_disk_curState

			aoe_disk_LIST.append( aoe_disk )
#			vi_errorAndLog_obj.vi_print_dbg( 0, aoe_disk )

	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Problems extracting AOE disk info\n%s" % ( traceback.format_exc() ) )
		return []

	if 0:
		vi_errorAndLog_obj.vi_print_dbg( 0, aoe_disk_LIST )

	global__cached__aoe_disk_LIST = aoe_disk_LIST
	return global__cached__aoe_disk_LIST

####################################################################################################
# END
