#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas, Konrad Rzeszutek
#
"""Contains the SCSI device ID and utilities
"""

import re

####################################################################################################

__all__ = [	'SCSI_device_ID_OBJ'
	,	'gen_SCSI_device_ID_OBJ_from_str' ]

####################################################################################################

class SCSI_device_ID_OBJ:

	def __init__( self ):
		self.mHost    = None
		self.mChannel = None
		self.mSCSIid  = None
		self.mLUN     = None

	def is_valid( self ):
		if type(self.mHost)    == None: return False
		if type(self.mChannel) == None: return False
		if type(self.mSCSIid)  == None: return False
		if type(self.mLUN)     == None: return False
		return True

	def as_str( self ):
		return "%s:%s:%s:%s" % (
						self.mHost
					,	self.mChannel
					,	self.mSCSIid
					,	self.mLUN )

	def __repr__(self):
		return self.as_str()

	def __cmp__(self, other):
		if self.mHost > other.mHost:
			return   1
		if self.mHost < other.mHost:
			return  -1

		if self.mChannel > other.mChannel:
			return   1
		if self.mChannel < other.mChannel:
			return  -1

		if self.mSCSIid > other.mSCSIid:
			return   1
		if self.mSCSIid < other.mSCSIid:
			return  -1

		if self.mLUN > other.mLUN:
			return   1
		if self.mLUN < other.mLUN:
			return  -1

		return 0

####################################################################################################
def gen_SCSI_device_ID_OBJ_from_str ( str ):
	"""
	Construct a SCSI_device_ID_OBJ based on the Host:Channel:SCSIID:LUN string.
	For example: "17:0:0:431" would be a valid entry.

	Returns:
	None if the entry was invalid.
	SCSI_device_ID_OBJ if entry was valid.
	"""

	regExp_match = re.match("^(\d+):(\d+):(\d+):(\d+)$", str)
	if regExp_match:
		mSCSI_device_id			= SCSI_device_ID_OBJ()
		mSCSI_device_id.mHost 		= int(regExp_match.group(1))
		mSCSI_device_id.mChannel 	= int(regExp_match.group(2))
		mSCSI_device_id.mSCSIid 	= int(regExp_match.group(3))
		mSCSI_device_id.mLUN 		= int(regExp_match.group(4))
		return mSCSI_device_id

	return None

####################################################################################################
# END
