#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Script for managing storage_discover operations
"""

import sys
import os

sys.path.append('..')

import vi_utils
import vi_xml
import storage_tech
import storage_discover

from optparse import OptionParser
from optparse import OptionGroup

####################################################################################################

global_errAndLog	= None		# VI_ErrAndLog_OBJ

####################################################################################################
def parse_given_args():
	"""
	This routine
	"""
#
	cacheDirective_DICT = {	'no_cache_change'					: storage_discover.no_cache_change
						,	'delete_entire_cache'				: storage_discover.delete_entire_cache
						,	'add_all_block_data'				: storage_discover.add_all_block_data
						,	'add_specific_block_data'			: storage_discover.add_specific_block_data
						,	'remove_specific_block_data'		: storage_discover.remove_specific_block_data
						,	'update_cached_block_data'			: storage_discover.update_cached_block_data # Will NOT add or remove blkDevs
						,	'ensure_specific_disk_and_lvm_data'	: storage_discover.ensure_specific_disk_and_lvm_data
						}

	def store_value____ck_cacheDirective( option, opt_str, value, parser ):
		if value not in cacheDirective_DICT.keys():
			print >> sys.stderr, cacheDirective_DICT.keys()
			print >> sys.stderr,  "'%s' is NOT a valid mode:    %s\n" % ( value, ' '.join(cacheDirective_DICT.keys()) )
			parser.print_help(sys.stderr)
			vi_utils.vi_exit( 1 )
		setattr( parser.values, option.dest, value )

	infoDumpType_DICT = {	'no_output'					: storage_discover.no_output
						,	'disk_list'					: storage_discover.disk_list
						,	'disk_blkdev_list'			: storage_discover.disk_blkdev_list
						,	'disk_blkdev_diff_list'		: storage_discover.disk_blkdev_diff_list
						,	'lvm_list'					: storage_discover.lvm_list
						,	'block_data_XML'			: storage_discover.block_data_XML
						,	'disk_and_lvm_data_XML'		: storage_discover.disk_and_lvm_data_XML
						,	'scan_vdisk_data_XML'		: storage_discover.scan_vdisk_data_XML
						,	'disk_XML'	        	: storage_discover.disk_XML
						,	'io_host_XML'   		: storage_discover.io_host_XML
						}

	def store_value____ck_infoDumpType( option, opt_str, value, parser ):
		if value not in infoDumpType_DICT.keys():
			print >> sys.stderr, infoDumpType_DICT.keys()
			print >> sys.stderr, "'%s' is NOT a valid mode:    %s\n" % ( value, ' '.join(infoDumpType_DICT.keys()) )
			parser.print_help(sys.stderr)
			vi_utils.vi_exit( 1 )
		setattr( parser.values, option.dest, value )

	parser = OptionParser()

#	parser.add_option( "-v", "--verbose", dest="verbose", default=True, action="store_false",
#					help="verbose message printing to stdout" )

	parser.add_option("", "--debugLvl", dest="debugLvl", default=0, type="int",
					help="debugLvl message printing to stderr" )

	parser.add_option( "", "--skipLVM", dest="skipLVM", default=False, action="store_true",
					help="Use to prevent LVM operations" )

	parser.add_option("", "--blkDevList", dest="blkDevList", default=None, type="string",
					help="blkDevList:  blkDev1,...,blkDevN  OR  <<STDIN>>  to read in the list from stdin" )

	parser.add_option("", "--diskList", dest="diskList", default=None, type="string",
					help="diskList:  disk1,...,diskN  OR  <<STDIN>>  to read in the list from stdin" )

	parser.add_option("", "--lvmList", dest="lvmList", type="string",
					help="lvmList: volumeGroup1[_logicalVol1],...,volumeGroupN[_logicalVolN]  OR  <<STDIN>>  to read in the list from stdin" )

	parser.add_option( "", "--vDiskMountScan", dest="doVDiskMntScan", default=False, action="store_true",
					help="Do VDisk mount scan for given lvmList instead of just scanning for scan_vdisk_data_XML" )

	group = OptionGroup( parser, "Disk Information Cache Directives", "%s" % '    '.join(cacheDirective_DICT.keys()) )

	group.add_option( "", "--cacheDirective", dest="cacheDirective", action="callback", callback=store_value____ck_cacheDirective,
					type="string",  # HACKISHLY required to get metavar to show in help AND helps in ck_mode
					default='add_all_block_data',
					help="", metavar='DIRECTIVE' )

	group.add_option("", "--itemToUpdate", dest="itemToUpdate", default=0, type="string",
					help="for use with update_specific_... cacheDirectives" )

	parser.add_option_group( group )

	group = OptionGroup( parser, "Information Dump Type", "%s" % '    '.join(infoDumpType_DICT.keys()) )

	group.add_option( "", "--infoDumpType", dest="infoDumpType", action="callback", callback=store_value____ck_infoDumpType,
					type="string",  # HACKISHLY required to get metavar to show in help AND helps in ck_mode
					default='no_output',
					help="", metavar='DIRECTIVE' )

	group.add_option("", "--indentXMLlvl", dest="indentXMLlvl", default=0, type="int",
					help="indent XML level" )

	parser.add_option_group( group )

	options_OBJ, args_LIST = parser.parse_args()

	item_LIST = []
	if options_OBJ.blkDevList == '<<STDIN>>' or options_OBJ.diskList == '<<STDIN>>' or options_OBJ.lvmList == '<<STDIN>>':
		if options_OBJ.debugLvl >= 4:
			print >> sys.stderr, 'Reading list from stdin'
		try:
			while True:
				stdinInputLine = raw_input()
				if stdinInputLine:
					stdinInputLine	= stdinInputLine.strip()
					if len(stdinInputLine):
						item_LIST.append( stdinInputLine )
		except EOFError:
			pass
	else:
		if options_OBJ.blkDevList:
			for givenItem in options_OBJ.blkDevList.split(','):
				givenItem	= givenItem.strip()
				if len(givenItem):
					item_LIST.append( givenItem )
		if options_OBJ.diskList:
			for givenItem in options_OBJ.diskList.split(','):
				givenItem	= givenItem.strip()
				if len(givenItem):
					item_LIST.append( givenItem )
		if options_OBJ.lvmList:
			for givenItem in options_OBJ.lvmList.split(','):
				givenItem	= givenItem.strip()
				if len(givenItem):
					item_LIST.append( givenItem )

	vi_utils.vi_sort_uniq( item_LIST, True )

	cacheDirectiveFcn = cacheDirective_DICT[ options_OBJ.cacheDirective ]
	infoDumpTypeFcn = infoDumpType_DICT[ options_OBJ.infoDumpType ]

	if options_OBJ.debugLvl >= 9:
		print >> sys.stderr, options_OBJ
		print >> sys.stderr, args_LIST
		print >> sys.stderr, dir(options_OBJ)
		print >> sys.stderr, parser.values
		print >> sys.stderr, 'Items:', item_LIST
		print >> sys.stderr, cacheDirectiveFcn
		print >> sys.stderr, infoDumpTypeFcn
		vi_utils.vi_dbg_stop()

	if options_OBJ.blkDevList and options_OBJ.diskList:
		print >> sys.stderr, ''
		print >> sys.stderr, 'blkDevList and diskList both specified?!?\n'
		parser.print_help( sys.stderr )
		print >> sys.stderr, ''
		vi_utils.vi_exit( 2 )

	if options_OBJ.blkDevList and options_OBJ.lvmList:
		print >> sys.stderr, ''
		print >> sys.stderr, 'blkDevList and lvmList both specified?!?\n'
		parser.print_help( sys.stderr )
		print >> sys.stderr, ''
		vi_utils.vi_exit( 2 )

	if options_OBJ.lvmList and options_OBJ.diskList:
		print >> sys.stderr, ''
		print >> sys.stderr, 'lvmList and diskList both specified?!?\n'
		parser.print_help( sys.stderr )
		print >> sys.stderr, ''
		vi_utils.vi_exit( 2 )

	if options_OBJ.cacheDirective == 'no_cache_change' and options_OBJ.infoDumpType == 'no_output':
		print >> sys.stderr, ''
		print >> sys.stderr, 'neither cacheDirective nor infoDumpType set?!?\n'
		parser.print_help( sys.stderr )
		print >> sys.stderr, ''
		vi_utils.vi_exit( 2 )

	return ( options_OBJ, args_LIST, item_LIST, cacheDirectiveFcn, infoDumpTypeFcn )

####################################################################################################
def init_complex_globals( options_OBJ, args_LIST ):
	"""
	This routine initializes and complex global variables
	"""
#
	global global_errAndLog

	errAndLog_dst_DICT =	{	'stdErr'			:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__ERR
														|	vi_utils.VI_ErrAndLog__FOR_FLAG__WRN
														|	vi_utils.VI_ErrAndLog__FOR_FLAG__DBG
														,	sys.stderr
														)
							,	'stdout'			:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__LOG
														|	vi_utils.VI_ErrAndLog__FOR_FLAG__STD
														,	sys.stdout
														)
							,	'errAccumlation'	:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__ERR
														,	vi_utils.VI_PrintAccumulator_OBJ( 20, 10 )
														)
							,	'wrnAccumlation'	:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__WRN
														,	vi_utils.VI_PrintAccumulator_OBJ( 20, 10 )
														)
							}

	global_errAndLog = vi_utils.VI_ErrAndLogRedirector_OBJ( errAndLog_dst_DICT, options_OBJ.debugLvl )

	global_errAndLog.vi_set_on_crash_fcn( storage_discover.on_crash_dump_errors_XML, options_OBJ.indentXMLlvl )

####################################################################################################
def main():
	"""
	This routine is the script entry point
	"""
#
	options_OBJ, args_LIST, item_LIST, cacheDirectiveFcn, infoDumpTypeFcn = parse_given_args()

	init_complex_globals( options_OBJ, args_LIST )

	cacheDirectiveFcn( global_errAndLog, options_OBJ, args_LIST, item_LIST, infoDumpTypeFcn )

#	vi_xml.vi_flush_XML()
#	return 0

	if global_errAndLog.get_num_of_errors() < 1:
		vi_xml.vi_flush_XML()
		return 0

	global_errAndLog.vi_print_dbg( 0, '' )

	global_errAndLog.vi_call_on_crash_fcn( "Runtime errors occured [%s]" % ( global_errAndLog.get_num_of_errors() ), '' )
	return 1

####################################################################################################
### root of all evil

sys.stdout = os.fdopen( sys.stdout.fileno(), sys.stdout.mode, 0 )	# UNbuffer stdout

exitValue  = main()

sys.exit( exitValue )

# END
####################################################################################################
