#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for interacting with the disk info cache
"""

import vi_utils

import os
import re
import traceback
import pickle

from sd_blkDevInfoUtils		import *

####################################################################################################

__all__ = [	'write_blk_and_disk_cache__single_wwid'
		,	'write_blk_and_disk_cache__single_blk_dev'
		,	'write_blk_and_disk_cache__single_disk'
		,	'write_blk_and_disk_cache__vgs'
		,	'write_blk_and_disk_cache__missingVGs'
		,	'write_blk_and_disk_cache'
		,	'merge_and_write_lvm_cache'
		,	'remove_blk_devs_from_cache'
		,	'read_blk_and_disk_cache__single_wwid'
		,	'read_blk_and_disk_cache__single_blk_dev'
		,	'read_blk_and_disk_cache__single_disk'
		,	'read_blk_and_disk_cache__vgs'
		,	'read_blk_and_disk_cache__missingVGs'
		,	'read_all_blk_cache'
		,	'read_specific_blk_cache'
		,	'read_blk_and_disk_cache'
		,	'delete_blk_and_disk_cache_cache' ]

####################################################################################################

global_infoCachePath           = '/var/cache/ovs-disk_info'
global_infoCacheLockPath       = "%s/*fileLock*"   % ( global_infoCachePath )
global_infoCacheVGSpath        = "%s/*vgsInfo*"    % ( global_infoCachePath )
global_infoCacheMissingVGsPath = "%s/*missingVGs*" % ( global_infoCachePath )
global_infoCache_fileLock      = None

####################################################################################################

class OnDisk_DiskInfo_OBJ:

	def __init__( self, theWWID, theIsSCSIdevice ):
		self.mWWID         = theWWID
		self.mIsSCSIdevice = theIsSCSIdevice

	def __repr__( self ):
		return "mWWID: %s mIsSCSIdevice: %s" % ( self.mWWID, self.mIsSCSIdevice )

####################################################################################################
def lock_cache( vi_errorAndLog_obj, forWhomMsg, forWrite_BOOL ):
	"""
	This routine
	"""
#
	global global_infoCache_fileLock

	if not global_infoCache_fileLock:
		global_infoCache_fileLock = vi_utils.vi_fileLock( 'cache', global_infoCacheLockPath )
		if global_infoCache_fileLock.CreateLockFile():
			vi_errorAndLog_obj.vi_croak( "Failed to create cache lock !?!" )

	if global_infoCache_fileLock.AcquireLock( forWrite_BOOL ):
		vi_errorAndLog_obj.vi_croak( "Failed to acquire cache lock for %s !?!" % ( forWhomMsg ) )

#####################################################################################################
def is_cache_locked( vi_errorAndLog_obj ):
	"""
	This routine
	"""
#
	global global_infoCache_fileLock

	return global_infoCache_fileLock.IsLocked()

#####################################################################################################
def unlock_cache( vi_errorAndLog_obj ):
	"""
	This routine
	"""
#
	global global_infoCache_fileLock

	global_infoCache_fileLock.ReleaseLock()

#####################################################################################################
def write_blk_and_disk_cache__single_wwid( vi_errorAndLog_obj, wwid, wwid_path, diskInfo ):
	"""
	This routine
	"""
#
	if not is_cache_locked( vi_errorAndLog_obj ):
		vi_errorAndLog_obj.vi_croak( "cache is not locked" )

	wwidInfoPath = wwid_path + '/*info*'
	try:
		wwidInfo_FILE = open( wwidInfoPath, 'w' )
		pickle.dump( OnDisk_DiskInfo_OBJ( diskInfo.mWWID, diskInfo.mIsSCSIdevice ), wwidInfo_FILE )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() ) # WARNING: caller is holding lock
		vi_errorAndLog_obj.vi_print_err( "write failure '%s'\n%s" % ( blkdevPath, traceback.format_exc() ) )

####################################################################################################
def write_blk_and_disk_cache__single_blk_dev( vi_errorAndLog_obj, wwid, wwid_path, blkDevInfo ):
	"""
	This routine
	"""
#
	if not is_cache_locked( vi_errorAndLog_obj ):
		vi_errorAndLog_obj.vi_croak( "cache is not locked" )

	blkdevPath = wwid_path + '/' + blkDevInfo.mBlkDev
	try:
		blkDevInfo_FILE = open( blkdevPath, 'w' )
		pickle.dump( blkDevInfo, blkDevInfo_FILE )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() ) # WARNING: caller is holding lock
		vi_errorAndLog_obj.vi_print_err( "write failure '%s'\n%s" % ( blkdevPath, traceback.format_exc() ) )

####################################################################################################
def write_blk_and_disk_cache__single_disk( vi_errorAndLog_obj, wwid, diskInfo ):
	"""
	This routine
	"""
#
	if not is_cache_locked( vi_errorAndLog_obj ):
		vi_errorAndLog_obj.vi_croak( "cache is not locked" )

	wwid_path = global_infoCachePath + '/' + wwid
	try:
		if not os.path.exists( wwid_path ):
			try:
				os.mkdir( wwid_path )
			except:
#				vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
				vi_errorAndLog_obj.vi_print_err( "disk dir creation failed '%s' :: new disk info was NOT written !?!\n%s" % ( wwid_path, traceback.format_exc() ) )

		write_blk_and_disk_cache__single_wwid( vi_errorAndLog_obj, wwid, wwid_path, diskInfo )

		for blkDevInfo in diskInfo.mBlkDevInfo_LIST:
			write_blk_and_disk_cache__single_blk_dev( vi_errorAndLog_obj, wwid, wwid_path, blkDevInfo )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() ) # WARNING: caller is holding lock
		vi_errorAndLog_obj.vi_print_err( "write failure '%s'\n%s" % ( wwid_path, traceback.format_exc() ) )

####################################################################################################
def write_blk_and_disk_cache__vgs( vi_errorAndLog_obj, vgInfo_uuid_DICT ):
	"""
	This routine
	"""
#
	if not is_cache_locked( vi_errorAndLog_obj ):
		vi_errorAndLog_obj.vi_croak( "cache is not locked" )

	vgsInfo_FILE = None
	try:
		vgsInfo_FILE = open( global_infoCacheVGSpath, 'w' )
		pickle.dump( vgInfo_uuid_DICT, vgsInfo_FILE )
		vgsInfo_FILE.close()
	except:
		if vgsInfo_FILE:
			vgsInfo_FILE.close()
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() ) # WARNING: caller is holding lock
		vi_errorAndLog_obj.vi_print_err( "write failure '%s'\n%s" % ( global_infoCacheVGSpath, traceback.format_exc() ) )

####################################################################################################
def write_blk_and_disk_cache__missingVGs( vi_errorAndLog_obj, missingVGs_DICT ):
	"""
	This routine
	"""
#
	if not is_cache_locked( vi_errorAndLog_obj ):
		vi_errorAndLog_obj.vi_croak( "cache is not locked" )

	missingVGsInfo_FILE = None
	try:
		missingVGsInfo_FILE = open( global_infoCacheMissingVGsPath, 'w' )
		pickle.dump( missingVGs_DICT, missingVGsInfo_FILE )
		missingVGsInfo_FILE.close()
	except:
		if missingVGsInfo_FILE:
			missingVGsInfo_FILE.close()
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() ) # WARNING: caller is holding lock
		vi_errorAndLog_obj.vi_print_err( "write failure '%s'\n%s" % ( global_infoCacheMissingVGsPath, traceback.format_exc() ) )

####################################################################################################
def write_blk_and_disk_cache( vi_errorAndLog_obj, diskInfo_DICT ):
	"""
	This routine
	"""
#
	try:
		lock_cache( vi_errorAndLog_obj, 'write_blk_and_disk_cache', True )

		for wwid in diskInfo_DICT:
			diskInfo = diskInfo_DICT[ wwid ]
			write_blk_and_disk_cache__single_disk( vi_errorAndLog_obj, wwid, diskInfo )

		unlock_cache( vi_errorAndLog_obj )
	except:
		unlock_cache( vi_errorAndLog_obj )
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() ) # WARNING: caller is holding lock
		vi_errorAndLog_obj.vi_print_err( "write failure '%s'\n%s" % ( wwid, traceback.format_exc() ) )

####################################################################################################
def merge_and_write_lvm_cache( vi_errorAndLog_obj, vgInfo_uuid_DICT, missingVGs_DICT, disk_LIST ):
	"""
	This routine
	"""
#
	disk_DICT = {} # Create a DICT for faster searching
	last_wwid = ''
	for wwid in disk_LIST:
		if wwid == last_wwid: # list is sorted so this will skip duplicates
			continue
		last_wwid = wwid
		disk_DICT[ wwid ] = True

	try:
		lock_cache( vi_errorAndLog_obj, 'merge_and_write_lvm_cache', True )

		# Read the cached info
		cache_vgInfo_uuid_DICT = read_blk_and_disk_cache__vgs( vi_errorAndLog_obj )
		cache_missingVGs_DICT  = read_blk_and_disk_cache__missingVGs( vi_errorAndLog_obj )

#		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
#		vi_errorAndLog_obj.vi_print_dbg( 0, "GvgI: %s\n" % ( vgInfo_uuid_DICT ) )
#		vi_errorAndLog_obj.vi_print_dbg( 0, "CvgI: %s\n" % ( cache_vgInfo_uuid_DICT ) )
#		vi_errorAndLog_obj.vi_print_dbg( 0, "Gmvg: %s\n" % ( missingVGs_DICT ) )
#		vi_errorAndLog_obj.vi_print_dbg( 0, "Cmvg: %s\n" % ( cache_missingVGs_DICT ) )
#		vi_errorAndLog_obj.vi_stop( 'pre-merge' )

		# Only keep those cached enteries that are NOT in the disk_DICT ( as we should already has those in the given )
		merged_vgInfo_uuid_DICT = {}
		merged_missingVGs_DICT  = {}
		for vg_uuid, vg_info in cache_vgInfo_uuid_DICT.iteritems():
			for pv_wwid in vg_info.mPVinfo_wwid_DICT.keys():
				if disk_DICT.has_key( pv_wwid ):
					break
			else:
				merged_vgInfo_uuid_DICT[ vg_uuid ] = vg_info

		for vg_name, pvInfo_LIST in cache_missingVGs_DICT.iteritems():
			for pv_info in pvInfo_LIST:
				if disk_DICT.has_key( pv_info.mWWID ):
					break
			else:
				merged_missingVGs_DICT[ vg_name ] = pvInfo_LIST

		# Add new values to cached ones ( this merge is additive only )
		merged_vgInfo_uuid_DICT.update( vgInfo_uuid_DICT )
		merged_missingVGs_DICT.update( missingVGs_DICT )

#		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
#		vi_errorAndLog_obj.vi_print_dbg( 0, "MvgI: %s\n" % ( merged_vgInfo_uuid_DICT ) )
#		vi_errorAndLog_obj.vi_print_dbg( 0, "Mmvg: %s\n" % ( merged_missingVGs_DICT ) )
#		vi_errorAndLog_obj.vi_stop( 'merged' )

		# Write out merged result
		write_blk_and_disk_cache__vgs( vi_errorAndLog_obj, merged_vgInfo_uuid_DICT )
		write_blk_and_disk_cache__missingVGs( vi_errorAndLog_obj, merged_missingVGs_DICT )

		unlock_cache( vi_errorAndLog_obj )
	except:
		unlock_cache( vi_errorAndLog_obj ) # After close to ensure write flush
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "write failure '%s' '%s'\n%s" % ( global_infoCacheVGSpath, global_infoCacheMissingVGsPath, traceback.format_exc() ) )

####################################################################################################
def remove_blk_devs_from_cache( vi_errorAndLog_obj, removed_blkdev_DICT ):
	"""
	This routine
	"""
#
	diskInfo_DICT = read_all_blk_cache( vi_errorAndLog_obj )

	# Find the wwid for the given blkDev
	removed_blkdev_disk_DICT = {}
	is_done = False
	for wwid in diskInfo_DICT:
		diskInfo = diskInfo_DICT[ wwid ]
		for blkDevInfo in diskInfo.mBlkDevInfo_LIST:
			if removed_blkdev_DICT.has_key( blkDevInfo.mBlkDev ):
				if removed_blkdev_disk_DICT.has_key( wwid ):
					blkDevInfo_LIST = removed_blkdev_disk_DICT[ wwid ][ 3 ]
					blkDevInfo_LIST.append( blkDevInfo )
				else:
					wwid_path = global_infoCachePath + '/' + wwid
					removed_blkdev_disk_DICT[ wwid ] = ( wwid_path, [ blkDevInfo ] )
				del( removed_blkdev_DICT[ blkDevInfo.mBlkDev ] )
				if len( removed_blkdev_DICT ) < 1:
					is_done = True
				break
		if is_done:
			break

	if len( removed_blkdev_disk_DICT ) < 1:
		return

	deleted_disk_DICT       = {} # Create a DICT for faster searching
	edited_vgInfo_uuid_DICT = {}
	edited_missingVGs_DICT  = {}
	try:
		lock_cache( vi_errorAndLog_obj, 'remove_blk_devs_from_cache', True )

		# Perform removal
		for wwid in removed_blkdev_disk_DICT:
			wwid_path, blkDevInfo_LIST = removed_blkdev_disk_DICT[ wwid ]

			if vi_errorAndLog_obj.vi_should_print_dbg( 4 ):
				dbg_blkDevs_str = ''
				for blkDevInfo in blkDevInfo_LIST:
					if dbg_blkDevs_str:
						dbg_blkDevs_str = dbg_blkDevs_str + ' ' + blkDevInfo.mBlkDev
					else:
						dbg_blkDevs_str = blkDevInfo.mBlkDev
				vi_errorAndLog_obj.vi_print_dbg( 4, "Disk %s :: attempting to remove blkdevs: %s" % ( wwid, dbg_blkDevs_str ) )

			try:
				# Remove the blkDevInfo files
				for blkDevInfo in blkDevInfo_LIST:
					blkDevInfo_path = wwid_path + '/' + blkDevInfo.mBlkDev
					if os.path.exists( blkDevInfo_path ):
						try:
							vi_errorAndLog_obj.vi_print_dbg( 4, "Disk %s :: removing blkdevInfo '%s'" % ( wwid, blkDevInfo_path ) )
							os.unlink( blkDevInfo_path )
						except:
							vi_errorAndLog_obj.vi_print_err( "Could not remove blkdevInfo '%s'\n%s" % ( blkDevInfo_path, traceback.format_exc() ) )
					else:
						vi_errorAndLog_obj.vi_print_dbg( 4, "Disk %s :: blkdevInfo '%s' no longer exists - skipping" % ( wwid, blkDevInfo_path ) )

				# See if there are any blkDevInfo files left
				blkDevInfo = None
				for blkdev in os.listdir( wwid_path ):
					if blkdev[0] == '_' or blkdev[0] == '*':
						continue
					blkDevInfo = read_blk_and_disk_cache__single_blk_dev( vi_errorAndLog_obj, wwid, wwid_path, blkdev )
					if blkDevInfo:
						break

				# if there are then move on to the next deletion set
				if blkDevInfo:
					continue

				# otherwise delete the entire disk
				if vi_utils.vi_delete_directory_including_file_contents( wwid_path ) == 0:
					deleted_disk_DICT[ wwid ] = True
			except:
#				vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
				vi_errorAndLog_obj.vi_print_err( "BlkDev remove failure:\n%s" % ( traceback.format_exc() ) )

		# Only keep those VG enteries that are NOT in the deleted_disk_DICT ( subtractive merge )

		cache_vgInfo_uuid_DICT = read_blk_and_disk_cache__vgs( vi_errorAndLog_obj )
		for vg_uuid in cache_vgInfo_uuid_DICT:
			vg_info = cache_vgInfo_uuid_DICT[ vg_uuid ]
			for pv_wwid in vg_info.mPVinfo_wwid_DICT.keys():
				if deleted_disk_DICT.has_key( pv_wwid ):
					vg_uuid = None
					break
			if vg_uuid:
				edited_vgInfo_uuid_DICT[ vg_uuid ] = vg_info

		write_blk_and_disk_cache__vgs( vi_errorAndLog_obj, edited_vgInfo_uuid_DICT )

		cache_missingVGs_DICT = read_blk_and_disk_cache__missingVGs( vi_errorAndLog_obj )
		for vg_name, pvInfo_LIST in cache_missingVGs_DICT.iteritems():
			for pv_info in pvInfo_LIST:
				if deleted_disk_DICT.has_key( pv_info.mWWID ):
					continue
				pvInfo_LIST = edited_missingVGs_DICT.get( pv_info.mVGname, None )
				if not pvInfo_LIST:
					missingVGs_DICT[ pv_info.mVGname ] = [ pv_info ]
				else:
					pvInfo_LIST.append( pv_info )

		write_blk_and_disk_cache__missingVGs( vi_errorAndLog_obj, edited_missingVGs_DICT )

		unlock_cache( vi_errorAndLog_obj )
	except:
		unlock_cache( vi_errorAndLog_obj )
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "BlkDev remove failure:\n%s" % ( traceback.format_exc() ) )

####################################################################################################
def read_blk_and_disk_cache__single_wwid( vi_errorAndLog_obj, wwid, wwid_path ):
	"""
	This routine
	"""
#
	if not is_cache_locked( vi_errorAndLog_obj ):
		vi_errorAndLog_obj.vi_croak( "cache is not locked" )

	wwidInfoPath = wwid_path + '/*info*'
	try:
		wwidInfo_FILE = None
		wwidInfo_FILE = open( wwidInfoPath, 'r' )
		onDisk_diskInfo = pickle.load( wwidInfo_FILE )
		wwidInfo_FILE.close()
	except:
		if wwidInfo_FILE:
			wwidInfo_FILE.close()
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "read failure '%s'\n%s" % ( wwidInfoPath, traceback.format_exc() ) )
		return None

	diskInfo                     = DiskInfo_OBJ()
	diskInfo.mWWID               = onDisk_diskInfo.mWWID
	diskInfo.mIsSCSIdevice       = onDisk_diskInfo.mIsSCSIdevice
	diskInfo.mCurState           = DiskInfo_CurState_OBJ()
	diskInfo.mCurState.mWWID     = diskInfo.mWWID
	diskInfo.mBlkDevInfo_LIST    = []

	return diskInfo

####################################################################################################
def read_blk_and_disk_cache__single_blk_dev( vi_errorAndLog_obj, wwid, wwid_path, blkdev ):
	"""
	This routine
	"""
#
	if not is_cache_locked( vi_errorAndLog_obj ):
		vi_errorAndLog_obj.vi_croak( "cache is not locked" )

	blkdevPath = wwid_path + '/' + blkdev
	try:
		blkDevInfo_FILE = None
		blkDevInfo_FILE = open( blkdevPath, 'r' )
		blkDevInfo = pickle.load( blkDevInfo_FILE )
		blkDevInfo_FILE.close()
	except:
		if blkDevInfo_FILE:
			blkDevInfo_FILE.close()
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() ) # WARNING: caller holds lock
		vi_errorAndLog_obj.vi_print_err( "read failure '%s'\n%s" % ( wwidInfoPath, traceback.format_exc() ) )
		return None

	return blkDevInfo

####################################################################################################
def read_blk_and_disk_cache__single_disk( vi_errorAndLog_obj, wwid ):
	"""
	This routine
	"""
#
	if not is_cache_locked( vi_errorAndLog_obj ):
		vi_errorAndLog_obj.vi_croak( "cache is not locked" )

	wwid_path = global_infoCachePath + '/' + wwid

	if not os.path.exists( wwid_path ):
		vi_errorAndLog_obj.vi_print_wrn( "disk does not exist '%s'" % ( wwid_path ) ) # perhaps it was just deleted
		return None

	try:
		diskInfo = read_blk_and_disk_cache__single_wwid( vi_errorAndLog_obj, wwid, wwid_path )
		if not diskInfo:
			return None

		for blkdev in os.listdir( wwid_path ):
			if blkdev[0] == '_' or blkdev[0] == '*':
				continue
			blkDevInfo = read_blk_and_disk_cache__single_blk_dev( vi_errorAndLog_obj, wwid, wwid_path, blkdev )
			if blkDevInfo:
				diskInfo.mBlkDevInfo_LIST.append( blkDevInfo )
				blkDevInfo = None
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() ) # WARNING: caller holds lock
		vi_errorAndLog_obj.vi_print_wrn( "read failure '%s' (removed ???)\n%s" % ( wwid_path, traceback.format_exc() ) )
		return None

	return diskInfo

####################################################################################################
def read_blk_and_disk_cache__vgs( vi_errorAndLog_obj ):
	"""
	This routine
	"""
#
	if not is_cache_locked( vi_errorAndLog_obj ):
		vi_errorAndLog_obj.vi_croak( "cache is not locked" )

	if not os.path.exists( global_infoCacheVGSpath ):
		return {}

	vgInfo_uuid_DICT = {}
	vgsInfo_FILE = None
	try:
		vgsInfo_FILE = open( global_infoCacheVGSpath, 'r' )
		vgInfo_uuid_DICT = pickle.load( vgsInfo_FILE )
		vgsInfo_FILE.close()
	except:
		if vgsInfo_FILE:
			vgsInfo_FILE.close()
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() ) # WARNING: caller holds lock
		vi_errorAndLog_obj.vi_print_err( "read failure '%s'" % ( global_infoCacheVGSpath ) )
		return {}

	return vgInfo_uuid_DICT

####################################################################################################
def read_blk_and_disk_cache__missingVGs( vi_errorAndLog_obj ):
	"""
	This routine
	"""
#
	if not is_cache_locked( vi_errorAndLog_obj ):
		vi_errorAndLog_obj.vi_croak( "cache is not locked" )

	if not os.path.exists( global_infoCacheMissingVGsPath ):
		return {}

	missingVGs_DICT = {}
	missingVGsInfo_FILE = None
	try:
		missingVGsInfo_FILE = open( global_infoCacheMissingVGsPath, 'r' )
		missingVGs_DICT = pickle.load( missingVGsInfo_FILE )
		missingVGsInfo_FILE.close()
	except:
		if missingVGsInfo_FILE:
			missingVGsInfo_FILE.close()
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() ) # WARNING: caller holds lock
		vi_errorAndLog_obj.vi_print_err( "read failure '%s'" % ( global_infoCacheMissingVGsPath ) )
		return {}

	return missingVGs_DICT

####################################################################################################
def build_pvInfo_uuid_DICT( vgInfo_uuid_DICT ):
	"""
	This routine
	"""
#
	pvInfo_uuid_DICT = {}

	for vg_uuid in vgInfo_uuid_DICT:
		vg_info = vgInfo_uuid_DICT[ vg_uuid ]
		for pv_wwid in vg_info.mPVinfo_wwid_DICT.keys():
			pv_info = vg_info.mPVinfo_wwid_DICT[ pv_wwid ]
			pvInfo_uuid_DICT[ pv_info.mUUID ] = pv_info

	return pvInfo_uuid_DICT

####################################################################################################
def read_all_blk_cache( vi_errorAndLog_obj ):
	"""
	This routine
	"""
#
	diskInfo_DICT = {}
	try:
		lock_cache( vi_errorAndLog_obj, 'read_all_blk_cache', False )

		for wwid in os.listdir( global_infoCachePath ):
			if wwid[0] == '*':
				continue
			diskInfo = read_blk_and_disk_cache__single_disk( vi_errorAndLog_obj, wwid )
			if diskInfo:
				diskInfo_DICT[ wwid ] = diskInfo

		unlock_cache( vi_errorAndLog_obj )
	except:
		unlock_cache( vi_errorAndLog_obj )
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "read all failure\n%s" % ( traceback.format_exc() ) )

	return diskInfo_DICT

####################################################################################################
def read_specific_blk_cache( vi_errorAndLog_obj, disk_LIST ):
	"""
	This routine
	"""
#
	diskInfo_DICT = {}
	try:
		lock_cache( vi_errorAndLog_obj, 'read_all_blk_cache', False )
	
		for wwid in disk_LIST:
			diskInfo = read_blk_and_disk_cache__single_disk( vi_errorAndLog_obj, wwid )
			if diskInfo:
				diskInfo_DICT[ wwid ] = diskInfo
			diskInfo = None

		unlock_cache( vi_errorAndLog_obj )
	except:
		unlock_cache( vi_errorAndLog_obj )
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "read specific failure\n%s" % ( traceback.format_exc() ) )

	return diskInfo_DICT

####################################################################################################
def read_blk_and_disk_cache( vi_errorAndLog_obj ):
	"""
	This routine
	"""
#
	diskInfo_DICT    = {}
	vgInfo_uuid_DICT = {}
	missingVGs_DICT  = {}
	try:
		lock_cache( vi_errorAndLog_obj, 'read_blk_and_disk_cache', False )

		for wwid in os.listdir( global_infoCachePath ):
			if wwid[0] == '*':
				continue
			diskInfo = read_blk_and_disk_cache__single_disk( vi_errorAndLog_obj, wwid )
			if diskInfo:
				diskInfo_DICT[ wwid ] = diskInfo

		vgInfo_uuid_DICT = read_blk_and_disk_cache__vgs( vi_errorAndLog_obj )
		missingVGs_DICT  = read_blk_and_disk_cache__missingVGs( vi_errorAndLog_obj )

		unlock_cache( vi_errorAndLog_obj )
	except:
		unlock_cache( vi_errorAndLog_obj)
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "lvm information read failed !?!\n%s" % ( traceback.format_exc() ) )

	pvInfo_uuid_DICT = build_pvInfo_uuid_DICT( vgInfo_uuid_DICT )

	return ( diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT )

####################################################################################################
def delete_blk_and_disk_cache_cache( vi_errorAndLog_obj ):
	"""
	This routine
	"""
#
	try:
		lock_cache( vi_errorAndLog_obj, 'delete_blk_and_disk_cache_cache', True )

		if os.path.exists( global_infoCacheVGSpath ):
			try:
				vi_errorAndLog_obj.vi_print_dbg( 4, "Removing '%s'" % ( global_infoCacheVGSpath ) )
				os.unlink( global_infoCacheVGSpath )
			except:
				vi_errorAndLog_obj.vi_print_err( "FAILED to remove '%s'\n%s" % ( global_infoCacheVGSpath, traceback.format_exc() ) )
		else:
			vi_errorAndLog_obj.vi_print_dbg( 4, "Item     '%s' does not exist - skipping" % ( global_infoCacheVGSpath ) )

		if os.path.exists( global_infoCacheMissingVGsPath ):
			try:
				vi_errorAndLog_obj.vi_print_dbg( 4, "Removing '%s'" % ( global_infoCacheMissingVGsPath ) )
				os.unlink( global_infoCacheMissingVGsPath )
			except:
				vi_errorAndLog_obj.vi_print_err( "FAILED to remove '%s'\n%s" % ( global_infoCacheMissingVGsPath, traceback.format_exc() ) )
		else:
			vi_errorAndLog_obj.vi_print_dbg( 4, "Item     '%s' does not exist - skipping" % ( global_infoCacheMissingVGsPath ) )

		for wwid in os.listdir( global_infoCachePath ):
			if wwid[0] == '*':
				continue
			wwid_path = global_infoCachePath + '/' + wwid
			vi_utils.vi_delete_directory_including_file_contents( wwid_path )

		unlock_cache( vi_errorAndLog_obj )
	except:
		unlock_cache( vi_errorAndLog_obj )
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "Doom: %s" % ( traceback.format_exc() ) )

####################################################################################################
# END
