#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for generating storage XML
"""

import vi_utils
import vi_xml
import storage_tech

import os
import re
import glob

from sd_blkDevInfoUtils		import *

####################################################################################################

__all__ = [	'output_disk_scan_issues_section_XML'
		,	'on_crash_dump_errors_XML'
		,	'output_LVM_info_XML'
		,	'output_disk_and_lvm_data_XML'
		,	'block_data_XML' 
		,	'output_disk_XML' 
		,	'io_host_XML' ]

####################################################################################################
def output_disk_scan_issues_section_XML( vi_errorAndLog_obj, indentXMLlvl ):
	"""
	This routine
	"""
#
	numOfErr			= vi_errorAndLog_obj.get_num_of_errors()
	firstFewErr_LIST	= vi_errorAndLog_obj.get_out_dst( 'errAccumlation' ).vi_get_accumulation()

	numOfWrn			= vi_errorAndLog_obj.get_num_of_warnings()
	firstFewWrn_LIST	= vi_errorAndLog_obj.get_out_dst( 'wrnAccumlation' ).vi_get_accumulation()

	vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'DiskScanIssues', None )
	indentXMLlvl = indentXMLlvl + 2

	vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'Errors', "Num=\"%s\"" % ( numOfErr ), True )

	for msg in firstFewErr_LIST:
		vi_xml.vi_print_XML( 0, msg )	# Data is in RAW XML tags - no need to sanitize

	vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'Errors', True )
	vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'Warnings', "Num=\"%s\"" % ( numOfWrn ), True )

	for msg in firstFewWrn_LIST:
		vi_xml.vi_print_XML( 0, msg )	# Data is in RAW XML tags - no need to sanitize

	vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'Warnings', True )

	indentXMLlvl = indentXMLlvl - 2
	vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'DiskScanIssues' )

####################################################################################################
def on_crash_dump_errors_XML( vi_errorAndLog_obj, onCrashFcnData, crashTitleMsg, crashMsg ):
	"""
	This routine
	"""
#
	global global_XML_accumulation_LIST

	indentXMLlvl = onCrashFcnData

	global_XML_accumulation_LIST = []
	vi_errorAndLog_obj.vi_print_str( vi_utils.VI_ErrAndLog__FOR_FLAG__ERR, crashTitleMsg )
	output_disk_scan_issues_section_XML( vi_errorAndLog_obj, indentXMLlvl )
	vi_xml.vi_flush_XML()

	return 1 # exitValue

####################################################################################################
def block_data_XML_single_block_device( vi_errorAndLog_obj, indentXMLlvl, wwid, diskInfo ):
	"""
	This routine
	"""
#
	if len(diskInfo.mBlkDevInfo_LIST) < 1:
		vi_errorAndLog_obj.vi_print_wrn( "disk '%s' has not blkDevs !?!" % ( wwid ) )
		return

	sorted_blkDevInfo_LIST = diskInfo.mBlkDevInfo_LIST
	sorted_blkDevInfo_LIST.sort()

	blkDevInfo = sorted_blkDevInfo_LIST[ 0 ]

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )

	domDocHelper.push_level( 'Disk', { 'WWID' : wwid, 'TYPE' : blkDevInfo.mKind } )

	domDocHelper.add_data_elem( 'Transport',			blkDevInfo.mTransport )
	domDocHelper.add_data_elem( 'Vendor',				blkDevInfo.mVendor )
	domDocHelper.add_data_elem( 'Product',				blkDevInfo.mProduct )
	domDocHelper.add_data_elem( 'Rev',					blkDevInfo.mRev )
	domDocHelper.add_data_elem( 'Size',					blkDevInfo.mCurState.mSizeInBytes, { 'units' : 'bytes' } )
	domDocHelper.add_data_elem( 'ExplicitMPdevCnfg',	str(blkDevInfo.mHasExplicitMPconf) )
	domDocHelper.add_data_elem( 'WWIDisGenerated',		str(blkDevInfo.mIsWWIDgenerated) )

	if blkDevInfo.mOldWWID != None:
		domDocHelper.add_data_elem( 'OldWWID',	blkDevInfo.mOldWWID )

	domDocHelper.add_data_elem( 'Target',		blkDevInfo.mTarget )

	domDocHelper.push_level( 'Multipath' )

	for blkDevInfo in sorted_blkDevInfo_LIST:

		domDocHelper.push_level( 'Path', { 'Dev' : blkDevInfo.mBlkDev } )

		domDocHelper.add_data_elem( 'DevPath',	blkDevInfo.mDevPath )
		domDocHelper.add_data_elem( 'Device',	blkDevInfo.mDevNums )

		if blkDevInfo.mSCSIdevID:
			domDocHelper.add_data_elem( 'SCSIdeviceID', str(blkDevInfo.mSCSIdevID) )
		else:
			domDocHelper.add_data_elem( 'SCSIdeviceID', '' )

		domDocHelper.add_data_elem( 'Target',		blkDevInfo.mTarget )
		domDocHelper.add_data_elem( 'TargetIP',		blkDevInfo.mTargetIP )
		domDocHelper.add_data_elem( 'TargetPort',	blkDevInfo.mTargetPort )
		domDocHelper.add_data_elem( 'TargetIQN',	blkDevInfo.mTargetIQN )
		domDocHelper.add_data_elem( 'TargetTPGT',	blkDevInfo.mTargetTPGT )

		domDocHelper.pop_level()

	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )

####################################################################################################
def disk_info_XML_single_disk( vi_errorAndLog_obj, indentXMLlvl, wwid, diskInfo, pvInfo_wwid_DICT ):
	"""
	This routine
	"""
#
	if len(diskInfo.mBlkDevInfo_LIST) < 1:
		vi_errorAndLog_obj.vi_print_wrn( "disk '%s' has not blkDevs !?!" % ( wwid ) )
		return

	sorted_blkDevInfo_LIST = diskInfo.mBlkDevInfo_LIST
	sorted_blkDevInfo_LIST.sort()

	blkDevInfo = sorted_blkDevInfo_LIST[ 0 ]

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )

	domDocHelper.push_level( 'Disk', { 'WWID' : wwid, 'TYPE' : blkDevInfo.mKind } )

	domDocHelper.add_data_elem( 'Transport',			blkDevInfo.mTransport )
	domDocHelper.add_data_elem( 'Vendor',				blkDevInfo.mVendor )
	domDocHelper.add_data_elem( 'Product',				blkDevInfo.mProduct )
	domDocHelper.add_data_elem( 'Rev',					blkDevInfo.mRev )
	domDocHelper.add_data_elem( 'Size',					blkDevInfo.mCurState.mSizeInBytes, { 'units' : 'bytes' } )
	domDocHelper.add_data_elem( 'ExplicitMPdevCnfg',	str(blkDevInfo.mHasExplicitMPconf) )
	domDocHelper.add_data_elem( 'WWIDisGenerated',		str(blkDevInfo.mIsWWIDgenerated) )

	if blkDevInfo.mOldWWID != None:
		domDocHelper.add_data_elem( 'OldWWID',	blkDevInfo.mOldWWID )

	domDocHelper.add_data_elem( 'Target',		blkDevInfo.mTarget )

	pvInfo = pvInfo_wwid_DICT.get( wwid, None )
	if pvInfo:
		domDocHelper.add_data_elem( 'PhysicalVolumeID',	pvInfo.mUUID )
		domDocHelper.add_data_elem( 'VolumeGroupName',	pvInfo.mVGname )
	else:
		domDocHelper.add_data_elem( 'PhysicalVolumeID',	'' )
		domDocHelper.add_data_elem( 'VolumeGroupName',	'' )

	domDocHelper.push_level( 'Multipath' )

	for blkDevInfo in sorted_blkDevInfo_LIST:

		domDocHelper.push_level( 'Path', { 'Dev' : blkDevInfo.mBlkDev } )

		domDocHelper.add_data_elem( 'DevPath',	blkDevInfo.mDevPath )
		domDocHelper.add_data_elem( 'Device',	blkDevInfo.mDevNums )

		if blkDevInfo.mSCSIdevID:
			domDocHelper.add_data_elem( 'SCSIdeviceID', str(blkDevInfo.mSCSIdevID) )
		else:
			domDocHelper.add_data_elem( 'SCSIdeviceID', '' )

		domDocHelper.add_data_elem( 'Target',		blkDevInfo.mTarget )
		domDocHelper.add_data_elem( 'TargetIP',		blkDevInfo.mTargetIP )
		domDocHelper.add_data_elem( 'TargetPort',	blkDevInfo.mTargetPort )
		domDocHelper.add_data_elem( 'TargetIQN',	blkDevInfo.mTargetIQN )
		domDocHelper.add_data_elem( 'TargetTPGT',	blkDevInfo.mTargetTPGT )

		domDocHelper.add_data_elem( 'Path_State',	blkDevInfo.mCurState.mPathDMmapState  )
		domDocHelper.add_data_elem( 'Check_State',	blkDevInfo.mCurState.mCheckerState )

		domDocHelper.pop_level()

	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )

####################################################################################################
def disk_info_XML( vi_errorAndLog_obj, indentXMLlvl, wwid, diskInfo ):
	"""
	This routine
	"""
#
	if len(diskInfo.mBlkDevInfo_LIST) < 1:
		vi_errorAndLog_obj.vi_add_wrn( "disk '%s' has no blkDevs !?!" % ( wwid ) )
		return

	sorted_blkDevInfo_LIST = diskInfo.mBlkDevInfo_LIST
	sorted_blkDevInfo_LIST.sort()

	blkDevInfo = sorted_blkDevInfo_LIST[ 0 ]

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )

	domDocHelper.push_level( 'Disk', { 'WWID' : wwid, 'TYPE' : blkDevInfo.mKind } )

	domDocHelper.add_data_elem( 'Transport',			blkDevInfo.mTransport )
	domDocHelper.add_data_elem( 'Vendor',				blkDevInfo.mVendor )
	domDocHelper.add_data_elem( 'Product',				blkDevInfo.mProduct )
	domDocHelper.add_data_elem( 'Rev',					blkDevInfo.mRev )
	domDocHelper.add_data_elem( 'Size',					blkDevInfo.mCurState.mSizeInBytes, { 'units' : 'bytes' } )
	domDocHelper.add_data_elem( 'ExplicitMPdevCnfg',	str(blkDevInfo.mHasExplicitMPconf) )
	domDocHelper.add_data_elem( 'WWIDisGenerated',		str(blkDevInfo.mIsWWIDgenerated) )

	if blkDevInfo.mOldWWID != None:
		domDocHelper.add_data_elem( 'OldWWID',	blkDevInfo.mOldWWID )

	domDocHelper.add_data_elem( 'Target',		blkDevInfo.mTarget )
	
	domDocHelper.push_level( 'Multipath', { 'DeviceMap' : "/dev/dm-%s" % blkDevInfo.mDevMapNum } )

	for blkDevInfo in sorted_blkDevInfo_LIST:

		domDocHelper.push_level( 'Path', { 'Dev' : blkDevInfo.mBlkDev } )

		domDocHelper.add_data_elem( 'DevPath',	blkDevInfo.mDevPath )
		domDocHelper.add_data_elem( 'Device',	blkDevInfo.mDevNums )

		if blkDevInfo.mSCSIdevID:
			domDocHelper.add_data_elem( 'SCSIdeviceID', str(blkDevInfo.mSCSIdevID) )
		else:
			domDocHelper.add_data_elem( 'SCSIdeviceID', '' )

		domDocHelper.add_data_elem( 'Target',		blkDevInfo.mTarget )
		domDocHelper.add_data_elem( 'TargetIP',		blkDevInfo.mTargetIP )
		domDocHelper.add_data_elem( 'TargetPort',	blkDevInfo.mTargetPort )
		domDocHelper.add_data_elem( 'TargetIQN',	blkDevInfo.mTargetIQN )
		domDocHelper.add_data_elem( 'TargetTPGT',	blkDevInfo.mTargetTPGT )

		domDocHelper.add_data_elem( 'Path_State',	blkDevInfo.mCurState.mPathDMmapState  )
		domDocHelper.add_data_elem( 'Check_State',	blkDevInfo.mCurState.mCheckerState )

		domDocHelper.pop_level()

	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )

####################################################################################################
def scsi_host_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, scsi_host_num ):
	"""
	This routine
	"""
#
	try:
		scsi_host				= "host%s" % ( scsi_host_num )
		scsi_host_path			= "/sys/class/scsi_host/%s" % ( scsi_host )

		sorted_devInfo_LIST		= os.listdir( scsi_host_path )

		scsi_host_devLink		= os.readlink( "/sys/class/scsi_host/%s/device" % ( scsi_host ) )
		scsi_host_devLink		= re.sub( "/%s" % ( scsi_host ), '', scsi_host_devLink )
		scsi_host_devLink		= re.sub( '^\.\./\.\./\.\./devices', '/sys/devices', scsi_host_devLink )
		sorted_devLinkInfo_LIST	= os.listdir( scsi_host_devLink )
	except:
		return

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )
	domDocHelper.push_level( 'SCSI_Host', { 'Num' : scsi_host_num } )

	exclude_LIST = [ 'ctlreg', 'resource', 'unchecked_isa_dma', 'max_sectors' ]

	sorted_devInfo_LIST.sort()
	for sys_item in sorted_devInfo_LIST:
		vi_xml.vi_sys_item_XML( vi_errorAndLog_obj, domDocHelper, scsi_host_path, sys_item, exclude_LIST )

	domDocHelper.push_level( 'SysFSDev' )

	exclude_LIST = [ 'config', 'local_cpus', 'modalias', 'pools', 'resource', 'vpd'  ]

	sorted_devLinkInfo_LIST.sort()
	for sys_item in sorted_devLinkInfo_LIST:
		vi_xml.vi_sys_item_XML( vi_errorAndLog_obj, domDocHelper, scsi_host_devLink, sys_item, exclude_LIST )

	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )

####################################################################################################
def fc_host_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, fc_host_num ):
	"""
	This routine
	"""
#
	fc_host = "host%s" % ( fc_host_num )
	fc_host_path = "/sys/class/fc_host/%s" % ( fc_host )

	try:
		sorted_fc_host_LIST = os.listdir( fc_host_path )
	except:
		return

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )
	domDocHelper.push_level( 'FC_Host', { 'Num' : fc_host_num } )

	exclude_LIST = [ 'active_fc', 'modalias', 'supported_fc', 'tgtid_bind_type' ]

	sorted_fc_host_LIST.sort()
	for sys_item in sorted_fc_host_LIST:
		vi_xml.vi_sys_item_XML( vi_errorAndLog_obj, domDocHelper, fc_host_path, sys_item, exclude_LIST )

	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )

####################################################################################################
def fc_target_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, fc_target, fc_target_num, wwpn ):
	"""
	This routine
	"""
#
	fc_target_path			= "/sys/class/fc_transport/%s" % ( fc_target )
	fc_target_devInfo_path	= "%s/device/%s" % ( fc_target_path, fc_target_num )

	try:
		sorted_fc_target_LIST	= os.listdir( fc_target_path )
		sorted_devInfo_LIST		= os.listdir( fc_target_devInfo_path )
	except:
		return

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )
	domDocHelper.push_level( 'FC_Target', { 'Target' : fc_target_num, 'WWPN' : wwpn } )

	exclude_LIST = [] # 'active_fc', 'modalias', 'supported_fc', 'tgtid_bind_type' ]

	sorted_fc_target_LIST.sort()
	for sys_item in sorted_fc_target_LIST:
		vi_xml.vi_sys_item_XML( vi_errorAndLog_obj, domDocHelper, fc_target_path, sys_item, exclude_LIST )

	domDocHelper.push_level( 'Device_Info' )

	exclude_LIST = []

	sorted_devInfo_LIST.sort()
	for sys_item in sorted_devInfo_LIST:
		vi_xml.vi_sys_item_XML( vi_errorAndLog_obj, domDocHelper, fc_target_devInfo_path, sys_item, exclude_LIST )

	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )

####################################################################################################
def iscsi_host_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, iscsi_host_num, initiator_name):
	"""
	This routine
	"""
#


	iscsi_host = "host%s" % ( iscsi_host_num )
	iscsi_host_path = "/sys/class/iscsi_host/%s" % ( iscsi_host )

	try:
		sorted_iscsi_host_LIST = os.listdir( iscsi_host_path )
	except:
		return

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )
	domDocHelper.push_level( 'ISCSI_Host', { 'Num' : iscsi_host_num } )

	exclude_LIST = [ 'hwaddress', 'netdev', 'initiatorname' ]

	sorted_iscsi_host_LIST.sort()
	for sys_item in sorted_iscsi_host_LIST:
		vi_xml.vi_sys_item_XML( vi_errorAndLog_obj, domDocHelper, iscsi_host_path, sys_item, exclude_LIST )

        domDocHelper.add_data_elem( "Initiatorname", initiator_name )
	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )

####################################################################################################
def iscsi_node_info_XML( vi_errorAndLog_obj, indentXMLlvl, initiator_name):
	"""
	This routine
	"""
#

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )
	domDocHelper.push_level( 'ISCSI_Node' )

        domDocHelper.add_data_elem( "Initiatorname", initiator_name )
	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )

####################################################################################################
def iscsi_target_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, iscsi_session, iscsi_session_num, iscsi_targetname, iscsi_target_ip ):
	"""
	This routine
	"""
#
	iscsi_target_path = "/sys/class/iscsi_session/%s" % ( iscsi_session )

	try:
		sorted_iscsi_target_LIST = os.listdir( iscsi_target_path )
	except:
		return

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )
	if iscsi_target_ip:
                domDocHelper.push_level( 'ISCSI_Target', { 'Session' : iscsi_session_num, 'Target_Ip' : iscsi_target_ip } )
        else:
	        domDocHelper.push_level( 'ISCSI_Target', { 'Session' : iscsi_session_num, 'WWNN' : iscsi_targetname} )

	exclude_LIST = [ 'password', 'username' ]

	sorted_iscsi_target_LIST.sort()
	for sys_item in sorted_iscsi_target_LIST:
		vi_xml.vi_sys_item_XML( vi_errorAndLog_obj, domDocHelper, iscsi_target_path, sys_item, exclude_LIST )

	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )


####################################################################################################
def sas_target_info_XML_single_host ( vi_errorAndLog_obj, indentXMLlvl, end_device_str, target, address):
	"""
	This routine
	"""
#
	sas_target_path = "/sys/class/sas_device/%s" % ( end_device_str )

	try:
		sorted_sas_target_LIST = os.listdir( sas_target_path )
	except:
		return

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )
	domDocHelper.push_level( 'SAS_Target', { 'Target' : target , 'Address' : address}  )

	exclude_LIST = [ 'uevent', 'device', ]

	sorted_sas_target_LIST.sort()
	for sys_item in sorted_sas_target_LIST:
		vi_xml.vi_sys_item_XML( vi_errorAndLog_obj, domDocHelper, sas_target_path, sys_item, exclude_LIST )

	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )


####################################################################################################
def ide_cdrom_info_XML_single_drive( vi_errorAndLog_obj, indentXMLlvl, ideCDrom_num ):
	"""
	This routine
	"""
#
	ideCDrom_path = "/sys/bus/ide/drivers/ide-cdrom/%s" % ( ideCDrom_num )
	ideCDrom_blkDev, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, "%s/drivename" % ( ideCDrom_path ) )
	if read_failed:
		return

	ideCDrom_proc_path = "/proc/ide/%s" % ( ideCDrom_blkDev )

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )
	domDocHelper.push_level( 'CD-ROM', { 'Name' : ideCDrom_blkDev } )

	domDocHelper.add_data_elem( 'IDE_Num', ideCDrom_num )

	vi_xml.vi_sys_item_XML( vi_errorAndLog_obj, domDocHelper, ideCDrom_proc_path, 'model', [] )

	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )

####################################################################################################
def sr_scsi_cdrom_info_XML_single_drive( vi_errorAndLog_obj, indentXMLlvl, scsiCDrom, srCDrom_path ):
	"""
	This routine
	"""
#
	try:
		sorted_srCDromPossible_LIST = os.listdir( srCDrom_path )
	except:
		return

	srCDrom_name = None
	srCDrom_cmpldRegEx = re.compile( '^block:(sr\d+)$' )
	sorted_srCDromPossible_LIST.sort()
	for srCDromPossible in sorted_srCDromPossible_LIST:
		srCDrom_match = srCDrom_cmpldRegEx.match( srCDromPossible )
		if not srCDrom_match:
			continue
		srCDrom_name = srCDrom_match.group( 1 )
		break

	if not srCDrom_name:
		vi_errorAndLog_obj.vi_add_wrn( "sr cdrom name not found for %s (VANISHED ???)" % ( srCDrom_path ) )
		return

	vendor, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, "%s/vendor" % ( srCDrom_path ) )
	if read_failed:
		vi_errorAndLog_obj.vi_add_wrn( "sr cdrom vendor not found for %s (VANISHED ???)" % ( srCDrom_path ) )
		return

	model, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, "%s/model" % ( srCDrom_path ) )
	if read_failed:
		vi_errorAndLog_obj.vi_add_wrn( "sr cdrom model not found for %s (VANISHED ???)" % ( srCDrom_path ) )
		return

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )
	domDocHelper.push_level( 'CD-ROM', { 'Name' : srCDrom_name } )

	domDocHelper.add_data_elem( 'Model', "%s %s" % ( vendor, model ) )

	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )

####################################################################################################
def output_LVM_info_XML( vi_errorAndLog_obj, indentXMLlvl, vgInfo_uuid_DICT, missingVGs_DICT ):
	"""
	This routine
	"""
#
	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )

	domDocHelper.push_level( 'LVM' )
	domDocHelper.push_level( 'VolumeGroups' )

	sorted_vgInfo_LIST = vgInfo_uuid_DICT.values()
	sorted_vgInfo_LIST.sort()

	for vgInfo in sorted_vgInfo_LIST:

		domDocHelper.push_level( 'VolGrp_Set' )

		domDocHelper.add_data_elem( 'VolumeGroup', vgInfo.mName )
		domDocHelper.add_data_elem( 'VolumeGroupUUID', vgInfo.mUUID )
		domDocHelper.add_data_elem( 'Size', vgInfo.mSizeInBytes, { 'units' : 'bytes' } )
		domDocHelper.add_data_elem( 'FreeSize', vgInfo.mFreeSizeInBytes, { 'units' : 'bytes' } )

		sorted_pv_wwid_LIST = vgInfo.mPVinfo_wwid_DICT.keys()
		sorted_pv_wwid_LIST.sort()
		for pv_wwid in sorted_pv_wwid_LIST:
			pvInfo = vgInfo.mPVinfo_wwid_DICT[ pv_wwid ]
			domDocHelper.add_data_elem( 'PhysicalVolume', pvInfo.mWWID, { 'UUID' : pvInfo.mUUID } )

		domDocHelper.pop_level()

	domDocHelper.pop_level()
	domDocHelper.push_level( 'VolGrp_PhyVol_Pairs' )

	for vgInfo in sorted_vgInfo_LIST:
		sorted_pv_wwid_LIST = vgInfo.mPVinfo_wwid_DICT.keys()
		sorted_pv_wwid_LIST.sort()
		for pv_wwid in sorted_pv_wwid_LIST:
			pvInfo = vgInfo.mPVinfo_wwid_DICT[ pv_wwid ]

			domDocHelper.push_level( 'VG_PV_Pair' )

			domDocHelper.add_data_elem( 'VolumeGroup', pvInfo.mVGname )
			domDocHelper.add_data_elem( 'PhysicalVolume', pvInfo.mWWID )

			domDocHelper.pop_level()

	domDocHelper.pop_level()
	domDocHelper.push_level( 'LogicalVolumes' )

	for vgInfo in sorted_vgInfo_LIST:
		lvInfo_DICT = vgInfo.mLVinfo_name_DICT
		sorted_lv_name_LIST = lvInfo_DICT.keys()
		sorted_lv_name_LIST.sort()
		for lv_name in sorted_lv_name_LIST:
			lvInfo = lvInfo_DICT[ lv_name ]
			lv_attr = lvInfo.mAttr
			lv_type = lv_attr[ 0 : 1 ]
			lv_perm = lv_attr[ 1 : 2 ]

			domDocHelper.push_level( 'LogVol_Set' )

			domDocHelper.add_data_elem( 'LogicalVolume', lvInfo.mName )
			domDocHelper.add_data_elem( 'LogicalVolumeUUID', lvInfo.mUUID )
			domDocHelper.add_data_elem( 'VolumeGroup', lvInfo.mVGname )
			domDocHelper.add_data_elem( 'Size', lvInfo.mSizeInBytes, { 'units' : 'bytes' } )
#			domDocHelper.add_data_elem( 'LogicalVolumeAttr', lv_attr )
			domDocHelper.add_data_elem( 'LogicalVolumeType', lv_type )
			domDocHelper.add_data_elem( 'LogicalVolumePerm', lv_perm )
			domDocHelper.add_data_elem( 'LogicalVolumeParent', lvInfo.mOrigin )
			domDocHelper.add_data_elem( 'SnapshotPercent', lvInfo.mSnapPercent )

			domDocHelper.pop_level()

	domDocHelper.pop_level()
	domDocHelper.push_level( 'PartitionIDs' )

	for vgInfo in sorted_vgInfo_LIST:
		lvInfo_DICT = vgInfo.mLVinfo_name_DICT
		sorted_lv_name_LIST = lvInfo_DICT.keys()
		sorted_lv_name_LIST.sort()
		for lv_name in sorted_lv_name_LIST:
			lvInfo = lvInfo_DICT[ lv_name ]

			mount_dev	= "/dev/%s/%s" % ( lvInfo.mVGname, lvInfo.mName )
			mount_pt	= "/mnt/lvp_%s_%s_p0" % ( lvInfo.mVGname, lvInfo.mName )

			domDocHelper.push_level( 'PartitionID_Set' )

			domDocHelper.add_data_elem( 'VolumeGroup', lvInfo.mVGname )
			domDocHelper.add_data_elem( 'LogicalVolume', lvInfo.mName )
			domDocHelper.add_data_elem( 'PartitionID', '0' )
			domDocHelper.add_data_elem( 'MountDevice', mount_dev )
			domDocHelper.add_data_elem( 'MountPoint', mount_pt )
			domDocHelper.add_data_elem( 'Size', lvInfo.mSizeInBytes, { 'units' : 'bytes' } )

			domDocHelper.pop_level()

	vi_xml.vi_print_XML( 0, '\n'.join( domDocHelper.finalize() ) )

####################################################################################################
def output_disk_and_lvm_data_XML( vi_errorAndLog_obj, indentXMLlvl, skipLVM, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "disk_and_lvm_data_XML" )

	pvInfo_wwid_DICT = {}
	for vg_uuid in vgInfo_uuid_DICT:
		vg_info = vgInfo_uuid_DICT[ vg_uuid ]
		for pv_wwid in vg_info.mPVinfo_wwid_DICT:
			pv_info = vg_info.mPVinfo_wwid_DICT[ pv_wwid ]
			pvInfo_wwid_DICT[ pv_wwid ] = pv_info

	vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'SCSI', None )
	indentXMLlvl = indentXMLlvl + 2

	if is_default_node_configuration():
		vi_xml.vi_print_XML_data( indentXMLlvl, 'NodeConfiguration', 'DEFAULT' )

	sorted_wwid_LIST = diskInfo_DICT.keys()
	sorted_wwid_LIST.sort()
	for wwid in sorted_wwid_LIST:
		diskInfo = diskInfo_DICT[ wwid ]
		disk_info_XML_single_disk( vi_errorAndLog_obj, indentXMLlvl, wwid, diskInfo, pvInfo_wwid_DICT )

	if not skipLVM:
		output_LVM_info_XML( vi_errorAndLog_obj, indentXMLlvl, vgInfo_uuid_DICT, missingVGs_DICT )

	indentXMLlvl = indentXMLlvl - 2
	vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'SCSI' )

	output_disk_scan_issues_section_XML( vi_errorAndLog_obj, indentXMLlvl )



####################################################################################################
def output_disk_XML( vi_errorAndLog_obj, indentXMLlvl, diskInfo_DICT, item_LIST):
	"""
	This routine
	"""
#

	vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'SCSI', None )
	indentXMLlvl = indentXMLlvl + 2

	if is_default_node_configuration():
		vi_xml.vi_print_XML_data( indentXMLlvl, 'NodeConfiguration', 'DEFAULT' )
        
	wwid_LIST = diskInfo_DICT.keys()

        if item_LIST:
                sorted_wwid_LIST = list(set(item_LIST).intersection(set(wwid_LIST))) 
                unknown_wwid = list(set(item_LIST).difference(set(sorted_wwid_LIST)))
		for unknown in unknown_wwid:
                        vi_errorAndLog_obj.vi_print_wrn( "disk does not exist '%s'" % ( unknown ) )
                
        else:
                sorted_wwid_LIST = wwid_LIST


	sorted_wwid_LIST.sort()
	for wwid in sorted_wwid_LIST:
		diskInfo = diskInfo_DICT[ wwid ]
		disk_info_XML( vi_errorAndLog_obj, indentXMLlvl, wwid, diskInfo)

	indentXMLlvl = indentXMLlvl - 2
	vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'SCSI' )

        output_disk_scan_issues_section_XML( vi_errorAndLog_obj, indentXMLlvl )

####################################################################################################
def fc_target_DICT__compare( fc_target_DICT__A, fc_target_DICT__B ):
	"""
	This routine
	"""
#
	if fc_target_DICT__A[ 'fc_target' ] > fc_target_DICT__A[ 'fc_target' ]:
		return  1
	if fc_target_DICT__A[ 'fc_target' ] < fc_target_DICT__A[ 'fc_target' ]:
		return -1
	fc_target__A__digits_LIST = fc_target_DICT__A[ 'fc_target_num' ].split( ':' )
	fc_target__B__digits_LIST = fc_target_DICT__B[ 'fc_target_num' ].split( ':' )
	if int(fc_target__A__digits_LIST[ 0 ]) > int(fc_target__B__digits_LIST[ 0 ]):
		return  1
	if int(fc_target__A__digits_LIST[ 0 ]) < int(fc_target__B__digits_LIST[ 0 ]):
		return -1
	if int(fc_target__A__digits_LIST[ 1 ]) > int(fc_target__B__digits_LIST[ 1 ]):
		return  1
	if int(fc_target__A__digits_LIST[ 1 ]) < int(fc_target__B__digits_LIST[ 1 ]):
		return -1
	if int(fc_target__A__digits_LIST[ 2 ]) > int(fc_target__B__digits_LIST[ 2 ]):
		return  1
	if int(fc_target__A__digits_LIST[ 2 ]) < int(fc_target__B__digits_LIST[ 2 ]):
		return -1
	return 0

####################################################################################################
def iscsi_session_DICT__compare( iscsi_session_DICT__A, iscsi_session_DICT__B ):
	"""
	This routine
	"""
#
	if iscsi_session_DICT__A[ 'iscsi_targetname' ] > iscsi_session_DICT__B[ 'iscsi_targetname' ]:
		return  1
	if iscsi_session_DICT__A[ 'iscsi_targetname' ] < iscsi_session_DICT__B[ 'iscsi_targetname' ]:
		return -1
	if iscsi_session_DICT__A[ 'iscsi_session_num' ] > iscsi_session_DICT__B[ 'iscsi_session_num' ]:
		return  1
	if iscsi_session_DICT__A[ 'iscsi_session_num' ] < iscsi_session_DICT__B[ 'iscsi_session_num' ]:
		return -1
	return 0

####################################################################################################
def sas_target_DICT__compare( sas_target_DICT__A, sas_target_DICT__B ):
	"""
	This routine does the comparison for sorting.
	"""
#
	if sas_target_DICT__A[ 'sas_target_num' ] > sas_target_DICT__B[ 'sas_target_num' ]:
		return  1
	if sas_target_DICT__A[ 'sas_target_num' ] < sas_target_DICT__B[ 'sas_target_num' ]:
		return -1
	return 0
####################################################################################################
def is_default_node_configuration():
	"""
	This routine
	"""
#
	if not os.path.exists( '/var/log/DEFAULT_NODE_CONFIGURATION' ):
		return False
	return True

####################################################################################################

def block_data_XML( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, pvInfo_uuid_DICT, vgInfo_uuid_DICT, missingVGs_DICT ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "block_data_XML" )

	indentXMLlvl = options_OBJ.indentXMLlvl

	vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'SCSI', None )
	indentXMLlvl = indentXMLlvl + 2

	if is_default_node_configuration():
		vi_xml.vi_print_XML_data( indentXMLlvl, 'NodeConfiguration', 'DEFAULT' )

	sorted_wwid_LIST = diskInfo_DICT.keys()
	sorted_wwid_LIST.sort()
	for wwid in sorted_wwid_LIST:
		diskInfo = diskInfo_DICT[ wwid ]
		block_data_XML_single_block_device( vi_errorAndLog_obj, indentXMLlvl, wwid, diskInfo )

	if os.path.exists( '/sys/class/scsi_host' ):
		scsi_host_LIST = os.listdir( '/sys/class/scsi_host' )
		scsi_host_num_LIST = []
		for scsi_host in scsi_host_LIST:
			scsi_host_num_LIST.append( int( scsi_host.replace( 'host', '' ) ) )
		for scsi_host_num in sorted(scsi_host_num_LIST):
			scsi_host_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, scsi_host_num )

	if os.path.exists( '/sys/class/fc_host' ):
		fc_host_LIST = os.listdir( '/sys/class/fc_host' )
		fc_host_num_LIST = []
		for fc_host in fc_host_LIST:
			fc_host_num_LIST.append( int( fc_host.replace( 'host', '' ) ) )
		for fc_host_num in sorted(fc_host_num_LIST):
			fc_host_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, fc_host_num )

	if os.path.exists( '/sys/class/fc_transport' ):
		fc_target_LIST = os.listdir( '/sys/class/fc_transport' )
		fc_target_DICT_LIST = []
		for fc_target in fc_target_LIST:
			fc_target_num = fc_target.replace( 'target', '' ) + ':0'
			wwpn, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, "/sys/class/fc_transport/%s/port_name" % ( fc_target ) )
			if read_failed:
				continue
			wwpn = re.sub( '^0x', '', wwpn )
			fc_target_DICT_LIST.append( { 'fc_target' : fc_target, 'fc_target_num' : fc_target_num, 'wwpn' : wwpn } )
		fc_target_DICT_LIST.sort( fc_target_DICT__compare )
		for fc_target_DICT in fc_target_DICT_LIST:
			fc_target_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, fc_target_DICT[ 'fc_target' ], fc_target_DICT[ 'fc_target_num' ], fc_target_DICT[ 'wwpn' ]  )

	if os.path.exists( '/sys/class/iscsi_host' ):
		iscsi_host_LIST = os.listdir( '/sys/class/iscsi_host' )
		iscsi_host_num_LIST = []
		for iscsi_host in iscsi_host_LIST:
			iscsi_host_num_LIST.append( int( iscsi_host.replace( 'host', '' ) ) )
		for iscsi_host_num in sorted(iscsi_host_num_LIST):
			iscsi_host_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, iscsi_host_num, '')

	if os.path.exists( '/sys/class/iscsi_session' ):
		iscsi_session_LIST = os.listdir( '/sys/class/iscsi_session' )
		iscsi_session_DICT_LIST = []
		for iscsi_session in iscsi_session_LIST:
			iscsi_session_num = iscsi_session.replace( 'session', '' )
			iscsi_targetname, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, "/sys/class/iscsi_session/%s/targetname" % ( iscsi_session ) )
			if read_failed:
				continue
			iscsi_session_DICT_LIST.append( { 'iscsi_session' : iscsi_session, 'iscsi_session_num' : int(iscsi_session_num), 'iscsi_targetname' : iscsi_targetname } )
		iscsi_session_DICT_LIST.sort( iscsi_session_DICT__compare )
		for iscsi_session_DICT in iscsi_session_DICT_LIST:
			iscsi_target_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, iscsi_session_DICT[ 'iscsi_session' ], iscsi_session_DICT[ 'iscsi_session_num' ], iscsi_session_DICT[ 'iscsi_targetname' ], None )

	# <SAS_Host> is the same as <SCSI_Host>
	# <SAS_Target>
	if os.path.exists( '/sys/class/sas_device' ):
		sas_target_DICT_LIST = []
		sas_device_LIST = os.listdir('/sys/class/sas_device' )
		for sas_device in sas_device_LIST:
			scsi_is_sas_expander_match = re.match("^end_device-(\d+):(\d+):(\d+)$", sas_device)
			if scsi_is_sas_expander_match:
				sas_address, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, "/sys/class/sas_device/%s/sas_address" % ( sas_device ) )
				sas_address = re.sub( '^0x', '', sas_address )
				if read_failed:
					continue
				sas_target_LIST = os.listdir( '/sys/class/sas_device/%s/device/' % (sas_device))
				for sas_target in sas_target_LIST:
					if 'target' in sas_target:
						sas_target_num = sas_target.replace('target','') + ':0'
						sas_target_DICT_LIST.append( {'sas_device' :  sas_device, 'sas_target_num' : sas_target_num, 'sas_address' : sas_address } )

		sas_target_DICT_LIST.sort (sas_target_DICT__compare)
		for sas_target_DICT in sas_target_DICT_LIST:
			sas_target_info_XML_single_host (vi_errorAndLog_obj, indentXMLlvl, sas_target_DICT['sas_device'], sas_target_DICT['sas_target_num'], sas_target_DICT['sas_address'])

	if os.path.exists( '/sys/bus/scsi/drivers/sr' ):
		scsiCDrom_cmpldRegEx = re.compile( '^\d+:\d+:\d+:\d+$' )
		try:
			sorted_srCDromPossible_LIST = os.listdir( '/sys/bus/scsi/drivers/sr' )
			sorted_srCDromPossible_LIST.sort()
		except:
			sorted_srCDromPossible_LIST = []
		for scsiCDromPossible in sorted_srCDromPossible_LIST:
			if not scsiCDrom_cmpldRegEx.match( scsiCDromPossible ):
				continue

			srCDrom_path = "/sys/bus/scsi/drivers/sr/%s" % ( scsiCDromPossible )

			typeValue, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, "%s/type" % ( srCDrom_path ) )
			if read_failed:
				continue

			if int(typeValue) != 5:
				continue

			sr_scsi_cdrom_info_XML_single_drive( vi_errorAndLog_obj, indentXMLlvl, scsiCDromPossible, srCDrom_path )

	indentXMLlvl = indentXMLlvl - 2
	vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'SCSI' )

	vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'IDE', None )
	indentXMLlvl = indentXMLlvl + 2

	if os.path.exists( '/sys/bus/ide/drivers/ide-cdrom' ):
		ideCDrom_cmpldRegEx		= re.compile( '^\d+.\d+$' )
		try:
			sorted_ideCDromPossible_LIST = os.listdir( '/sys/bus/ide/drivers/ide-cdrom' )
			sorted_ideCDromPossible_LIST.sort()
		except:
			sorted_ideCDromPossible_LIST = []
		ideCDrom_num_LIST		= []
		for ideCDromPossible in sorted_ideCDromPossible_LIST:
			if ideCDrom_cmpldRegEx.match( ideCDromPossible ):
				ideCDrom_num_LIST.append( float( ideCDromPossible ) )
		for ideCDrom_num in sorted(ideCDrom_num_LIST):
			ide_cdrom_info_XML_single_drive( vi_errorAndLog_obj, indentXMLlvl, ideCDrom_num )

	indentXMLlvl = indentXMLlvl - 2
	vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'IDE' )

	output_disk_scan_issues_section_XML( vi_errorAndLog_obj, indentXMLlvl )

####################################################################################################
def io_host_XML( vi_errorAndLog_obj, options_OBJ, args_LIST, item_LIST, diskInfo_DICT, dummy2, dummy3, dummy4 ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "block_data_XML" )

	indentXMLlvl = options_OBJ.indentXMLlvl

	vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'SCSI', None )
	indentXMLlvl = indentXMLlvl + 2

	if is_default_node_configuration():
		vi_xml.vi_print_XML_data( indentXMLlvl, 'NodeConfiguration', 'DEFAULT' )

#	sorted_wwid_LIST = diskInfo_DICT.keys()
#	sorted_wwid_LIST.sort()
#	for wwid in sorted_wwid_LIST:
#		diskInfo = diskInfo_DICT[ wwid ]
#		block_data_XML_single_block_device( vi_errorAndLog_obj, indentXMLlvl, wwid, diskInfo )

	if os.path.exists( '/sys/class/scsi_host' ):
		scsi_host_LIST = os.listdir( '/sys/class/scsi_host' )
		scsi_host_num_LIST = []
		for scsi_host in scsi_host_LIST:
			scsi_host_num_LIST.append( int( scsi_host.replace( 'host', '' ) ) )
		for scsi_host_num in sorted(scsi_host_num_LIST):
			scsi_host_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, scsi_host_num )

	if os.path.exists( '/sys/class/fc_host' ):
		fc_host_LIST = os.listdir( '/sys/class/fc_host' )
		fc_host_num_LIST = []
		for fc_host in fc_host_LIST:
			fc_host_num_LIST.append( int( fc_host.replace( 'host', '' ) ) )
		for fc_host_num in sorted(fc_host_num_LIST):
			fc_host_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, fc_host_num )

	if os.path.exists( '/sys/class/fc_transport' ):
		fc_target_LIST = os.listdir( '/sys/class/fc_transport' )
		fc_target_DICT_LIST = []
		for fc_target in fc_target_LIST:
			fc_target_wild = fc_target.replace( 'target', '' ) + ':*'
		        fc_target_dir = glob.glob( "/sys/class/fc_transport/%s/device/%s*" % (fc_target, fc_target_wild))
			if not fc_target_dir:
				continue
                        fc_target_num = fc_target_dir[0].replace("/sys/class/fc_transport/%s/device/" % fc_target, "")
                        
			#fc_target_num = fc_target.replace( 'target', '' ) + ':0'

			wwpn, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, "/sys/class/fc_transport/%s/port_name" % ( fc_target ) )
			if read_failed:
				continue
			wwpn = re.sub( '^0x', '', wwpn )
			fc_target_DICT_LIST.append( { 'fc_target' : fc_target, 'fc_target_num' : fc_target_num, 'wwpn' : wwpn } )
		fc_target_DICT_LIST.sort( fc_target_DICT__compare )
		for fc_target_DICT in fc_target_DICT_LIST:
			fc_target_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, fc_target_DICT[ 'fc_target' ], fc_target_DICT[ 'fc_target_num' ], fc_target_DICT[ 'wwpn' ]  )


        initiator_lines_LIST, read_failed = vi_utils.vi_read_text_lines( vi_errorAndLog_obj, '/etc/iscsi/initiatorname.iscsi' )
	if read_failed:
		initiator_lines_LIST[0] = ''   
        initiator_name = initiator_lines_LIST[0].replace('InitiatorName=','')
        iscsi_node_info_XML( vi_errorAndLog_obj, indentXMLlvl, initiator_name )
    


	if os.path.exists( '/sys/class/iscsi_host' ):
            	initiator_lines_LIST, read_failed = vi_utils.vi_read_text_lines( vi_errorAndLog_obj, '/etc/iscsi/initiatorname.iscsi' )
	    	if read_failed:
		    	initiator_lines_LIST[0] = ''   
			
                initiator_name = initiator_lines_LIST[0].replace('InitiatorName=','')
		iscsi_host_LIST = os.listdir( '/sys/class/iscsi_host' )
		iscsi_host_num_LIST = []
		for iscsi_host in iscsi_host_LIST:
			iscsi_host_num_LIST.append( int( iscsi_host.replace( 'host', '' ) ) )
		for iscsi_host_num in sorted(iscsi_host_num_LIST):
			iscsi_host_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, iscsi_host_num, initiator_name )

	if os.path.exists( '/sys/class/iscsi_session' ):


                cmd_LIST	= [ 'iscsiadm', '-m', 'session' ]
		stdout_LIST, stderr_LIST, exitStatus, deltaTime	= storage_tech.iscsiadm_exe_sync( vi_errorAndLog_obj, cmd_LIST )
                
		adm_session_DICT = {}
                for i,entry in enumerate(stdout_LIST):
                        session = stdout_LIST[i].split()[ 1 ]
                        session = session.strip(']')
                        session = session.strip('[')
                        ip = stdout_LIST[i].split()[ 2 ]
                        ip = ip.split(',')
                        adm_session_DICT[session] =  ip[0]

		iscsi_session_LIST = os.listdir( '/sys/class/iscsi_session' )
		iscsi_session_DICT_LIST = []
		for iscsi_session in iscsi_session_LIST:
			iscsi_session_num = iscsi_session.replace( 'session', '' )
			iscsi_targetname, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, "/sys/class/iscsi_session/%s/targetname" % ( iscsi_session ) )
			if read_failed:
				continue
			iscsi_session_DICT_LIST.append( { 'iscsi_session' : iscsi_session, 'iscsi_session_num' : int(iscsi_session_num), 'iscsi_targetname' : iscsi_targetname, 'target_ip': adm_session_DICT.get(iscsi_session_num) } )
		iscsi_session_DICT_LIST.sort( iscsi_session_DICT__compare )
		for iscsi_session_DICT in iscsi_session_DICT_LIST:
			iscsi_target_info_XML_single_host( vi_errorAndLog_obj, indentXMLlvl, iscsi_session_DICT[ 'iscsi_session' ], iscsi_session_DICT[ 'iscsi_session_num' ], iscsi_session_DICT[ 'iscsi_targetname' ], iscsi_session_DICT[ 'target_ip' ])

	# <SAS_Host> is the same as <SCSI_Host>
	# <SAS_Target>
	if os.path.exists( '/sys/class/sas_device' ):
		sas_target_DICT_LIST = []
		sas_device_LIST = os.listdir('/sys/class/sas_device' )
		for sas_device in sas_device_LIST:
			scsi_is_sas_expander_match = re.match("^end_device-(\d+):(\d+):(\d+)$", sas_device)
			if scsi_is_sas_expander_match:
				sas_address, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, "/sys/class/sas_device/%s/sas_address" % ( sas_device ) )
				sas_address = re.sub( '^0x', '', sas_address )
				if read_failed:
					continue
				sas_target_LIST = os.listdir( '/sys/class/sas_device/%s/device/' % (sas_device))
				for sas_target in sas_target_LIST:
					if 'target' in sas_target:
						sas_target_num = sas_target.replace('target','') + ':0'
						sas_target_DICT_LIST.append( {'sas_device' :  sas_device, 'sas_target_num' : sas_target_num, 'sas_address' : sas_address } )

		sas_target_DICT_LIST.sort (sas_target_DICT__compare)
		for sas_target_DICT in sas_target_DICT_LIST:
			sas_target_info_XML_single_host (vi_errorAndLog_obj, indentXMLlvl, sas_target_DICT['sas_device'], sas_target_DICT['sas_target_num'], sas_target_DICT['sas_address'])

	if os.path.exists( '/sys/bus/scsi/drivers/sr' ):
		scsiCDrom_cmpldRegEx = re.compile( '^\d+:\d+:\d+:\d+$' )
		try:
			sorted_srCDromPossible_LIST = os.listdir( '/sys/bus/scsi/drivers/sr' )
			sorted_srCDromPossible_LIST.sort()
		except:
			sorted_srCDromPossible_LIST = []
		for scsiCDromPossible in sorted_srCDromPossible_LIST:
			if not scsiCDrom_cmpldRegEx.match( scsiCDromPossible ):
				continue

			srCDrom_path = "/sys/bus/scsi/drivers/sr/%s" % ( scsiCDromPossible )

			typeValue, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, "%s/type" % ( srCDrom_path ) )
			if read_failed:
				continue

			if int(typeValue) != 5:
				continue

			sr_scsi_cdrom_info_XML_single_drive( vi_errorAndLog_obj, indentXMLlvl, scsiCDromPossible, srCDrom_path )

	indentXMLlvl = indentXMLlvl - 2
	vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'SCSI' )

	vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'IDE', None )
	indentXMLlvl = indentXMLlvl + 2

	if os.path.exists( '/sys/bus/ide/drivers/ide-cdrom' ):
		ideCDrom_cmpldRegEx		= re.compile( '^\d+.\d+$' )
		try:
			sorted_ideCDromPossible_LIST = os.listdir( '/sys/bus/ide/drivers/ide-cdrom' )
			sorted_ideCDromPossible_LIST.sort()
		except:
			sorted_ideCDromPossible_LIST = []
		ideCDrom_num_LIST		= []
		for ideCDromPossible in sorted_ideCDromPossible_LIST:
			if ideCDrom_cmpldRegEx.match( ideCDromPossible ):
				ideCDrom_num_LIST.append( float( ideCDromPossible ) )
		for ideCDrom_num in sorted(ideCDrom_num_LIST):
			ide_cdrom_info_XML_single_drive( vi_errorAndLog_obj, indentXMLlvl, ideCDrom_num )

	indentXMLlvl = indentXMLlvl - 2
	vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'IDE' )

	output_disk_scan_issues_section_XML( vi_errorAndLog_obj, indentXMLlvl )

####################################################################################################
# END
