#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for generating storage XML
"""

import vi_utils
import vi_xml
import storage_tech

import os
import re

from sd_storageXML			import *
from sd_diskAccessControl	import *

####################################################################################################

__all__ = [	'output_scan_vdisk_data_XML' ]

####################################################################################################
def scan_single_lv_for_vdisks_XML( vi_errorAndLog_obj, indentXMLlvl, vg_info, lv_info, mount_dev, mount_pt, isTransientMount, mountStatusMsg, mountStatusStatus ):
	"""
	This routine

	NOTE: WARNING: IMPORTANT:
		This function is NOT allowed to exit, but MUST return so that any
		unmounting that is necessary can occur.
	"""
#
	vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'Mount_Set', "transient=\"%s\"" % ( str(isTransientMount).lower() ) )
	indentXMLlvl = indentXMLlvl + 2

	vi_xml.vi_print_XML_data( indentXMLlvl, 'VolumeGroup', lv_info.mVGname )
	vi_xml.vi_print_XML_data( indentXMLlvl, 'LogicalVolume', lv_info.mName )
	vi_xml.vi_print_XML_data( indentXMLlvl, 'PartitionID', 0 )

	if mountStatusStatus == 0:

		df_data_cmpldRegEx = re.compile( "^%s\s+(\d+)\s+(\d+)\s+(\S+)$" % ( mount_pt ) )
		df_cmd = [ 'df', '-k', mount_pt ]
		stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, df_cmd )
		for df_line in stdout_LIST:
			df_data_match = df_data_cmpldRegEx.match( df_line )
			if not df_data_match:
				continue
			vi_xml.vi_print_XML_data_with_specs( indentXMLlvl, 'MountedFSsize', 'units=\"KiB\"', df_data_match.group(1) )
			vi_xml.vi_print_XML_data_with_specs( indentXMLlvl, 'MountedFreeSize', 'units=\"KiB\"', df_data_match.group(2) )
			break
		del( stdout_LIST )
		del( stderr_LIST )

		vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'MountStatus', 'status=\"success\" exitCode=\"0\"' )
		vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'MountStatus' )
	else:
		vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'MountStatus', "status=\"failed\" exitCode=\"%s\"" % ( mountStatusStatus ), True )
		vi_xml.vi_print_XML( 0, mountStatusMsg )	# Data is in RAW XML tags - no need to sanitize
		vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'MountStatus', True )

	indentXMLlvl = indentXMLlvl - 2
	vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'Mount_Set' )

	if not os.path.exists( mount_pt ):
		return

	vhd_file_extension	=  '.vhd'
	vhd_file_cmpldRegEx	= re.compile( "^\S+.%s$" % ( vhd_file_extension ) )

	for vhd_filename in os.listdir( mount_pt ):

		if not vhd_file_cmpldRegEx.match( vhd_filename ):
			continue
		virtualDiskName	= vhd_filename[ : -len( vhd_file_extension ) ]

		vhd_file_path	= "%s/%s" % ( mount_pt, vhd_filename )
		vhdFileInfo		= storage_tech.extract_vhd_info( vi_errorAndLog_obj, vhd_file_path )
		if not vhdFileInfo:
			continue

		vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'Vdisk_Set', None )
		indentXMLlvl = indentXMLlvl + 2

		vi_xml.vi_print_XML_data( indentXMLlvl, 'VirtualDisk_Format', 'vhd' )
		vi_xml.vi_print_XML_data( indentXMLlvl, 'VolumeGroup', lv_info.mVGname )
		vi_xml.vi_print_XML_data( indentXMLlvl, 'LogicalVolume', lv_info.mName )
		vi_xml.vi_print_XML_data( indentXMLlvl, 'PartitionID', 0 )
		vi_xml.vi_print_XML_data( indentXMLlvl, 'VirtualDisk', virtualDiskName )
		vi_xml.vi_print_XML_data( indentXMLlvl, 'MountPath', vhd_file_path )	# really WhenMountedPath
		vi_xml.vi_print_XML_data_with_specs( indentXMLlvl, 'FileSize', 'units=\"bytes\"', vhdFileInfo.mFileSizeInBytes )
		vi_xml.vi_print_XML_data_with_specs( indentXMLlvl, 'MaxFileSize', 'units=\"bytes\"', vhdFileInfo.mMaxDataSizeInBytes )
		vi_xml.vi_print_XML_data_with_specs( indentXMLlvl, 'OriginalSize', 'units=\"bytes\"', vhdFileInfo.mFtrLayout.Get( 'original_size' ) )
		vi_xml.vi_print_XML_data_with_specs( indentXMLlvl, 'CurrentSize', 'units=\"bytes\"', vhdFileInfo.mFtrLayout.Get( 'current_size' ) )
		vi_xml.vi_print_XML_data( indentXMLlvl, 'Type', "0x%08X" % ( vhdFileInfo.mFtrDiskType ) )

		indentXMLlvl = indentXMLlvl - 2
		vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'Vdisk_Set' )

		del vhdFileInfo

####################################################################################################
def mount_scan_single_lv_for_vdisks_XML( vi_errorAndLog_obj, indentXMLlvl, vg_info, lv_info, mount_dev, mount_pt ):
	"""
	This routine
	"""
#
	desired_mount_line_cmpldRegEx = re.compile( "^%s\s+on\s+%s\s+type\s+ext3\s+" % ( mount_dev, mount_pt ) )
	mount_cmd                     = [ 'mount', '-r', '-t', 'ext3', mount_dev, mount_pt ]
	umount_cmd                    = [ 'umount', mount_pt ]
	created_mount_pt_dir          = False
	performing_mount              = False
	try:
		if not os.path.exists( mount_pt ):
			created_mount_pt_dir = True
			os.mkdir( mount_pt )

		try:
			stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, 'mount' )

			performing_mount = True
			for mount_line in stdout_LIST:
				if not desired_mount_line_cmpldRegEx.match( mount_line ):
					continue
				performing_mount = False
				break

			del( stdout_LIST )
			del( stderr_LIST )

			if performing_mount:
				stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, mount_cmd )
				mountStatusMsg = "%s\n%s" % ( ''.join( stdout_LIST ), ''.join( stderr_LIST ) )
				del( stdout_LIST )
				del( stderr_LIST )
			else:
				mountStatusMsg = ''
				exitStatus = 0

			scan_single_lv_for_vdisks_XML( vi_errorAndLog_obj, indentXMLlvl, vg_info, lv_info, mount_dev, mount_pt, performing_mount, mountStatusMsg, exitStatus )
		finally:
			if performing_mount:
				stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, umount_cmd )
				del( stdout_LIST )
				del( stderr_LIST )

	finally:
		if created_mount_pt_dir:
			try:
				os.rmdir( mount_pt )
			except:
				pass

####################################################################################################
def output_scan_vdisk_data_XML( vi_errorAndLog_obj, indentXMLlvl, doVDiskMntScan, item_LIST, diskInfo_DICT, vgInfo_uuid_DICT ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 1, "scan_vdisk_data_XML" )

	# Convert given list into better searching DICT
	vglv_to_scan_DICT = {}
	for vg_lv_pair in item_LIST:
		vg_name, lv_name = vg_lv_pair.split('_')
		if not storage_tech.is_vdisk_lv_name( lv_name ):
#			vi_errorAndLog_obj.vi_print_wrn( "Given lv_name %s is not valid for containing vdisks" % ( lv_name ) )
			continue

		lv_to_scan_DICT = vglv_to_scan_DICT.get( vg_name, None )
		if lv_to_scan_DICT:
			lv_to_scan_DICT[ lv_name ] = True
		else:
			vglv_to_scan_DICT[ vg_name ] = { lv_name : True }

	# Whittle down to only given item list
	vdisk_vgInfo_uuid_DICT = {}
	for vg_uuid in vgInfo_uuid_DICT:
		vg_info = vgInfo_uuid_DICT[ vg_uuid ]
		vg_name = vg_info.mName
		if not vglv_to_scan_DICT.has_key( vg_name ):
			continue

		lv_to_scan_DICT = vglv_to_scan_DICT[ vg_name ]
		vdisk_LVinfo_DICT = {}
		for lv_name in vg_info.mLVinfo_name_DICT:
			if not lv_to_scan_DICT.has_key( lv_name ):
				continue
			vdisk_LVinfo_DICT[ lv_name ] = vg_info.mLVinfo_name_DICT[ lv_name ]
		if len(vdisk_LVinfo_DICT) < 1:
			continue

		vg_info.mLVinfo_name_DICT = vdisk_LVinfo_DICT
		vdisk_vgInfo_uuid_DICT[ vg_uuid ] = vg_info

	if len(vdisk_vgInfo_uuid_DICT) < 1:
		return

	# Break up into managable pieces
	wwid_LIST = []
	wwid_LIST_LIST = []
	vgInfo_LIST = []
	vgInfo_LIST_LIST = []
	for vg_uuid in vdisk_vgInfo_uuid_DICT:
		vg_info = vdisk_vgInfo_uuid_DICT[ vg_uuid ]
		pv_info_DICT = vg_info.mPVinfo_wwid_DICT
		if ( len(wwid_LIST) + len(pv_info_DICT) ) > global_numOfDisksSetupAtOnce or len(vgInfo_LIST) > 9:
			if len(wwid_LIST) > 0:
				wwid_LIST.sort()
				wwid_LIST_LIST.append( wwid_LIST )
				wwid_LIST = []
				vgInfo_LIST_LIST.append( vgInfo_LIST )
				vgInfo_LIST = []
		for pv_wwid in pv_info_DICT:
			wwid_LIST.append( pv_wwid )
		vgInfo_LIST.append( vg_info )
	if len(wwid_LIST) > 0:
		wwid_LIST.sort()
		wwid_LIST_LIST.append( wwid_LIST )
		vgInfo_LIST_LIST.append( vgInfo_LIST )

	vi_xml.vi_print_beg_XML_tag( indentXMLlvl, 'MountAndVdiskScan', None )
	indentXMLlvl = indentXMLlvl + 2

	# Do the work on the managable pieces
	for idx in range(len(wwid_LIST_LIST)):
		wwid_LIST = wwid_LIST_LIST[ idx ]
		vgInfo_LIST = vgInfo_LIST_LIST[ idx ]

		for wwid in wwid_LIST:
			setup_disk_for_access( vi_errorAndLog_obj, wwid, diskInfo_DICT )

		for vg_info in vgInfo_LIST:
			for lv_name in vg_info.mLVinfo_name_DICT:
				lv_info = vg_info.mLVinfo_name_DICT[ lv_name ]
				mount_dev = "/dev/%s/%s" % ( lv_info.mVGname, lv_info.mName )
				mount_pt = "/mnt/lvp_%s_%s_p0" % ( lv_info.mVGname, lv_info.mName )
				if doVDiskMntScan:
					mount_scan_single_lv_for_vdisks_XML( vi_errorAndLog_obj, indentXMLlvl, vg_info, lv_info, mount_dev, mount_pt )
				else:
					scan_single_lv_for_vdisks_XML( vi_errorAndLog_obj, indentXMLlvl, vg_info, lv_info, mount_dev, mount_pt, False, '', 0 )

		for wwid in wwid_LIST:
			done_accessing_disk( vi_errorAndLog_obj, wwid, diskInfo_DICT )

	indentXMLlvl = indentXMLlvl - 2
	vi_xml.vi_print_end_XML_tag( indentXMLlvl, 'MountAndVdiskScan' )

	output_disk_scan_issues_section_XML( vi_errorAndLog_obj, indentXMLlvl )

####################################################################################################
# END
