#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Script for managing storage_stats operations
"""

import sys
import os
import re
import traceback

sys.path.append('..')

import vi_utils
import vi_xml
import storage_tech
#import storage_discover

from optparse import OptionParser
from optparse import OptionGroup

####################################################################################################

global_errAndLog	= None		# VI_ErrAndLog_OBJ

####################################################################################################

class Task_OBJ:

	def __init__(self, theArgNameStr, theHelpStr, theFcn):

		self.mArgNameStr	= theArgNameStr
		self.mHelpStr		= theHelpStr
		self.mDoScan		= False
		self.mValue			= None
		self.mFcn			= theFcn

	def __repr__(self):

		return "mArgNameStr: %s mHelpStr: %s mDoScan: %s mValue: %s mFcn: %s" % (
						self.mArgNameStr
					,	self.mHelpStr
					,	self.mDoScan
					,	self.mValue
					,	self.mFcn )

class xenstore_devInfo_OBJ:

	def __init__( self, xenstore_devID ):

		self.mDevID		= xenstore_devID
		self.mDevNum	= None
		self.mDevNumHex	= None
		self.mDevPath	= None
		self.mDevName	= None
		self.mBlkDev	= None

	def __repr__(self):

		return "mDevID: %s mDevNum: %s mDevNumHex: %s mDevPath: %s mDevName: %s mBlkDev: %s" % (
						self.mDevID
					,	self.mDevNum
					,	self.mDevNumHex
					,	self.mDevPath
					,	self.mDevName
					,	self.mBlkDev )

####################################################################################################
def fc_host_statistics_XML( vi_errorAndLog_obj, domDocHelper, fc_host_num ):
	"""
	This routine
	"""
#
	fc_host_statistics_path	= "/sys/class/fc_host/host%d/statistics" % ( fc_host_num )

	if not os.path.exists( fc_host_statistics_path ):
		return

	domDocHelper.push_level( 'Host', { 'Num' : fc_host_num } )
	domDocHelper.push_level( 'Statistics' )

	exclude_LIST = []

	sorted_statisticsItems_LIST = os.listdir( fc_host_statistics_path )
	sorted_statisticsItems_LIST.sort()
	for sys_item in sorted_statisticsItems_LIST:
		vi_xml.vi_sys_item_XML( vi_errorAndLog_obj, domDocHelper, fc_host_statistics_path, sys_item, exclude_LIST )

	domDocHelper.pop_level()
	domDocHelper.pop_level()

####################################################################################################
def task_host_statistics( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
	indentXMLlvl	= options_OBJ.indentXMLlvl

	if not os.path.exists( '/sys/class/fc_host' ):
		return

	domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )
	domDocHelper.push_level( 'SCSI' )

	if os.path.exists( '/sys/class/fc_host' ):
		fc_host_LIST = os.listdir( '/sys/class/fc_host' )
		fc_host_num_LIST = []
		for fc_host in fc_host_LIST:
			fc_host_num_LIST.append( int( fc_host.replace( 'host', '' ) ) )
		for fc_host_num in sorted(fc_host_num_LIST):
			fc_host_statistics_XML( vi_errorAndLog_obj, domDocHelper, fc_host_num )

	vi_errorAndLog_obj.vi_print_std( '\n'.join( domDocHelper.finalize() ) )

####################################################################################################
def dump_domain_stats( vi_errorAndLog_obj, xs_domain_root, isAccelerated, ignore_xenstore_dev_DICT, indentXMLlvl ):
	"""
	This routine
	"""
#
	phydev_cmpldRegEx	= re.compile( '^%s/(\d+)/physical-device = "([0-9a-fA-F]+):([0-9a-fA-F]+)"$' % ( xs_domain_root ) )
	params_cmpldRegEx	= re.compile( '^%s/(\d+)/params = "(/dev.*/)([^/\s]+)"$' % ( xs_domain_root ) )
	stat_tag_LIST		=	[	'Read_Ios'
							,	'Read_Merges'
							,	'Read_Sectors'
							,	'Read_Ticks'
							,	'Write_Ios'
							,	'Write_Merges'
							,	'Write_Sectors'
							,	'Write_Ticks'
							,	'In_Flight'
							,	'Io_Ticks'
							,	'Time_In_Queue'
							]

	# xenstore-ls -f provides full path AND data without truncation
	stdout_LIST, stderr_LIST, exitStatus, deltaTime = vi_utils.vi_exe_sync( vi_errorAndLog_obj, [ 'xenstore-ls', '-f', xs_domain_root ] )
	del stderr_LIST

	xenstore_dev_DICT		= {}
	for xs_line in stdout_LIST:

		phydev_match	= phydev_cmpldRegEx.match( xs_line.strip() )
		if phydev_match:

			# Convert from hex into decimal ( since this is what the kernel uses DUH ! )
			phydev_major		= int( phydev_match.group(2), 16 )
			phydev_minor		= int( phydev_match.group(3), 16 )
			phydev_devNum		= "%d:%d" % ( phydev_major, phydev_minor )
			phydev_devNumHex	= "%s:%s" % ( phydev_match.group(2), phydev_match.group(3) )

			xenstore_devID						= int( phydev_match.group(1) )
			if ignore_xenstore_dev_DICT.has_key( xenstore_devID ):
				continue

			xenstore_devInfo					= xenstore_dev_DICT.get( xenstore_devID, xenstore_devInfo_OBJ( xenstore_devID ) )
			xenstore_devInfo.mDevNum			= phydev_devNum
			xenstore_devInfo.mDevNumHex			= phydev_devNumHex
			xenstore_dev_DICT[ xenstore_devID ]	= xenstore_devInfo

			del phydev_match
			continue

		params_match	= params_cmpldRegEx.match( xs_line.strip() )
		if params_match:

			devPath			= params_match.group(2)
			devName			= params_match.group(3)

			xenstore_devID						= int( params_match.group(1) )
			if ignore_xenstore_dev_DICT.has_key( xenstore_devID ):
				continue
	
			xenstore_devInfo					= xenstore_dev_DICT.get( xenstore_devID, xenstore_devInfo_OBJ( xenstore_devID ) )
			xenstore_devInfo.mDevPath			= devPath
			xenstore_devInfo.mDevName			= devName
			xenstore_dev_DICT[ xenstore_devID ]	= xenstore_devInfo

			del params_match
			continue

	del stdout_LIST

	devNums_to_blkDev_DICT	= storage_tech.get_devNums_to_blkDev_mapping( vi_errorAndLog_obj, False )

	xenstore_dev_LIST		= []
	for xenstore_devID, xenstore_devInfo in xenstore_dev_DICT.iteritems():

		if not xenstore_devInfo.mDevNum or not xenstore_devInfo.mDevPath or not xenstore_devInfo.mDevName:
#			vi_errorAndLog_obj.vi_print_err( "Only partial xenstore_devInfo %s" % ( xenstore_devInfo ) )
			continue

		xenstore_devInfo.mBlkDev	= devNums_to_blkDev_DICT.get( xenstore_devInfo.mDevNum, None )
		if not xenstore_devInfo.mBlkDev:
#			vi_errorAndLog_obj.vi_print_err( "No block device for phydev = oHex: '%s' cDec: '%s' :: %s" % ( xenstore_devInfo.mDevNumHex, xenstore_devInfo.mDevNum, xenstore_devInfo ) )
			continue

		if True and xenstore_devInfo.mDevPath != '/dev/mapper/':
#			vi_errorAndLog_obj.vi_print_dbg( 0, "mDevPath is not /dev/mapper/ : %s" % ( xenstore_devInfo ) )
			continue

		xenstore_dev_LIST.append( xenstore_devInfo )

	devNums_to_blkDev_DICT	= None
	xenstore_dev_DICT		= {}

	for xenstore_devInfo in xenstore_dev_LIST:

		stat_path					= "/sys/block/%s/stat" % ( xenstore_devInfo.mBlkDev )
		stat_data_line, read_failed	= vi_utils.vi_read_text_value( vi_errorAndLog_obj, stat_path )
		if read_failed:
#			vi_errorAndLog_obj.vi_print_err( "blk dev stat '%s' could not be read : %s" % ( stat_path, xenstore_devInfo ) )
			continue

		xenstore_dev_DICT[ xenstore_devInfo.mDevID ]	= xenstore_devInfo

		stat_data_LIST	= stat_data_line.split()

		domDocHelper	= vi_xml.VI_DOMdocHelper_OBJ( indentXMLlvl, 2 )
		domDocHelper.push_level( 'Block_Device', { 'Id' : xenstore_devInfo.mDevName } )

		if isAccelerated:
			domDocHelper.add_data_elem( 'Accelerated', '1' )
		else:
			domDocHelper.add_data_elem( 'Accelerated', '0' )

		domDocHelper.push_level( 'Statistics' )

		for idx in range( 0, len(stat_tag_LIST) ):
			domDocHelper.add_data_elem( stat_tag_LIST[idx], stat_data_LIST[idx] )

		vi_errorAndLog_obj.vi_print_std( '\n'.join( domDocHelper.finalize() ) )

		del domDocHelper

	return xenstore_dev_DICT

####################################################################################################
def task_domain_stats( vi_errorAndLog_obj, options_OBJ, theValue ):
	"""
	This routine
	"""
#
	xs_domain_backend_root		= "/local/domain/0/backend/vbd/%s"  % ( theValue )
	xs_domain_emulated_root		= "/local/domain/0/emulated/vbd/%s" % ( theValue )

	# Start with the accelerated
	ignore_xenstore_dev_DICT	= dump_domain_stats( vi_errorAndLog_obj, xs_domain_backend_root, True, {}, options_OBJ.indentXMLlvl )
	# If an accelerated was found, then ignore it now
	dump_domain_stats( vi_errorAndLog_obj, xs_domain_emulated_root, False, ignore_xenstore_dev_DICT, options_OBJ.indentXMLlvl )

####################################################################################################
def parse_given_args():
	"""
	This routine
	"""
#
	task_no_arg_LIST	=	[	Task_OBJ(	'host_statistics'
										,	'Iterate through SCSI host and dump statistics XML'
										,	task_host_statistics
										)
							]

	task_with_arg_LIST	=	[	Task_OBJ(	'domain_stats'
										,	'For the given domain dump block device statistics XML'
										,	task_domain_stats
										)
							]

	parser = OptionParser()

	parser.add_option("", "--debugLvl", dest="debugLvl", default=0, type="int",
					help="debugLvl message printing to stderr" )

	parser.add_option("", "--indentXMLlvl", dest="indentXMLlvl", default=0, type="int",
					help="indent XML level" )

	def store_value____task_no_arg_do( option, opt_str, value, parser ):
		taskIdx 		= int(option.dest)
		task			= task_no_arg_LIST[ taskIdx ]
		task.mDoScan	= True
		task.mValue		= value

	def store_value____task_with_arg_do( option, opt_str, value, parser ):
		taskIdx 		= int(option.dest)
		task			= task_with_arg_LIST[ taskIdx ]
		task.mDoScan	= True
		task.mValue		= value

	taskIdx = -1
	for task in task_no_arg_LIST:
		taskIdx = taskIdx + 1

		parser.add_option(	"", "--do_%s" % ( task.mArgNameStr ), dest=str(taskIdx), action="callback",
							callback=store_value____task_no_arg_do,  help="Do Task :: %s" % ( task.mHelpStr ) )

	taskIdx = -1
	for task in task_with_arg_LIST:
		taskIdx = taskIdx + 1

		parser.add_option(	"", "--do_%s" % ( task.mArgNameStr ), dest=str(taskIdx), action="callback", type="str",
							callback=store_value____task_with_arg_do,  help="Do Task :: %s" % ( task.mHelpStr ) )

	options_OBJ, args_LIST = parser.parse_args()

	tasksToPerform_LIST = []
	for task in task_no_arg_LIST:
		if task.mDoScan:
			tasksToPerform_LIST.append( task )
	for task in task_with_arg_LIST:
		if task.mDoScan:
			tasksToPerform_LIST.append( task )

	if 0:
		print >> sys.stderr, options_OBJ
		print >> sys.stderr, args_LIST
		print >> sys.stderr, dir(options_OBJ)
		print >> sys.stderr, parser.values
		print >> sys.stderr, tasksToPerform_LIST
		vi_utils.vi_dbg_stop()

	if len( tasksToPerform_LIST ) == 0:
		print >> sys.stderr, 'no work requested\n'
		parser.print_help(sys.stderr)
		print >> sys.stderr, ''
		vi_utils.vi_exit( 2 )

	return ( options_OBJ, args_LIST, tasksToPerform_LIST )

####################################################################################################
def init_complex_globals( options_OBJ, args_LIST ):
	"""
	This routine initializes and complex global variables
	"""
#
	global global_errAndLog

	errAndLog_dst_DICT =	{	'stdout'			:	(	vi_utils.VI_ErrAndLog__FOR_FLAG__ALL
														,	sys.stdout
														)
							}

	global_errAndLog = vi_utils.VI_ErrAndLogRedirector_OBJ( errAndLog_dst_DICT, options_OBJ.debugLvl )

####################################################################################################
def main():
	"""
	This routine is the script entry point
	"""
#
	options_OBJ, args_LIST, tasksToPerform_LIST = parse_given_args()

	init_complex_globals( options_OBJ, args_LIST )

	for task in tasksToPerform_LIST:
		try:
			task.mFcn( global_errAndLog, options_OBJ, task.mValue )
		except:
			global_errAndLog.vi_croak_exception( traceback.format_exc() )

	return 0

####################################################################################################
### root of all evil

sys.stdout = os.fdopen( sys.stdout.fileno(), sys.stdout.mode, 0 )	# UNbuffer stdout

exitValue  = main()

sys.exit( exitValue )

# END
####################################################################################################
