#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for miscellaneous purposes
"""

import os
import sys

####################################################################################################

__all__ = [	'vi_get_code_location'
		,	'vi_raise_not_implemented'
		,	'vi_exit'
		,	'vi_dbg_stop'
		,	'vi_sort_uniq' ]

####################################################################################################
def vi_get_code_location():
	"""
	This routine returns ( lineNumber, routine_method_name )
	"""
#
	calling_frame	= sys._getframe( 1 )
	return calling_frame.f_lineno, calling_frame.f_code.co_name

####################################################################################################
def vi_raise_not_implemented( extraMsg=None ):
	"""
	This routine returns ( lineNumber, routine_method_name )
	"""
#
	calling_frame	= sys._getframe( 1 )

	if not extraMsg:
		raise NotImplementedError, "%s @ %d" % (	calling_frame.f_code.co_name
												,	calling_frame.f_lineno )
	else:
		raise NotImplementedError, "%s @ %d :: %s" % (	calling_frame.f_code.co_name
													,	calling_frame.f_lineno
													,	str(extraMsg) )

####################################################################################################
def vi_exit( exitStatus ):
	"""
	This routine exits the process immediately
	"""
#
	os._exit( exitStatus )		# sys.exit() just throws :: really exit instead 

####################################################################################################
def vi_dbg_stop( *itemsToPrint_TPL ):
	"""
	This routine
	"""
#
	calling_frame	= sys._getframe( 1 )
	caller_name		= calling_frame.f_code.co_name
	caller_lineno	= calling_frame.f_lineno

	outputBuffer	= "\n\nStopping @ %s() line %s :" % ( caller_name, caller_lineno )
	for itemToPrint in itemsToPrint_TPL:
		outputBuffer = outputBuffer + ' ' + itemToPrint

	sys.stdout.flush()
	sys.stderr.flush()

	print >> sys.stderr, outputBuffer

	os._exit( 127 )		# sys.exit() just throws :: really exit instead 

####################################################################################################
def vi_sort_uniq( given_LIST, stripLenZero ):
	"""
	This routine
	"""
#
	if len(given_LIST) < 2:
		if stripLenZero and len(given_LIST) == 1 and len(given_LIST[0]) == 0:
			del( given_LIST[0] )
		return given_LIST

	given_LIST.sort()

	dupIdx_LIST		= []
	itr				= enumerate( given_LIST )
	idx, lastItem	= itr.next()
	if stripLenZero and len(lastItem) == 0:
		dupIdx_LIST.append( 0 )
	for idx, item in itr:
		if item != lastItem:
			lastItem	= item
			if stripLenZero and len(item) == 0:
				dupIdx_LIST.append( idx )
		else:
			dupIdx_LIST.append( idx )

	for idx in reversed( dupIdx_LIST ):
		del( given_LIST[idx] )

	return given_LIST

####################################################################################################
# END
