#!/usr/bin/python
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for accumulating and printing XML
"""

import vi_utils

import os
import re
import string

####################################################################################################

__all__ = [	'vi_print_XML'
		,	'vi_flush_XML'
		,	'vi_sanitize_for_XML_tag'
		,	'vi_sanitize_for_XML_tagSpecifiers'
		,	'vi_sanitize_for_XML_data'
		,	'vi_print_beg_XML_tag'
		,	'vi_print_end_XML_tag'
		,	'vi_print_XML_data'
		,	'vi_print_XML_data_with_specs'
		,	'vi_sys_item_XML' ]

####################################################################################################

global_XML_accumulation_LIST   = []

####################################################################################################
def vi_print_XML( indentXMLlvl, *itemsToPrint_TPL ):
	"""
	This routine
	"""
#
	global global_XML_accumulation_LIST

	if indentXMLlvl > 0:
		indentXMLlvl = indentXMLlvl - 1
		xmlBuffer = ' ' * indentXMLlvl
	else:
		xmlBuffer = ''

	for itemToPrint in itemsToPrint_TPL:
		if xmlBuffer:
			xmlBuffer = "%s %s" % ( xmlBuffer, itemToPrint )
		else:
			xmlBuffer = str(itemToPrint)

#	print xmlBuffer
#	return

	global_XML_accumulation_LIST.append( xmlBuffer )

####################################################################################################
def vi_flush_XML():
	"""
	This routine
	"""
#
	global global_XML_accumulation_LIST

	for xmlBuffer in global_XML_accumulation_LIST:
		print xmlBuffer

	global_XML_accumulation_LIST = []

####################################################################################################
def vi_sanitize_for_XML_tag( theXMLtag ):
	"""
	This routine
	"""
#
	if theXMLtag == None:
		return None

	sanitized_XMLtag = str( theXMLtag )
	sanitized_XMLtag = sanitized_XMLtag.replace( '<', '_' )
	sanitized_XMLtag = sanitized_XMLtag.replace( '>', '_' )

	return sanitized_XMLtag

####################################################################################################
def vi_sanitize_for_XML_tagSpecifiers( theXMLtagSpecifiers ):
	"""
	This routine
	"""
#
	if theXMLtagSpecifiers == None:
		return None

	sanitized_XMLtagSpecifiers = str( theXMLtagSpecifiers )
	sanitized_XMLtagSpecifiers = sanitized_XMLtagSpecifiers.replace( '<', '_' )
	sanitized_XMLtagSpecifiers = sanitized_XMLtagSpecifiers.replace( '>', '_' )

	return sanitized_XMLtagSpecifiers

####################################################################################################
def vi_sanitize_for_XML_data( theXMLdata ):
	"""
	This routine
	"""
#
	if theXMLdata == None:
		return None

	sanitized_XMLdata = str( theXMLdata )
	sanitized_XMLdata = sanitized_XMLdata.replace( '<', '_' )
	sanitized_XMLdata = sanitized_XMLdata.replace( '>', '_' )

	return sanitized_XMLdata

####################################################################################################
def vi_print_beg_XML_tag( indentXMLlvl, theXMLtag, theXMLdata, rawData=False ):
	"""
	This routine
	"""
#
	sanitized_XMLtag = vi_sanitize_for_XML_tag( theXMLtag )

	if not rawData:
		sanitized_XMLdata = vi_sanitize_for_XML_data( theXMLdata )
		if sanitized_XMLdata:
			vi_print_XML( indentXMLlvl, "<%s %s>" % ( sanitized_XMLtag, sanitized_XMLdata ) )
		else:
			vi_print_XML( indentXMLlvl, "<%s>" % ( sanitized_XMLtag ) )
	else:
		# no need to sanitize data as it is in RAW XML tags
		if theXMLdata:
			vi_print_XML( indentXMLlvl, "<%s %s><![CDATA[" % ( sanitized_XMLtag, theXMLdata ) )
		else:
			vi_print_XML( indentXMLlvl, "<%s><![CDATA[" % ( sanitized_XMLtag ) )

####################################################################################################
def vi_print_end_XML_tag( indentXMLlvl, theXMLtag, rawData=False ):
	"""
	This routine
	"""
#
	sanitized_XMLtag = vi_sanitize_for_XML_tag( theXMLtag )

	if not rawData:
		vi_print_XML( indentXMLlvl, "</%s>" % ( sanitized_XMLtag ) )
	else:
		vi_print_XML( indentXMLlvl, "]]></%s>" % ( sanitized_XMLtag ) )

####################################################################################################
def vi_print_XML_data( indentXMLlvl, theXMLtag, theXMLdata ):
	"""
	This routine
	"""
#
	sanitized_XMLtag  = vi_sanitize_for_XML_tag(  theXMLtag )
	sanitized_XMLdata = vi_sanitize_for_XML_data( theXMLdata )

	vi_print_XML( indentXMLlvl, "<%s>%s</%s>" % ( sanitized_XMLtag, sanitized_XMLdata, sanitized_XMLtag ) )

####################################################################################################
def vi_print_XML_data_with_specs( indentXMLlvl, theXMLtag, theXMLtagSpecifiers, theXMLdata ):
	"""
	This routine
	"""
#
	sanitized_XMLtag           = vi_sanitize_for_XML_tag(           theXMLtag )
	sanitized_XMLtagSpecifiers = vi_sanitize_for_XML_tagSpecifiers( theXMLtagSpecifiers )
	sanitized_XMLdata          = vi_sanitize_for_XML_data(          theXMLdata )

	vi_print_XML( indentXMLlvl, "<%s %s>%s</%s>" % ( sanitized_XMLtag, sanitized_XMLtagSpecifiers, sanitized_XMLdata, sanitized_XMLtag ) )

####################################################################################################
def vi_sys_item_XML( vi_errorAndLog_obj, domHelper, sys_host_path, sys_item, exclude_LIST ):
	"""
	This routine
	"""
#
	sys_item_path = "%s/%s" % ( sys_host_path, sys_item )
	if os.path.islink( sys_item_path ):
		vi_errorAndLog_obj.vi_print_dbg( 7, "vi_sys_item_XML( %s ) isLink" % ( sys_item_path ) )
		return

	for exclude_item in exclude_LIST:
		if re.match( "^%s" % ( exclude_item ), sys_item ):
			vi_errorAndLog_obj.vi_print_dbg( 7, "vi_sys_item_XML( %s ) excluded" % ( sys_item_path ) )
			return

	nums = string.digits;
        sys_item_tag = sys_item.title()
        sys_item_tag = sys_item_tag.replace( ' ', '_' )
        if sys_item_tag[0] in nums:
	        sys_item_tag = 'X' + sys_item_tag

	sys_item_lines, read_failed = vi_utils.vi_read_text_value( vi_errorAndLog_obj, sys_item_path )
	if read_failed:
		vi_errorAndLog_obj.vi_print_dbg( 7, "vi_sys_item_XML( %s ) read_failed" % ( sys_item_path ) )
		return
        for idx in range(len(sys_item_lines)):
                if sys_item_lines[idx] not in string.printable:
                        return

	domHelper.add_data_elem( sys_item_tag, sys_item_lines )

####################################################################################################
# END
